SHELL=/bin/sh
BASE=${PWD}

#=====================================================
MY_MODULE=-I/$(BASE)/src/class
PETSC_DIR=/workdir/rech/dyc/rdyc711/petsc-2.3.0
include $(PETSC_DIR)/bmake/common/variables
include $(PETSC_DIR)/bmake/petscconf


#==================================================================
#=== f90 optimization flags
#==================================================================
 FOPTFLAGS = -C -O3 -qarch=auto 

#==================================================================
#=== the following can be set as environment 
#=== variables (the preferred approach) or defined 
#=== explicitly here
#==================================================================

 F77        = /usr/bin/xlf -g     # how to compile fixed format .f files
 F90        = /usr/bin/mpxlf90_r -qsuffix=f=f90 -qsuffix=cpp=f90 -qnoextname # f90 compiler that is mpi aware

NETCDF_INC = $(NETCDF)
NETCDF_LIB = $(NETCDF)

FFTW_INC = $(FFTW3_I4S4D8)
FFTW_LIB = $(FFTW3_I4S4D8) -lfftw3 -lfftw3f

XTRA_LIBS = $(BLAS_I4S4D8) $(LAPACK_I4S4D8)

PETSC_LIBS = ${PACKAGES_LIBS} -L/$(PETSC_DIR)/lib/$(PETSC_ARCH) \
                                $(PETSC_FORTRAN_LIB_BASIC)      \
                                $(PETSC_LIB_BASIC)


#==================================================================
# combine  variables for (PETSC + EXTERNAL PACKAGES), NETCDF and FFTW3
#==================================================================

ALL_INCLUDES = $(MY_MODULE) $(PETSC_INCLUDE) $(NETCDF_INC) $(FFTW_INC) $(MPI_INC)
ALL_EXT_LIBS = $(PETSC_LIBS) $(NETCDF_LIB) $(FFTW_LIB) $(XTRA_LIBS)

showconfig:
	@echo
	-@echo  "host:          " `hostname -s`
	-@echo  "               " `uname -sr`
	-@echo  "user:          " `whoami`
	-@echo  "compile date:  " `date`
	-@echo  "PETSC_DIR:     " ${PETSC_DIR}
	-@echo  "PETSC_ARCH:    " ${PETSC_ARCH}
	-@echo  "OUTPUT_ROOT:   " ${OUTPUT_ROOT}
	-@echo  "F90:           " ${F90}
	-@echo  "F77:           " ${F77}
	-@echo  "FOPTFLAGS:     " ${FOPTFLAGS}
	-@echo  "FLINKER:       " ${FLINKER}
	-@echo  "NETCDF_INC:    " ${NETCDF_INC}
	-@echo  "NETCDF_LIB:    " ${NETCDF_LIB}
	-@echo  "FFTW_INC:      " ${FFTW_INC}
	-@echo  "FFTW_LIB:      " ${FFTW_LIB}
	-@echo  "LAPACK_LIB:    " ${LAPACK_LIB}
	-@echo  "BLAS_LIB:      " ${BLAS_LIB}
	-@echo  "PETSC_LIBS:    " ${PETSC_LIBS}
	-@echo  "XTRA_LIBS:     " ${XTRA_LIBS}
