#!/usr/bin/ksh
#######################################################
#####          LANCEMENT DU RUN                 ####### 
#######################################################
#_____________________________________________________
#
#       DEFINITION DES VARIABLES
#_____________________________________________________

MODE=$1
VERSION=$2
RUN=$3


if [ $# -ne 3 ]; then
  echo ' '
  echo " *** Mauvais arguments d'entree ! *** "
  echo '   exemple: ./lanc.ksh mode version numero_run'
  echo ' '
  exit 1
fi

echo ' '
echo ' '
case $1 in
  0)
   echo '========================'
   echo '==     MODE BATCH     =='
   echo '========================'
  ;;
  1)
   echo '========================'
   echo '==   MODE INTERACTIF  =='
   echo '========================'
  ;;
  *)
   echo '========================'
   echo '==     MODE TEST      =='
   echo '========================'
  ;;
esac
 echo ' '
 echo ' '

#_____________________________________________________
#
# PARAMETRES 
#_____________________________________________________

DATE=`date '+%Y%m%d'_'%H%M%S'`
echo `date`

HOME_DIR=$PWD
HOME_PROG=00FlowSolve_KW
SUB_PROJECT=sub_${HOME_PROG}
WORK_PROG=${RUN}_${HOST}_${HOME_PROG}_${VERSION}_${DATE}
WORK_DIR=$WORKDIR

if [ ! -d $WORK_DIR  ]; then
   WORK_DIR=$HOME_DIR
   echo ' '
   echo ' pas de WORK_DIR definit, execution sur:'
   echo '    --> '$WORK_DIR
   echo ' '
fi

echo ' '
echo 'Definitions des variables:'
echo '   HOME_DIR    = '$HOME_DIR
echo '   HOME_PROG   = '$HOME_PROG
echo '   HOME_PROJECT= '$VERSION
echo '   SUB_PROJECT = '$SUB_PROJECT
echo '   WORK_DIR    = '$WORK_DIR
echo '   WORK_PROG   = '$WORK_PROG


#_____________________________________________________
#
# CREATION DU REPERTOIRE DE COMPILATION SUR LE WORKDIR
#_____________________________________________________

echo ' '
echo 'Creation du repertoire de compilation et copie des fichiers'
if [ ! -d $HOME_DIR/setups/$VERSION ]; then
   echo ' '
   echo '   WARNING: le projet '$VERSION" n'existe pas"
   exit 1
fi

cd $WORK_DIR
if  [ -d $WORK_PROG ]; then 
  rm -rf $WORK_PROG
fi
mkdir $WORK_PROG 
cd $WORK_PROG
cp -rf $HOME_DIR/* .
\rm -rf input
ln -s setups/$VERSION input

#_____________________________________________________
#
#       COMPILATION DU PROG SUR LE WORKDIR
#_____________________________________________________

echo ' '
echo 'Compilation du code pour le projet: '$HOME_PROJECT
# Compilation
cd $WORK_DIR/$WORK_PROG
make clean
make libs
make flow.x
make outdirs
if [ -f flow.x ]; then
   echo '   Compilation successful ! Here we go'
else
   echo '   WARNING: Compilation failed !'
   cd $WORK_DIR
   rm -rf $WORK_PROG
   exit 1
fi

echo ' '
case $1 in
	0)
	#_____________________________________________________
	#
	#        OFFLINE NOHUP
	#_____________________________________________________
	echo ' Soumission du job... offline.'
	cp $HOME_DIR/$SUB_PROJECT $WORK_DIR/$WORK_PROG/.
	cd $WORK_DIR/$WORK_PROG
	llsubmit $SUB_PROJECT
	;;
	1)
        #_____________________________________________________
        #
        #        ONLINE 
        #_____________________________________________________
        echo ' Mode interactif: execution sur le workdir ...'
        cd $WORK_DIR/$WORK_PROG
        echo 'Combien de processeurs requis ?'
        read NPROCS
        export MP_PROCS=$NPROCS
        echo '   Execution sur '$MP_PROCS' procs.' 
        cd $WORK_DIR/$WORK_PROG
        ./flow.x 
        echo ' End : successful run !'
	;;
	*)
	#_____________________________________________________
	#
	#        TEST COMPILATION
	#_____________________________________________________
	echo ' End : successful test !'
	cd $WORK_DIR
	rm -rf $WORK_PROG
	;;
esac
echo ' '

