Module class_BoundaryConditions
  use class_BoundaryOperator
  use class_TensorProductGrid
  use class_DataDecomposition
  use class_ScalarVariable
  implicit none
  public  :: InitializeBoundaryConditions
  public  :: ApplyBCs
  public  :: ApplyBdryOp
  private :: MakeStdBdryOps
  
  !!Members:  
  integer                   :: NeumannOrder  
  type( BoundaryOperator )  :: BdryOp_X0DH
  type( BoundaryOperator )  :: BdryOp_X1DH
  type( BoundaryOperator )  :: BdryOp_X0NH
  type( BoundaryOperator )  :: BdryOp_X1NH
 
  type( BoundaryOperator )  :: BdryOp_Y0DH
  type( BoundaryOperator )  :: BdryOp_Y1DH
  type( BoundaryOperator )  :: BdryOp_Y0NH
  type( BoundaryOperator )  :: BdryOp_Y1NH
 
  type( BoundaryOperator )  :: BdryOp_Z0DH
  type( BoundaryOperator )  :: BdryOp_Z1DH
  type( BoundaryOperator )  :: BdryOp_Z0NH
  type( BoundaryOperator )  :: BdryOp_Z1NH
  
 Contains
 
 subroutine InitializeBoundaryConditions  &
             (Grid,DataMap,NeumannOrder,HomDirBCs,HomNeuBCs)
  use class_TensorProductGrid
  use class_DataDecomposition
  use class_BoundaryOperator

  implicit none
  integer,intent(in)                      :: NeumannOrder
  type( TensorProductGrid ),intent(in)    :: Grid
  type( DataDecomposition ),intent(in)    :: DataMap
  type( BoundaryOperator ),intent(inout)  :: HomDirBCs(3,2)
  type( BoundaryOperator ),intent(inout)  :: HomNeuBCs(3,2)
    
  call MakeStdBdryOps                                                  &
                    (NeumannOrder,Grid,DataMap,                        &
                     HomDirBCs(1,1),HomDirBCs(1,2), &
                     HomNeuBCs(1,1),HomNeuBCs(1,2),  &
                     HomDirBCs(2,1),HomDirBCs(2,2), &
                     HomNeuBCs(2,1),HomNeuBCs(2,2),  &
                     HomDirBCs(3,1),HomDirBCs(3,2), &
                     HomNeuBCs(3,1),HomNeuBCs(3,2)  )
                               
 end subroutine InitializeBoundaryConditions
 
 subroutine MakeStdBdryOps                                    &
           (order,Grid,DataMap,                               &
            BdryOp_X0DH,BdryOp_X1DH,BdryOp_X0NH,BdryOp_X1NH,  &
            BdryOp_Y0DH,BdryOp_Y1DH,BdryOp_Y0NH,BdryOp_Y1NH,  &
            BdryOp_Z0DH,BdryOp_Z1DH,BdryOp_Z0NH,BdryOp_Z1NH)
                           
   use class_BoundaryOperator
   use class_TensorProductGrid
   use class_DataDecomposition
   implicit none
  
   integer,intent(in)                      :: order
   type( TensorProductGrid ),intent(in)    :: Grid
   type( DataDecomposition ),intent(in)    :: DataMap
   
   type( BoundaryOperator ),intent(inout)  :: BdryOp_X0DH
   type( BoundaryOperator ),intent(inout)  :: BdryOp_X1DH
   type( BoundaryOperator ),intent(inout)  :: BdryOp_X0NH
   type( BoundaryOperator ),intent(inout)  :: BdryOp_X1NH
 
   type( BoundaryOperator ),intent(inout)  :: BdryOp_Y0DH
   type( BoundaryOperator ),intent(inout)  :: BdryOp_Y1DH
   type( BoundaryOperator ),intent(inout)  :: BdryOp_Y0NH
   type( BoundaryOperator ),intent(inout)  :: BdryOp_Y1NH
 
   type( BoundaryOperator ),intent(inout)  :: BdryOp_Z0DH
   type( BoundaryOperator ),intent(inout)  :: BdryOp_Z1DH
   type( BoundaryOperator ),intent(inout)  :: BdryOp_Z0NH
   type( BoundaryOperator ),intent(inout)  :: BdryOp_Z1NH
  
   integer                                 :: dim(3)=(/1,2,3/)
   integer                                 :: endpt(0:1)=(/0,1/)
   real(kind=8)                            :: val=0.d0
   character( len=80 )                     :: labelD,labelN
   
   labelD = 'Dirichlet'
   labelN = 'Neumann'
   
   !!X dimension, start, Dirichlet, homogeneous
    call NewBoundaryOperator  &
            (labelD,order,dim(1),endpt(0),Grid,DataMap,val,BdryOp_X0DH)
   
   !!X dimension, end, Dirichlet, homogeneous
    call NewBoundaryOperator  &
            (labelD,order,dim(1),endpt(1),Grid,DataMap,val,BdryOp_X1DH)
                
   !!X dimension, start, Neumann, homogeneous
    call NewBoundaryOperator  &
           (labelN,order,dim(1),endpt(0),Grid,DataMap,val,BdryOp_X0NH)
   
   !!X dimension, end, Neumann, homogeneous
    call NewBoundaryOperator  &
            (labelN,order,dim(1),endpt(1),Grid,DataMap,val,BdryOp_X1NH)
                           
   
   !!Y dimension, start, Dirichlet, homogeneous
    call NewBoundaryOperator  &
            (labelD,order,dim(2),endpt(0),Grid,DataMap,val,BdryOp_Y0DH)
   
   !!Y dimension, end, Dirichlet, homogeneous
    call NewBoundaryOperator  &
            (labelD,order,dim(2),endpt(1),Grid,DataMap,val,BdryOp_Y1DH)
              
   !!Y dimension, start, Neumann, homogeneous
    call NewBoundaryOperator  &
            (labelN,order,dim(2),endpt(0),Grid,DataMap,val,BdryOp_Y0NH)
   
   !!Y dimension, end, Neumann, homogeneous
    call NewBoundaryOperator  &
            (labelN,order,dim(2),endpt(1),Grid,DataMap,val,BdryOp_Y1NH)
     
    !!Z dimension, start, Dirichlet, homogeneous   
    call NewBoundaryOperator  &
            (labelD,order,dim(3),endpt(0),Grid,DataMap,val,BdryOp_Z0DH)
   
   !!Z dimension, end, Dirichlet, homogeneous
    call NewBoundaryOperator  &
            (labelD,order,dim(3),endpt(1),Grid,DataMap,val,BdryOp_Z1DH)
           
   !!Z dimension, start, Neumann, homogeneous
    call NewBoundaryOperator  &
            (labelN,order,dim(3),endpt(0),Grid,DataMap,val,BdryOp_Z0NH)
   
   !!Z dimension, end, Neumann, homogeneous
    call NewBoundaryOperator  &
            (labelN,order,dim(3),endpt(1),Grid,DataMap,val,BdryOp_Z1NH)  
  end subroutine MakeStdBdryOps
  


  
  subroutine ApplyBCs(phi,B)
  use class_ScalarVariable
  use class_BoundaryOperator
  implicit none 
  
  type( ScalarVariable ),intent(inout)   :: phi
  type( BoundaryOperator ),intent(inout) :: B(3,2)
  integer                                :: i,j
  integer,SAVE                           :: myid
  integer,SAVE                           :: numprocs
  integer                                :: ierr
  logical,SAVE                           :: first_entry=.TRUE.

  
  include 'mpif.h'
  if( first_entry ) then
   call mpi_comm_rank (MPI_COMM_WORLD,myid,ierr)     
   call mpi_comm_size (MPI_COMM_WORLD,numprocs,ierr) 
   first_entry=.FALSE.
  endif
  
  !!Apply BCs to both ends of coordinates x & y
  !!Operators for endpts "0" and "1" are 
  !!stored in B(:,1) and B(:,2). The coordinate
  !!dimensions 1,2,3 correspond to B(1,:),B(2,:),B(3,:).
  do i=1,2
    do j=1,2
      call ApplyBdryOp( B(i,j), phi )
    enddo
  enddo  
  
  !!Apply BCs in x and y
  !!Only top/bottom processors impose top/bottom bcs
  i=3   !! dimension along which data is distributed
   if(myid==0) then
     j = 1
     call ApplyBdryOp( B(i,j), phi )
   endif
   
   if(myid==numprocs-1) then
     j = 2
     call ApplyBdryOp( B(i,j), phi )
   endif
 end subroutine ApplyBCs
 
 
 subroutine ApplyBdryOp(B,phi)
  use class_ScalarVariable
  use class_BoundaryOperator
  
  implicit none 
  type( BoundaryOperator ),intent(in) :: B
  type( ScalarVariable ),intent(inout):: phi
  character( len=80 )                 :: label  
  real(kind=8)             :: ds,h
  real(kind=8),allocatable :: F(:,:,:)
  real(kind=8),allocatable :: C(:,:,:)
  real(kind=8),allocatable :: ones(:,:)
  real(kind=8),allocatable :: factor(:,:)
  integer                  :: dim
  integer                  :: endpt
  integer                  :: i,j
  integer                  :: n1,n2
  integer                  :: N
  integer                  :: isign
  integer                  :: idx
  integer                  :: order,adj_order    
  integer,SAVE             :: myid
  integer,SAVE             :: numprocs
  integer                  :: ierr
  logical,SAVE             :: first_entry=.TRUE.

  
  include 'mpif.h'
  if( first_entry ) then
   call mpi_comm_rank (MPI_COMM_WORLD,myid,ierr)     
   call mpi_comm_size (MPI_COMM_WORLD,numprocs,ierr) 
   first_entry=.FALSE.
  endif
  
  n1=B%n1
  n2=B%n2
  dim=B%DimId
  endpt=B%EndId  !!convention: 0 / 1 for start / end  
  order=B%order
  N=phi%DataMap%SlabDims(dim,myid)  
  label=B%label
    
  
  !!Deal w/ trigonometric BCs first. Nothing needs to be done
  !!when the derivs are type fourier, sin or cos as these
  !!are explicitly enforced via the basis functions. If, for
  !!some reason, periodicity is required but Fourier methods
  !!are not being used, we apply the pde constraint to both
  !!endpoints independently and then set as boundary conditions
  !!the average of the two independently computed values.
  if( label == 'Fourier' .or. label == 'fourier' ) return
  if( label == 'Cos' .or. label == 'cos' ) return
  if( label == 'Sin' .or. label == 'sin' ) return
  
  if( .NOT. allocated(F) ) allocate( F(n1,n2,1:order) )
  if( .NOT. allocated(C) ) allocate( C(n1,n2,0:order) )
  if( .NOT. allocated(ones) ) allocate( ones(n1,n2) )
  if( .NOT. allocated(factor) ) allocate( factor(n1,n2) )
  ones=1.d0
  
  if( label == 'Periodic' ) then
    select case(dim)
         case(1)  
            phi%SlabVals(n1,:,:) =  &
              ( phi%SlabVals(n1-1,:,:) + phi%SlabVals(1,:,:) )/2.
         case(2)  
            phi%SlabVals(:,n2,:) =  &
              ( phi%SlabVals(:,n2-1,:) + phi%SlabVals(:,1,:) )/2.     
         case(3)
            F(1:n1,1:n2,0) = ( phi%SlabVals(1:n1,1:n2,1)      &
                           +   phi%SlabVals(1:n1,1:n2,N) )/2.
            phi%SlabVals(1:n1,1:n2,1) = F(1:n1,1:n2,0)
            phi%SlabVals(1:n1,1:n2,N) = F(1:n1,1:n2,0)  
         case default
     end select   
   endif
  
   !! Set up coefficients for mixed bc representation
   !! Same discretizations used at each end, except forward/backward
   !! property which introduces a sign change to the coefficients
    if( endpt == 0 ) then  
     idx=1     !! *START* of coord. range 
     isign=1  
    elseif( endpt == 1 ) then
     idx=N     !! *END* of coord. range
     isign=-1
    else
     stop 'Error in designating endpoint in ApplyBdryOp (B%endpt)'
    endif
    
     !! Reduce order as needed for small grid sizes
     adj_order=minval( (/order,N-2/) )
     
     !! Discrete derivatives are expressed w/ respect to the uniformly
     !! spaced parameter s. What's really needed (for Neumann) are derivatives
     !! with respect to x. The stretching factor dx/ds is used to convert the
     !! s derivative to an x derivative. I need to think a bit about endpt
     !! singularities in mappings.
     if( phi%Grid%SpaceDims(dim)%label == 'StandardCheby' ) then
     !! use unevenly spaced formula in x, bypass x<->s altogether 
     !! as the transformation dx/ds is singular at the ends
       select case(adj_order)
        case(1)
       end select
       
       h=phi%Grid%SpaceDims(dim)%ds
      
     else
     
       h=phi%Grid%SpaceDims(dim)%ds * phi%Grid%SpaceDims(dim)%dxds(idx)
      
     endif

     !! Set up the discretization coefficients for the Neumann
     !! part of the boundary operator: B[f] = alpha*f + beta*df/dx
     !! Convention: the Neumann part is the derivative with respect
     !! to the coordinate, say x,y,z, not the parameter s and not the
     !! "outward facing unit normal".
     !! Mixed form for alpha f + beta df/dx = Data
     !! dfdx(0) = (1/h)(-f0 + f1) + O(h) 
     !! ---> f0 = c0 + c1*f1 + c2*f2 + ...
     
     !!  DOUBLE CHECK SIGNS for d/dn IN THIS SECTION
     
     select case(adj_order)
      case(1) 
      
         factor = isign*ones / (h*B%alpha - B%beta)
         C(1:n1,1:n2,0) =  h*B%BoundaryData*factor 
         C(1:n1,1:n2,1) = -B%beta*factor
         
      case(2)
         !! Mixed form for alpha f + beta df/dx = Data
         !! dfdx(0) = (1/2h)(-3f0 + 4f1 -f2) + O(h^2)
         
         factor = isign*ones / (2.*h*B%alpha -3.*B%beta)
         C(1:n1,1:n2,0) =  2.*h*B%BoundaryData*factor 
         C(1:n1,1:n2,1) = -4.*B%beta*factor
         C(1:n1,1:n2,2) =  B%beta*factor
      
      case(3)
         !! Mixed form for alpha f + beta df/dx = Data
         !! dfdx(0) = (1/6h)(-11f0 + 18f1 -9f2 +2f3) + O(h^3)
         
         factor = isign*ones / (6.*h*B%alpha -11.*B%beta)
         C(1:n1,1:n2,0) =  6.*h*B%BoundaryData*factor 
         C(1:n1,1:n2,1) =  -18.*B%beta*factor
         C(1:n1,1:n2,2) =   9.*B%beta*factor
         C(1:n1,1:n2,3) =  -2.*B%beta*factor
      
      case(4)
         !! Mixed form for alpha f + beta df/dx = Data
         !! dfdx(0) = (1/24h)(-50f0 + 96f1 -72f2 +32f3 -6f4) + O(h^4)
         factor = isign*ones / (24.*h*B%alpha -50.*B%beta)
         C(1:n1,1:n2,0) =  24.*h*B%BoundaryData*factor 
         C(1:n1,1:n2,1) =  -96.*B%beta*factor
         C(1:n1,1:n2,2) =   72.*B%beta*factor
         C(1:n1,1:n2,3) =  -32.*B%beta*factor
         C(1:n1,1:n2,4) =    6.*B%beta*factor
         
       case(5)
         !! Mixed form for alpha f + beta df/dx = Data
         !! dfdx(0) = (1/120h)(-274f0 + 600f1 -600f2 +400f3 -150f4 + 24f5) + O(h^4)
         factor = isign*ones / (120.*h*B%alpha -274.*B%beta)
         C(1:n1,1:n2,0) =  120.*h*B%BoundaryData*factor 
         C(1:n1,1:n2,1) =  -600.*B%beta*factor
         C(1:n1,1:n2,2) =   600.*B%beta*factor
         C(1:n1,1:n2,3) =  -400.*B%beta*factor
         C(1:n1,1:n2,4) =   150.*B%beta*factor
         C(1:n1,1:n2,5) =   -24.*B%beta*factor
      end select    
    
    
      !!Use the discretization coefficients to form the
      !!linear combination f0 = c0 + c1*f1  + c2*f2 ....
      !! idx+j = 2,3,4 ...        left end, idx=1
      !!         N-1,N-2,N-3 ...  right end idx=N
      select case(dim)
       case(1) 
        phi%SlabVals(idx,:,:)=C(:,:,0)
        do i=1,order
          j=isign*i
          F(1:n1,1:n2,i) = phi%SlabVals(idx+j,1:n1,1:n2)
          phi%SlabVals(idx,1:n1,1:n2) = phi%SlabVals(idx,1:n1,1:n2)    &
                                    + C(1:n1,1:n2,i)*F(1:n1,1:n2,i)
        enddo 
       case(2) 
        phi%SlabVals(:,idx,:)=C(:,:,0)
        do i=1,order
          j=isign*i
          F(1:n1,1:n2,i) = phi%SlabVals(1:n1,idx+j,1:n2)          
          phi%SlabVals(1:n1,idx,1:n2) = phi%SlabVals(1:n1,idx,1:n2)    &
                                      + C(1:n1,1:n2,i)*F(1:n1,1:n2,i)
        enddo 
       case(3)
         phi%SlabVals(:,:,idx)=C(:,:,0)
         do i=1,order
          j=isign*i
          F(1:n1,1:n2,i) = phi%SlabVals(1:n1,1:n2,idx+j)
          phi%SlabVals(1:n1,1:n2,idx) = phi%SlabVals(1:n1,1:n2,idx)    &
                                      + C(1:n1,1:n2,i)*F(1:n1,1:n2,i)
         enddo
       case default
        
      end select
       
100 if(allocated( F ))      deallocate( F )
    if(allocated( C ))      deallocate( C )
    if(allocated( ones ))   deallocate( ones )
    if(allocated( factor )) deallocate( factor )
 end subroutine ApplyBdryOp
 
 end Module class_BoundaryConditions
