!!=============================================
!!   flow_solve Data Modules
!!=============================================


!!====================================================================================
  module user_data  
!!====================================================================================
   include 'OOFlowSolve.h'
   integer                             :: nx, ny, nz
   integer                             :: AB_order,AM_order
   integer,allocatable                 :: mgrefine(:,:)
   integer,parameter                   :: mglevels=1
   
   real(kind=8)                        :: Lx, Ly, Lz
   real(kind=8)                        :: dt,t_start,t_end
   real(kind=8)                        :: f,g,rho_0,u0,nu
   real(kind=8)                        :: beta_dimensional,beta_tilde
   real(kind=8)                        :: kappa_1,kappa_2
   real(kind=8)                        :: s1_scale,s2_scale
   real(kind=8)                        :: p_scale
   real(kind=8)                        :: Ri,Rot,Re,Pr(NumScalars)
   real(kind=8)                        :: dgrad,bfreq,N2
   real(kind=8)                        :: time_scale,t_scale
   real(kind=8)                        :: MapParams(3,10)  !! for user maps
   real(kind=8)                        :: alpha_les(3),p_les(3),T_les
   
   character(len=1)                    :: scalar_kind(NumScalars)
   character(len=80)                   :: runlabel
   character(len=80)                   :: bc_desc(NumDims,2)
   character(len=80)                   :: u_deriv_types(2*NumDims)!! 1st&2nd derivs
   character(len=80)                   :: v_deriv_types(2*NumDims)
   character(len=80)                   :: w_deriv_types(2*NumDims)
   character(len=80)                   :: s1_deriv_types(2*NumDims)
   character(len=80)                   :: s2_deriv_types(2*NumDims)
   character(len=80)                   :: phi_deriv_types(2*NumDims)
   character(len=80)                   :: GradPhi_deriv_types(2*NumDims)
   character(len=80)                   :: div_deriv_types(2*NumDims)
   character(len=80)                   :: tmp_deriv_types(2*NumDims)
   character(len=80)                   :: BCtype(NumVars,NumDims,2)!! 2 endpts
   
   logical                             :: xdim_periodic,xdim_cos,xdim_sin
   logical                             :: ydim_periodic,ydim_cos,ydim_sin
   logical                             :: zdim_periodic,zdim_cos,zdim_sin
   logical                             :: xdim_implicit_bcs
   logical                             :: ydim_implicit_bcs
   logical                             :: zdim_implicit_bcs
   logical                             :: do_forcing
   logical                             :: LES,DNS
   logical                             :: do_immersed_boundary
   logical                             :: do_explicit_hydrostatic
   logical                             :: psolve_at_gridpoints
   logical                             :: psolve_exp_coeffs
   logical                             :: usolve_at_gridpoints
   logical                             :: usolve_exp_coeffs
   logical                             :: vsolve_at_gridpoints
   logical                             :: vsolve_exp_coeffs
   logical                             :: wsolve_at_gridpoints
   logical                             :: wsolve_exp_coeffs
   logical                             :: s1solve_at_gridpoints
   logical                             :: s1solve_exp_coeffs
   logical                             :: s2solve_at_gridpoints
   logical                             :: s2solve_exp_coeffs
!!====================================================================================
  end module user_data
!!====================================================================================



!!====================================================================================  
  module dependent_variables
!!====================================================================================
   use class_ScalarVariable
   use user_data,                   only: MaxNumDims,MaxNumScalars
   type( ScalarVariable )              :: velocity   (MaxNumDims)
   type( ScalarVariable )              :: scalars (MaxNumScalars)
!!====================================================================================
  end module dependent_variables
!!====================================================================================





!!====================================================================================  
  module independent_variables
!!====================================================================================
   use class_DataDecomposition
   use user_data,              only: MaxMGIndex
   type( DataDecomposition )      :: U_DataMap
   type( DataDecomposition )      :: V_DataMap
   type( DataDecomposition )      :: W_DataMap
   type( DataDecomposition )      :: S1_DataMap
   type( DataDecomposition )      :: S2_DataMap
   type( DataDecomposition )      :: Div_DataMap
   type( DataDecomposition )      :: Phi_DataMap(0:MaxMGIndex)
   type( DataDecomposition )      :: GradPhi_DataMap
   type( DataDecomposition )      :: b_DataMap
   type( TensorProductGrid )      :: Grid(0:MaxMGIndex)   
!!====================================================================================
  end module independent_variables
!!====================================================================================





!!====================================================================================
  module intermediate_variables
!!====================================================================================
   use class_ScalarVariable
   use user_data,              only: MaxMGIndex,NumVars,MaxABSlices,NumScratchVars
   type( ScalarVariable )         :: pd
   type( ScalarVariable )         :: phi(0:MaxMGIndex)
   type( ScalarVariable )         :: rhs_b,div_u
   type( ScalarVariable )         :: implicit_rhs(NumVars,MaxABSlices)
   type( ScalarVariable )         :: explicit_rhs(NumVars,MaxABSlices)
   type( ScalarVariable )         :: scratch(NumScratchVars)
   
   real(kind=8), allocatable,dimension(:)      ::      s1_bar(:)
   real(kind=8), allocatable,dimension(:)      ::      s2_bar(:)
   real(kind=8), allocatable,dimension(:)      ::  ddz_s1_bar(:)
   real(kind=8), allocatable,dimension(:)      ::  ddz_s2_bar(:)
   real(kind=8), allocatable,dimension(:)      :: ddzz_s1_bar(:)
   real(kind=8), allocatable,dimension(:)      :: ddzz_s2_bar(:)
!!====================================================================================
  end module intermediate_variables
!!====================================================================================


!!====================================================================================  
  module operators
!!====================================================================================
  use class_BoundaryOperator
  use class_SAOperator
  use class_Derivative
  use user_data,              only: MaxMGIndex,NumDims,NumVars

#include "finclude/petsc.h"
#include "finclude/petscvec.h"
#include "finclude/petscmat.h"
#include "finclude/petscda.h"
#include "finclude/petscksp.h"

  Vec     petsc_b, petsc_soln
  KSP     ksp(NumVars),kspfd(NumVars)
  Mat     A_PC,A

  character(len=80)             :: SolverType

  type( SAOperator )            :: L(0:MaxMGIndex)

  type( BoundaryOperator )      :: DirBdryOps(NumDims,2)
  type( BoundaryOperator )      :: NeuBdryOps(NumDims,2)
  type( BoundaryOperator )      :: PhiBdryOps(NumDims,2)
  type( BoundaryOperator )      :: u_BdryOps (NumDims,2)
  type( BoundaryOperator )      :: v_BdryOps (NumDims,2)
  type( BoundaryOperator )      :: w_BdryOps (NumDims,2)
  type( BoundaryOperator )      :: s1_BdryOps(NumDims,2)
  type( BoundaryOperator )      :: s2_BdryOps(NumDims,2)
  
  real(kind=8)                  :: alpha(NumVars)
  real(kind=8)                  :: beta(NumVars)
  real(kind=8)                  :: h2_scale_factor
  

!!====================================================================================  
  end module operators
!!====================================================================================



!!====================================================================================   
  module mpi_params
!!====================================================================================
   use user_data,             only: MaxNumMGLevels,MaxNumProcs
   character(len=3 )             :: DoMPITypes(0:MaxNumMGLevels)
   integer                       :: numprocs
   integer                       :: myid
   integer                       :: comm
   integer                       ::    subslab(0:MaxNumProcs-1,0:MaxNumProcs-1)
   integer                       ::     subcol(0:MaxNumProcs-1,0:MaxNumProcs-1)
   integer                       :: slab_sheet(0:MaxNumProcs-1,0:MaxNumProcs-1)
   integer                       ::  col_sheet(0:MaxNumProcs-1,0:MaxNumProcs-1)
   integer                       :: pencil
   integer                       :: status
   integer                       :: iproc
   integer                       :: numwords
   integer                       :: source
   integer                       :: dest
   integer                       :: tag
   integer                       :: nreqs
!!====================================================================================   
  end module mpi_params
!!====================================================================================  




!!====================================================================================
  module io_params
!!====================================================================================
   use user_data,             only: MaxNumOutputFileSets
   integer                       :: num_netcdf_files
   integer                       :: ncid
   integer                       :: idim
   integer                       :: jdim
   integer                       :: file_id
   integer                       :: rcode
   
   integer                       :: start       (4,MaxNumOutputFileSets)    
   integer                       :: count       (4,MaxNumOutputFileSets)
   integer                       :: ilocs       (3,MaxNumOutputFileSets)    
   integer                       :: jlocs       (3,MaxNumOutputFileSets)    
   integer                       :: klocs       (3,MaxNumOutputFileSets)
   integer                       :: variable_key(9,MaxNumOutputFileSets)

   integer                       :: my_kstart     (MaxNumOutputFileSets)
   integer                       :: my_kend       (MaxNumOutputFileSets)
   integer                       :: my_kinc       (MaxNumOutputFileSets)
   integer                       :: nsteps        (MaxNumOutputFileSets)
   integer                       :: nspace        (MaxNumOutputFileSets)
   integer                       :: time_counter  (MaxNumOutputFileSets)
   integer                       :: dimid_x       (MaxNumOutputFileSets)
   integer                       :: dimid_y       (MaxNumOutputFileSets)
   integer                       :: dimid_z       (MaxNumOutputFileSets)
   integer                       :: dimid_t       (MaxNumOutputFileSets)
   
   character(len=80)             :: filename_root(MaxNumOutputFileSets)
   character(len=80)             :: fullname     (MaxNumOutputFileSets)
   character(len=80)             :: mode         (MaxNumOutputFileSets)
   
   character(len=6)              :: cnum
   character(len=3)              :: cid,dimstr
!!====================================================================================
  end module io_params
!!====================================================================================

