#!/usr/bin/ksh
#######################################################
#####          LANCEMENT DU RUN                 ####### 
#######################################################
#_____________________________________________________
#
#       DEFINITION DES VARIABLES
#_____________________________________________________

MODE=$1
VERSION=$2
RUN=$3



if [ $# -ne 3 ]; then
  echo ' '
  echo " *** Mauvais arguments d'entree ! *** "
  echo '   exemple: ./lanc.ksh mode version numero_run'
  echo ' '
  exit 1
fi

echo ' '
echo ' '
case $1 in
  0)
   echo '========================'
   echo '==     MODE BATCH     =='
   echo '========================'
  ;;
  1)
   echo '========================'
   echo '==   MODE INTERACTIF  =='
   echo '========================'
  ;;
  *)
   echo '========================'
   echo '==     MODE TEST      =='
   echo '========================'
  ;;
esac
 echo ' '
 echo ' '

#_____________________________________________________
#
# PARAMETRES 
#_____________________________________________________

DATE=`date '+%Y%m%d'_'%H%M%S'`
echo `date`

HOME_DIR=$PWD
HOME_PROG=00FlowSolve_PL
SUB_PROJECT=sub_lelong
WORK_PROG=${RUN}_$HOST_${HOME_PROG}_${VERSION}_${DATE}

if [ ! -d $WORK_DIR  ]; then
   WORK_DIR=$HOME_DIR
   echo ' '
   echo ' pas de WORK_DIR definit, execution sur:'
   echo '    --> '$WORK_DIR
   echo ' '
fi

echo ' '
echo 'Definitions des variables:'
echo '   HOME_DIR    = '$HOME_DIR
echo '   HOME_PROG   = '$HOME_PROG'     Version: '$VERSION
echo '   SUB_PROJECT = '$SUB_PROJECT
echo '   WORK_DIR    = '$WORK_DIR
echo '   WORK_PROG   = '$WORK_PROG

#_____________________________________________________
#
# CREATION DU REPERTOIRE DE COMPILATION SUR LE WORKDIR
#_____________________________________________________

echo ' '
echo 'Creation du repertoire de compilation et copie des fichiers'
if [ ! -d $HOME_DIR/PROJECTS/$VERSION ]; then
   echo ' '
   echo '   WARNING: le projet '$VERSION" n'existe pas"
   exit 1
fi

cd $WORK_DIR
if  [ -d $WORK_PROG ]; then 
  rm -rf $WORK_PROG
fi
mkdir $WORK_PROG 
cd $WORK_PROG
mkdir output
cp -r $HOME_DIR/SRC src
cp -r $HOME_DIR/Makefile src/.
cp -r $HOME_DIR/Make.inc src/.
cp -r $HOME_DIR/PROJECTS/NEW_SRC/* src/.
cp -r $HOME_DIR/PROJECTS/$VERSION input
mv input/*.f90 src/.


#_____________________________________________________
#
#       COMPILATION DU PROG SUR LE WORKDIR
#_____________________________________________________

echo ' '
echo 'Compilation du code pour le projet: '$VERSION
# Compilation
cd $WORK_DIR/$WORK_PROG/src
make clean
make
if [ -f cfd.x ]; then
   echo '   Compilation successful ! Here we go'
else
   echo '   WARNING: Compilation failed !'
   cd $WORK_DIR
   rm -rf $WORK_PROG
   exit 1
fi

echo ' '
case $1 in
	0)
	#_____________________________________________________
	#
	#        OFFLINE NOHUP
	#_____________________________________________________
	echo ' Soumission du job... offline.'
	cp $HOME_DIR/$SUB_PROJECT $WORK_DIR/$WORK_PROG/.
	cd $WORK_DIR/$WORK_PROG
	nohup ./$SUB_PROJECT $WORK_DIR/$WORK_PROG $HOME/$WORK_PROG > $WORK_DIR/$WORK_PROG/nohup.out &
	;;
	1)
        #_____________________________________________________
        #
        #        ONLINE 
        #_____________________________________________________
        echo ' Mode interactif: execution sur le workdir ...'
        cd $WORK_DIR/$WORK_PROG
        NPROCS=`grep 'numprocs' $WORK_DIR/$WORK_PROG/input/problem_size.h | tr -d ' ' | grep 'numprocs=' | cut -c 20-21`
        export MP_PROCS=$NPROCS
        echo '   Execution sur '$MP_PROCS' procs.' 
        cd $WORK_DIR/$WORK_PROG
        cd src
        ./cfd.x 
        rm $HOME/$WORK_PROG
	;;
	*)
	#_____________________________________________________
	#
	#        TEST COMPILATION
	#_____________________________________________________
	echo ' End : successful test !'
	cd $WORK_DIR
	rm -rf $WORK_PROG
	;;
esac
echo ' '

