
!*****************************************************************************
immersed_bdry_flag='no'        ! yes or no
closure_flag='DNS'             ! DNS or LES
psolve_scheme='total_pressure' ! change_in_pressure,total_pressure

restart_flag='no'              ! 'yes' to start from a save/restart file
save_file=00000                ! ignored if 'no' above
user_ics_flag='no'             ! yes invokes subroutine user_ics, sets up ics (ignored if 'yes' above)
                               ! and Km_of_z/Kt_of_z
ics_flag='homogeneous'         ! homogeneous,user

user_rhs_flag='no'             ! yes invokes execution of user_rhs.f90

user_bcs_flag='yes'             ! yes invokes execution of user_bcs.f90 --> ex: forcage oscillant en surface
steady_BCS='yes'                ! yes : determines whether set_BCs is re-evaluated each time step
                               !       our F_of_t is applied in apply_BCs.f90, so treat as 'steady' here
!*****************************************************************************


!*****************************************************************************
ts_profil='linear'             ! 'interp'/'linear'/'constant'  if 'no' POMME data
pomme_data_flag='no'           ! 'yes' lecture des champs POMME [T,S] 
pomme_filename='../input/pomme_break_rho_alpha_T_ics.ascii'
dTdz= 4*1.4133829e-3
alpha_rho= -3.105172e-4 / dTdz ! ambiet_density_gradient=-3.105172e-4 [kg/m4] souhait 
print*,'alpha_rho=',alpha_rho
                               ! rho = rho0 + alpha_rho * ( T - T0)
	                       ! T0 = Tmin set in set_ambient_stratification.f90 
!*****************************************************************************


!************************     Stress parameters     ***************************
Patch_Radius=50.e3             ! [m]
Patch_Decay_Scale=50.e3        ! [m]
delta=1./(Patch_Decay_Scale**2)
tau_0=0.0                     ! [kg/(ms2)]
!******************************************************************************


!****************  DIMENSIONAL SCALES ****************************************
pi=acos(-1.0)                          ! definition of pi
g =9.81                                ! gravitational acceleration [m/s2]
xlat= 47.5
xOmega= 2*pi/(24*3600)
f=2*xOmega*sin(xlat*pi/180)            ! Coriolis parameter [rad/s] Ekman layer scale: delta=sqrt(2nu/f)
beta_dimensional=2*xOmega*cos(xlat*pi/180)/(6400.e3) ! beta effect [rad/(m.s)]
L_scale = 4000.                        ! length scale to be used in nondimensionalization [m]
U_scale = 0.10                         ! characteristic velocity scale [m/s]
rho0 = 1027.                           ! characteristic fluid density [kg/m3]
delta_rho=2.50                         ! characteristic density contrast [kg/m3]  (25)
nu = 1.e-6                             ! eddy viscosity [m2/s]
kappa1 = nu                            ! scalar 1 diffusivity [m2/s] stored in Kt   (Temp)
kappa2 = nu                            ! scalar 2 diffusivity [m2/s] stored in Kt   (Salinity)
kappa = nu                             !  not used
Kh = 0.02                              ! horizontal diffusivity for [u,v,w,s1,s2] on (x,y) 
s1_scale=1.                           ! characteristic s1 contrast     (Temp, deg C)
s2_scale=1.                           ! characteristic s2 contrast     (Salinity, psu)
HV_RATIO=1.                            ! horizontal/vertical viscosity ratio  (50)
HV_METHOD = 2
!*******************************************************************************


!************     Derived scales, DO NOT ALTER. *******************************
t_scale = L_scale/U_scale    
DEL = (delta_rho)/L_scale  
density_scale=DEL*L_scale
pressure_scale=rho0*U_scale**2
Nsquared = (g/rho0)*DEL     
N0 = sqrt(Nsquared)    
if(N0 /= 0 ) then; BV_per = 2.*pi/N0; else; BV_per=-1.e32; endif
if(f  /= 0 ) then; inertial_per = 2.*pi/f; else; inertial_per=-1.e32; endif
!*******************************************************************************


!*******************  TIME STEP / OUTPUT ***************************************
dt = (300)/t_scale        ! dimensionless time step
t_start =  0./t_scale     ! dimensionless starting time
nsteps =  2000            ! number of timesteps to integrate over
i_full_fields = 5        ! # of steps between output of full 3d fields
i_restart = 1000             ! # of steps between output of save/restart files
i_filter = 1               ! # of steps between compact filtering calls
i_stat = 90                ! # of steps between executions of energetics
i_mean_pressure=1          ! # of steps between removal of mean pressure
!*****************************************************************************


!*************  ALLOCATION TABLEAUX BOUNDARY CONDITIONS  *********************
  allocate ( boundary_type1(ny,locnz),boundary_type2(ny,locnz) )
  allocate ( boundary_type3(nx,locnz),boundary_type4(nx,locnz) )
  allocate ( boundary_type5(nx,ny),boundary_type6(nx,ny) )
  allocate ( BC_pressure1(ny,locnz),BC_pressure2(ny,locnz) )
  allocate ( BC_pressure3(nx,locnz),BC_pressure4(nx,locnz) )
  allocate ( BC_pressure5(nx,ny),BC_pressure6(nx,ny) )
  allocate ( BC_vel1(ny,locnz),BC_vel2(ny,locnz) )
  allocate ( BC_vel3(nx,locnz),BC_vel4(nx,locnz) )
  allocate ( BC_vel5(nx,ny),BC_vel6(nx,ny) )
  allocate ( BC_scalar1(ny,locnz,num_scalars),BC_scalar2(ny,locnz,num_scalars) )
  allocate ( BC_scalar3(nx,locnz,num_scalars),BC_scalar4(nx,locnz,num_scalars) )
  allocate ( BC_scalar5(nx,ny,num_scalars),BC_scalar6(nx,ny,num_scalars) )
!*****************************************************************************


!***********************   BOUNDARY TYPE  ************************************
! periodic in X
boundary_type1(:,:)='periodic'                   ! solid, permeable or periodic
BC_pressure1(:,:)  ='N/A'                        ! Neumann, periodic or none
BC_vel1(:,:)       ='N/A'                        ! specified_stress/gradient or N/A
BC_scalar1(:,:,:)  ='N/A'                        ! ""

boundary_type2(:,:)='periodic'                   ! solid, permeable or periodic
BC_pressure2(:,:)  ='N/A'                        ! Neumann, periodic or none
BC_vel2(:,:)       ='N/A'                        ! specified_stress/gradient or N/A
BC_scalar2(:,:,:)  ='N/A'                        ! ""

! periodic in Y
boundary_type3(:,:)='periodic'                   ! solid, permeable or periodic
BC_pressure3(:,:)  ='n/a'                        ! Neumann, periodic or none
BC_vel3(:,:)       ='n/a'                        ! specified_stress/gradient or N/A
BC_scalar3(:,:,:)   ='n/a'                        ! ""

boundary_type4(:,:)='periodic '                  ! solid, permeable or periodic
BC_pressure4(:,:)  ='n/a'                        ! Neumann, periodic or none
BC_vel4(:,:)       ='n/a'                        ! specified_stress/gradient or N/A
BC_scalar4(:,:,:)  ='n/a'                        ! ""

! solid, free-slip, adiabatic bottom boundary
boundary_type5(:,:)='solid'                      ! solid, permeable or periodic
BC_pressure5(:,:)  ='Neumann'                    ! Neumann, periodic or none
BC_vel5(:,:)       ='specified_stress'           ! 
BC_scalar5(:,:,:)  ='specified_gradient'         ! 

! solid specified stress, adiabatic upper surface 
boundary_type6(:,:)='solid'                      ! solid, permeable or periodic
BC_pressure6(:,:)  ='Neumann'                    ! Neumann, periodic or none
BC_vel6(:,:)       ='specified_stress'           ! specified dv/dz as function of x 
BC_scalar6(:,:,:)  ='specified_gradient'         ! 
!***************************************************************************
