subroutine set_diffusion

! routine to set the diffusivity profiles, Km_of_z and Kt_of_z

use grid_info
use user_parameters       ! for Km_of_z and Kt_of_z
use dependent_variables   
use dimensional_scales    
use counters_flags_etc
use mpi_parameters
implicit none
include '../input/problem_size.h'
include 'mpif.h'

! local variables
 real                :: zpos
 real,dimension(nz)  :: ztot
 real                :: max_nu,min_nu,depth,Patch_Decay_Scale,delta 
 real,dimension(nz)  :: coeff_diff
 character(len=3)    :: cid
 character(len=80)   :: filename

!!! real, dimension(locnz)  :: zz

!!! Paramtres du profil de diffusion verticale
 max_nu=1.e-4   ! valeur du coeff. de diffusion maximal
 min_nu=1.e-2     ! valeur du coeff. de diffusion minimal 
 depth=100.      ! profondeur pour un coeff. quasi constant
 Patch_Decay_Scale=10.
 delta=1./(Patch_Decay_Scale**2)

 !! Collecte du vecteur profondeur
 call MPI_GATHER(Grid(0)%z(1,:)*L_scale,locnz,MPI_REAL,ztot,locnz,MPI_REAL,0,comm,ierr)

 print*,'diffusion, Lz',Lz

 !! Calcul du coefficient de diffusion verticale
 if ( myid == 0 ) then
  print*,'max_nu=',max_nu,'  min_nu=',min_nu,'  depth=',depth,'  delta=',delta
  do k=1,nz
    zpos = ztot(k)     ! [m]
    coeff_diff(k) = 1 - tanh( delta*(Lz-depth-zpos) )
  enddo
  coeff_diff = coeff_diff / maxval(abs(coeff_diff)) 
  coeff_diff = min_nu  + (max_nu-min_nu) * coeff_diff 
 endif
 
 !! repartition des donnes
 call MPI_SCATTER(coeff_diff,locnz,MPI_REAL,Km_of_z,locnz,MPI_REAL,0,comm,ierr)
 
 !! Meme coeff de diffusion pour le moment et les scalaires
 Kt_of_z = Km_of_z

!!! Enregistrement de Km_of_z pour chaque processeur (pour vrification)
 write(unit=cid,fmt=501) myid
501 format(I3.3)
 filename='../output/coeff_diffusion.data_'//cid
 open(905,file=filename,status='unknown')
 do k=1,locnz
   write(905,*) Grid(0)%z(1,k)*L_scale,Km_of_z(k)
 enddo
 close(905)

 return

end subroutine set_diffusion
