program cfd                                 ! XZ version

#define DEBUG_LEVEL 1
! Module definitions in cfd_modules.F90
use  grid_info
use  mpi_parameters
use  dimensional_scales
use  boundary_information
use  pde_parameters
use  dependent_variables
use  intermediate_variables
use  counters_flags_etc
use  immersed_boundary
use  user_parameters
use  mpi_times

implicit none

include 'mpif.h'
include '../input/problem_size.h'

!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% ONLY EXECUTABLE STATEMENTS BELOW HERE  %%%%%%%%%%%%%%%%%%%

 include 'start_MPI.h'

 call MPI_Time(0)

!!! LAPACK_FLAG='double'
 LAPACK_FLAG='single'
 call initialize_grids(0)              ! initialize spatial grids at all discretization levels
 call initialize_cfd                   ! basic setup, read input parameters etc.
 call initialize_user                  ! e.g. user can set nontrivial initial conditions


 print*,'loop...'
 
 do istep=istart,iend                ! MAIN TIME STEPPING LOOP
  t = t_start + (istep-istart)*dt    ! dimensionless time

  call save                          ! write restart files {i_restart}
!  print*,'...  call save ...'
!  call maxim

  call write3d_netcdf                ! output 3d fields {i_full_fields}
!  if ( prob_eos == 'yes' ) then  
!   print*,'...  call write3d_netcdf ...'
!   call maxim
!   print*,' ARRET IMPOSE ! '
!   call mpi_barrier(comm,ierr)
!   call MPI_Time(1)
!   call mpi_finalize(ierr)
!   STOP
!  endif

  call rhs                           ! compute rhs for U,v,W & pd neglecting
  !print*,'...  call rhs ...'
  !call maxim
    ! only pressure gradients

  call time_step                     ! time step w/o pressure gradient => u_star
  !print*,'...  call time_step ...'
  !call maxim
  
  call divergence                    ! compute "divergence" of u_star -> div_u
  !print*,'...  call divergence ...'
  !call maxim
  
  call set_rhs_for_pressure          ! set the rhs for pressure equation L[p]=b
  !print*,'...  call set_rhs_for_pressure ...'
  !call maxim

  call psolve_xy_periodic            ! solve pressure using xyfft & direct solve in z
  !print*,'...  call psolve_xy_periodic ...'
  !call maxim

   
  call gradient                      ! compute pressure "gradient"
  !print*,'...  call gradient ...'
  !call maxim
  
  call project                       ! project u_star onto div-free subspace =>[U,v,W]
  !print*,'...  call project ...'
  !call maxim
  
  call filter                        ! compact spatial filtering(i_filter,alpha)
  !print*,'...  call filter ...'
  !call maxim
  
  call set_BC_values                 ! compute u boundary values for faces 1:6
  !print*,'...  call set_BC_values ...'
  !call maxim

  call apply_BCs                     ! impose exact satisfaction of BCs
  !print*,'...  call apply_BCs ...'
  !call maxim

  call user_BCs                      ! impose user-defined
  !print*,'...  call user_BCs ...'
  !call maxim

  include 'toggle.h'                 ! toggle counters, indices etc
  !print*,'...  call toggle.h'

  call maxim

enddo

 call mpi_barrier(comm,ierr)

 call MPI_Time(1)

 call mpi_finalize(ierr)

end program cfd
