!================================
subroutine set_BC_values
!================================

#define DEBUG_LEVEL 1

! ****  Most arrays available via use statements ******
 use grid_info
 use boundary_information
 use dependent_variables
 use pde_parameters
 use dimensional_scales
 use mpi_parameters
 use counters_flags_etc
 use user_parameters
 use user_routines
 implicit none
!******************************************************
 include '../input/problem_size.h'
 include 'mpif.h'
!******************************************************
 real dv_dzeta,f_of_t
 integer ifield

!!! xlv
!==================================================================
 integer         :: field_id
 real            :: alpha1,alpha2,alpha3,alpha4
 real,dimension(:,:),allocatable   :: valsu,valsw 
!===================================================================

 if( istep > istart .and. steady_BCS=='yes' ) return
!#if DEBUG_LEVEL >= 1
 if(myid==0) write(0,*) 'hello world from set_BC_values'
!#endif

! FACE 1:  xi=0
 BC_values1(:,:,1)=0.0    ! BC_vel1(:,:)='solid' => U
 BC_values1(:,:,2)=0.0    ! BC_vel1(:,:)='specified_value' => v
 BC_values1(:,:,3)=0.0    ! BC_vel1(:,:)='specified_value' => W
 BC_values1(:,:,4)=0.0    ! BC_scalar1(:,:,1)='specified_gradient' => d/dxi (s1)
 if(num_scalars>1)  &
 BC_values1(:,:,5)=0.0    ! BC_scalar1(:,:,2)='specified_gradient' => d/dxi (s2)

! FACE 2:  xi=1
 BC_values2(:,:,1)=0.0    ! BC_vel1(:,:)='solid' => U
 BC_values2(:,:,2)=0.0    ! BC_vel1(:,:)='specified_value' => v
 BC_values2(:,:,3)=0.0    ! BC_vel1(:,:)='specified_value' => W
 BC_values2(:,:,4)=0.0    ! BC_scalar1(:,:,1)='specified_gradient' => d/dxi (s1)
 if(num_scalars>1)  &
 BC_values2(:,:,5)=0.0    ! BC_scalar1(:,:,2)='specified_gradient' => d/dxi (s2)

! FACE 3:  eta=0
 BC_values3(:,:,1)=0.0    ! BC_vel3(:,:)='specified "stress"' => du/deta
 BC_values3(:,:,2)=0.0    ! BC_vel3(:,:)='specified "stress"' => "dv/deta"
 BC_values3(:,:,3)=0.0    ! BC_vel3(:,:)='specified "stress"' => dw/deta
 BC_values3(:,:,4)=0.0    ! BC_scalar3(:,:,1)='specified_gradient' => ds1/deta
 if(num_scalars>1)  &
 BC_values3(:,:,5)=0.0    ! BC_scalar3(:,:,2)='specified_gradient' => ds2/deta

! FACE 4:  eta=1
 BC_values4(:,:,1)=0.0    ! BC_vel4(:,:)='specified "stress"' => du/deta
 BC_values4(:,:,2)=0.0    ! BC_vel4(:,:)='specified "stress"' => "dv/deta"
 BC_values4(:,:,3)=0.0    ! BC_vel3(:,:)='specified "stress"' => dw/deta
 BC_values4(:,:,4)=0.0    ! BC_scalar4(:,:,1)='specified_gradient' => ds1/deta
 if(num_scalars>1)  &
 BC_values4(:,:,5)=0.0    ! BC_scalar4(:,:,2)='specified_gradient' => ds2/deta

! FACE 5:  zeta=0   ! values set for free-slip wall
if( myid == 0 ) then
 BC_values5(:,:,1)=0.0    ! BC_vel5(:,:)='specified_stress' => dU/dzeta=0
 BC_values5(:,:,2)=0.0    ! BC_vel5(:,:)='specified_stress' => dv/dzeta=0
 BC_values5(:,:,3)=0.0    ! BC_vel5(:,:)='specified_value' => W=0
 do j=1,ny
  BC_values5(:,j,4)=0.0                   ! d/dzeta( s1 )=0, total gradient=ambient value
  if(num_scalars>1)  &
  BC_values5(:,j,5)=0.0                   ! d/dzeta( s2 )=0, total gradient=ambient value
 enddo
endif

! FACE 6:  zeta=1
if( myid == numprocs-1 ) then


  if ( BC_vel6(1,1) == 'specified_stress' ) then

       allocate( valsu(nx,ny) , valsw(nx,ny) )

       do field_id = 1,5
          call user_bdry_forcing( Grid(0)%x(:,1)*L_scale , Grid(0)%y(:)*L_scale , Grid(0)%z(1,:)*L_scale , &
                                  t*t_scale , BC_values6(:,:,field_id) , field_id ,                        &
                                  nx , ny , locnz )
       enddo

       !!! Oprations pour coller  la rotation de la grille
       k=locnz
       valsu = BC_values6(:,:,1)
       valsw = BC_values6(:,:,3)
       do i=1,nx
          alpha1=Grid(0)%x_xi(i,k)
          alpha2=Grid(0)%x_zeta(i,k)
          alpha3=Grid(0)%z_xi(i,k)
          alpha4=Grid(0)%z_zeta(i,k)
          do j=1,ny
               BC_values6(i,j,1)=1./(alpha1*alpha4-alpha3*alpha2)*( alpha4*valsu(i,j) - alpha2*valsw(i,j) )
               BC_values6(i,j,3)=1./(alpha1*alpha4-alpha3*alpha2)*(-alpha3*valsu(i,j) + alpha1*valsw(i,j) )
          enddo
       enddo

       deallocate( valsu , valsw )

  else  !! any other condition

    BC_values6(:,:,1)=0.0                    ! BC_vel5(:,:)='specified_stress' => dU/dzeta=0
    BC_values6(:,:,2)=0.0                    ! BC_vel5(:,:)='specified_stress' => dv/dzeta=0
    BC_values6(:,:,3)=0.0                    ! BC_vel5(:,:)='specified_value' => W=0
    BC_values6(:,:,4)=0.0                    ! d/dzeta( s1 )=0, total
    if(num_scalars>1) BC_values6(:,:,5)=0.   ! d/dzeta( s2 )=0, total
      
  endif


endif


end subroutine set_BC_values


