subroutine write3D_netcdf

#define DEBUG_LEVEL 1
 use mpi_parameters
 use grid_info
 use dependent_variables
 use intermediate_variables
 use dimensional_scales
 use counters_flags_etc
 use pde_parameters
 use immersed_boundary
 use user_parameters
 implicit none

 include 'netcdf.inc'
 include '../input/problem_size.h'

 character(len=5)                     ::  cnum
 character(len=3)                     ::  cid
 character(len=80)                    ::  filename
 integer                              ::  ncid,rcode 
 integer                              ::  ix,iz 
 integer                              ::  kglobal

!Dimensions and dimension variables
 integer                              ::  tdimensionID  
 integer                              ::  start1D(1)
 integer                              ::  count1D(1)   
      
!4D Variables
 integer                              ::  s1VarID,s2VarID,uVarID,vVArID,divVarID
 integer                              ::  wVarID,pVarID,KmVarID
 integer                              ::  start4D(4),count4D(4)

!Arrays for holding data
 real,dimension(:),allocatable        ::  tdimValues         ! 1D array for holding time data
 real,dimension(:),allocatable        ::  hold1D             ! 1D scratch array 
 real,dimension(:,:,:),allocatable    ::  hold3D             ! 3D scratch array 
 real                                 ::  uu,ww
!!! real                                 ::  maxu,minu,maxv,minv
!!! real                                 ::  maxxi,minxi,maxzeta,minzeta

!Execute routine only at specified time steps
 if( mod(istep,i_full_fields) /= 0 ) return
 allocate ( tdimValues(1), hold1D(locnz), hold3D(nx,ny,locnz) )

!Initialize a new netcdf file
 call init3D_netcdf
#if DEBUG_LEVEL >= 1
 if( myid == 0 ) write(0,*) 'hello world from write3D_netcdf, init3D complete'
#endif

!Create the filename
 write(unit=cnum,fmt=100) istep
100 format(I5.5)
 write(unit=cid,fmt=101) myid
101 format(I3.3)
 if( numprocs > 1 ) then
  filename='../output/cfd3D_'//cnum//'_'//cid
 else
  filename='../output/cfd3D_'//cnum
 endif

!Open the netcdf file and get variable id's.
 rcode = NF_OPEN(filename,NF_WRITE,ncid)
  if( rcode /= NF_NOERR ) call handle_err(rcode)

 rcode = NF_INQ_VARID(ncid,'timedimension',tdimensionID)
  if( rcode /= NF_NOERR ) call handle_err(rcode)
 rcode = NF_INQ_VARID(ncid,'TempVar',s1VarID)
  if( rcode /= NF_NOERR ) call handle_err(rcode)
if(num_scalars>1) then
 rcode = NF_INQ_VARID(ncid,'SalinityVar',s2VarID)
  if( rcode /= NF_NOERR ) call handle_err(rcode)
endif
 rcode = NF_INQ_VARID(ncid,'uVar',uVarID)
  if( rcode /= NF_NOERR ) call handle_err(rcode)
 rcode = NF_INQ_VARID(ncid,'vVar',vVarID)
  if( rcode /= NF_NOERR ) call handle_err(rcode)
 rcode = NF_INQ_VARID(ncid,'wVar',wVarID)
  if( rcode /= NF_NOERR ) call handle_err(rcode)
 rcode = NF_INQ_VARID(ncid,'pVar',pVarID)
  if( rcode /= NF_NOERR ) call handle_err(rcode)
 rcode = NF_INQ_VARID(ncid,'KmVar',KmVarID)
  if( rcode /= NF_NOERR ) call handle_err(rcode)
 rcode = NF_INQ_VARID(ncid,'divVar',divVarID)
  if( rcode /= NF_NOERR ) call handle_err(rcode)

!Write time value into time dimension
 start1D(1) = write3D_counter
 count1D(1) = 1
 tdimValues(1) = t * t_scale
 rcode = NF_PUT_VARA_REAL(ncid,tdimensionID,start1D,count1D,tdimValues)
  if( rcode /= NF_NOERR ) call handle_err(rcode)
      
!Write 4D data to netcdf file
 start4D(1) = 1
 start4D(2) = 1
 start4D(3) = 1
 start4D(4) = write3D_counter
 count4D(1) = nx
 count4D(2) = ny
 count4D(3) = locnz   ! Local data space
 count4D(4) = 1


!scalar 1 (dimensional)
 do j=1,ny
!!  hold3D(1:nx,j,1:locnz) = (s1_bar(1:nx,1:locnz) + s1(1:nx,j,1:locnz)) - s1_scale
     hold3D(1:nx,j,1:locnz) = (s1_bar(1:nx,1:locnz) + s1(1:nx,j,1:locnz)) * s1_scale
 enddo

!  do k=1,locnz
!  kglobal= k + myid*locnz
!  print*,'kglobal=',kglobal,'   s1_total=',hold3D(1,1,k)
!  enddo

  filename='../output/TEMP128_'//cnum//'_'//cid
  open(21,file=filename,status='new')
  do ix=1,nx
   write(21,221) (hold3D(ix,int(ny/2),iz),iz=1,locnz)
  enddo
  close(21)
221 format(6(1pe13.5))
!
!Write variable to netcdf file
 rcode = NF_PUT_VARA_REAL(ncid,s1VarID,start4D,count4D,hold3D)
  if( rcode /= NF_NOERR ) call handle_err(rcode)

if(num_scalars>1) then
!scalar 2 (dimensional)
 do j=1,ny
  hold3D(1:nx,j,1:locnz) = (s2_bar(1:nx,1:locnz)+s2(1:nx,j,1:locnz)) * s2_scale
 enddo

!  do k=1,locnz
!  kglobal= k + myid*locnz
!  print*,'kglobal=',kglobal,'   s2_total=',hold3D(1,1,k)
!  enddo

!Write variable to netcdf file
 rcode = NF_PUT_VARA_REAL(ncid,s2VarID,start4D,count4D,hold3D)
  if( rcode /= NF_NOERR ) call handle_err(rcode)
endif

!!!!density (dimensional)
!!! do j=1,ny
!!!  hold3D(1:nx,j,1:locnz) = (rho_bar(1:nx,1:locnz)+pd(1:nx,j,1:locnz))*density_scale
!!! enddo
!!!!
!!!  filename='../output/DENS3_'//cnum//'_'//cid
!!!  open(21,file=filename,status='new')
!!!  do ix=1,nx
!!!   write(21,221) (hold3D(ix,3,iz),iz=1,locnz)
!!!  enddo
!!!  close(21)
!!!!!!  filename='../output/DENS65_'//cnum//'_'//cid
!!!!!!  open(21,file=filename,status='new')
!!!!!!  do ix=1,nx
!!!!!!   write(21,221) (hold3D(ix,65,iz),iz=1,locnz)
!!!!!!  enddo
!!!!!!  close(21)
!!!  filename='../output/DENS128_'//cnum//'_'//cid
!!!  open(21,file=filename,status='new')
!!!  do ix=1,nx
!!!   write(21,221) (hold3D(ix,128,iz),iz=1,locnz)
!!!  enddo
!!!  close(21)
!!!!!!  filename='../output/DENS193_'//cnum//'_'//cid
!!!!!!  open(21,file=filename,status='new')
!!!!!!  do ix=1,nx
!!!!!!   write(21,221) (hold3D(ix,193,iz),iz=1,locnz)
!!!!!!  enddo
!!!!!!  close(21)
!!!221 format(7(1pe11.3))
!
!u velocity component
!convert to Cartesian and express in dimensional units
!!! maxu = MAXVAL(u)
!!! minu = MINVAL(u)
!!! maxxi = MAXVAL(Grid(0)%x_xi(:,:))
!!! minxi = MINVAL(Grid(0)%x_xi(:,:))
!!! maxzeta = MAXVAL(Grid(0)%x_zeta(:,:))
!!! minzeta = MINVAL(Grid(0)%x_zeta(:,:))
!!! write(0,*) 'A_MAXMIN_U',myid,maxu,minu,maxxi,minxi,maxzeta,minzeta
 do i=1,nx
  do j=1,ny
   do k=1,locnz
    uu = Grid(0)%x_xi(i,k)*U(i,j,k) + Grid(0)%x_zeta(i,k)*W(i,j,k)
!       uu = Grid(0)%x_xi(i,k)*u_star(i,j,k) + Grid(0)%x_zeta(i,k)*w_star(i,j,k)
    hold3D(i,j,k) = uu * U_scale
   enddo
  enddo
 enddo
!!! maxu = MAXVAL(hold3D)
!!! minu = MINVAL(hold3D)
!!! write(0,*) 'B_MAXMIN_U',myid,maxu,minu
!Write variable to netcdf file
 rcode = NF_PUT_VARA_REAL(ncid,uVarID,start4D,count4D,hold3D)
  if( rcode /= NF_NOERR ) call handle_err(rcode)

!v velocity component
!!! maxv = MAXVAL(v)
!!! minv = MINVAL(v)
!!! write(0,*) 'A_MAXMIN_V',myid,maxv,minv
 hold3D(:,:,:) = v(:,:,:) * U_scale
!!! maxv = MAXVAL(hold3D)
!!! minv = MINVAL(hold3D)
!!! write(0,*) 'B_MAXMIN_V',myid,maxv,minv
!Write variable to netcdf file
 rcode = NF_PUT_VARA_REAL(ncid,vVarID,start4D,count4D,hold3D)
  if( rcode /= NF_NOERR ) call handle_err(rcode)

!w velocity component
!convert to Cartesian and express in dimensional units
 do i=1,nx
  do j=1,ny
   do k=1,locnz
    ww = Grid(0)%z_xi(i,k)*U(i,j,k) + Grid(0)%z_zeta(i,k)*W(i,j,k)
!      ww = Grid(0)%z_xi(i,k)*u_star(i,j,k) + Grid(0)%z_zeta(i,k)*w_star(i,j,k)
    hold3D(i,j,k) = ww * U_scale
   enddo
  enddo
 enddo
!Write variable to netcdf file
 rcode = NF_PUT_VARA_REAL(ncid,wVarID,start4D,count4D,hold3D)
  if( rcode /= NF_NOERR ) call handle_err(rcode)
!
!!!  filename='../output/WVEL3_'//cnum//'_'//cid
!!!  open(21,file=filename,status='new')
!!!  do ix=1,nx
!!!   write(21,221) (hold3D(ix,3,iz),iz=1,locnz)
!!!  enddo
!!!  close(21)
!!!!!!  filename='../output/WVEL65_'//cnum//'_'//cid
!!!!!!  open(21,file=filename,status='new')
!!!!!!  do ix=1,nx
!!!!!!   write(21,221) (hold3D(ix,65,iz),iz=1,locnz)
!!!!!!  enddo
!!!!!!  close(21)
  filename='../output/WVEL128_'//cnum//'_'//cid
  open(21,file=filename,status='new')
  do ix=1,nx
   write(21,221) (hold3D(ix,int(ny/2),iz),iz=1,locnz)
  enddo
  close(21)
!!!!!!  filename='../output/WVEL193_'//cnum//'_'//cid
!!!!!!  open(21,file=filename,status='new')
!!!!!!  do ix=1,nx
!!!!!!   write(21,221) (hold3D(ix,193,iz),iz=1,locnz)
!!!!!!  enddo
!!!!!!  close(21)
!!!!
!!!!!!   nslice = 27
!!!!!!   nzout = 4
!!!!!!   if(myid.eq.nslice) then
!!!!!!      filename = '../output/TWVEL_'//cnum//'_'//cid
!!!!!!      open(21,file=filename,status='new')
!!!!!!      do ix=1,nx
!!!!!!         write(21,221) (hold3D(ix,iy,nzout),iy=1,ny)
!!!!!!      enddo
!!!!!!      close(21)
!!!!!!   endif
!
!pressure variable (total stored in block 2, dP in block 1)
 hold3D(:,:,:) = pressure(:,:,:,2) * pressure_scale
!Write variable to netcdf file
 rcode = NF_PUT_VARA_REAL(ncid,pVarID,start4D,count4D,hold3D)
  if( rcode /= NF_NOERR ) call handle_err(rcode)

!Km variable
 hold3D(:,:,:) = K_m(:,:,:) * U_scale * L_scale
!Write variable to netcdf file
 rcode = NF_PUT_VARA_REAL(ncid,KmVarID,start4D,count4D,hold3D)
  if( rcode /= NF_NOERR ) call handle_err(rcode)

!div variable
 hold3D(:,:,:) = div_u(:,:,:,new) * (1./t_scale)
!hold3D(:,:,:) = b
!Write variable to netcdf file
 rcode = NF_PUT_VARA_REAL(ncid,divVarID,start4D,count4D,hold3D)
  if( rcode /= NF_NOERR ) call handle_err(rcode)

!Close netcdf file.
 rcode = NF_CLOSE(ncid)
  if( rcode /= NF_NOERR ) call handle_err(rcode)

 deallocate ( tdimValues, hold1D, hold3D )
end subroutine write3D_netcdf
