program cfd                                 ! XZ version

#define DEBUG_LEVEL 1
! Module definitions in cfd_modules.F90
use  grid_info
use  mpi_parameters
use  dimensional_scales
use  boundary_information
use  pde_parameters
use  dependent_variables
use  intermediate_variables
use  counters_flags_etc
use  immersed_boundary
use  user_parameters
use  mpi_times

implicit none

include 'mpif.h'
include '../input/problem_size.h'

!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% ONLY EXECUTABLE STATEMENTS BELOW HERE  %%%%%%%%%%%%%%%%%%%

 include 'start_MPI.h'

 call MPI_Time(0)

!!! LAPACK_FLAG='double'
 LAPACK_FLAG='single'
 call initialize_grids(0)              ! initialize spatial grids at all discretization levels
 call initialize_cfd                   ! basic setup, read input parameters etc.
 call initialize_user                  ! e.g. user can set nontrivial initial conditions


 print*,'loop...'
 
 do istep=istart,iend                ! MAIN TIME STEPPING LOOP
  t = t_start + (istep-istart)*dt    ! dimensionless time

  call save                          ! write restart files {i_restart}

  call rhs                           ! compute rhs for U,v,W & pd neglecting
    ! only pressure gradients

  call time_step                     ! time step w/o pressure gradient => u_star
  
  call divergence                    ! compute "divergence" of u_star -> div_u
  
  call set_rhs_for_pressure          ! set the rhs for pressure equation L[p]=b

  call psolve_xy_periodic            ! solve pressure using xyfft & direct solve in z

  call gradient                      ! compute pressure "gradient"
  
  call project                       ! project u_star onto div-free subspace =>[U,v,W]
  
  call filter                        ! compact spatial filtering(i_filter,alpha)
  
  call set_BC_values                 ! compute u boundary values for faces 1:6
  
  call apply_BCs                     ! impose exact satisfaction of BCs on [U,v,W,pd]
  
  call user_BCs                      ! impose user-defined conditions on [U,v,W,pd]
    
  call write3d_netcdf                ! output 3d fields {i_full_fields}
    
  include 'toggle.h'                 ! toggle counters, indices etc

enddo

 call mpi_barrier(comm,ierr)

 call MPI_Time(1)

 call mpi_finalize(ierr)

end program cfd
