!!!!    modules used in f90, pim, MG version of cfdXZ code   !!!!

!*****************************************************************
module grid_info

! define a grid_object to hold information about each spatial grid
type grid_object
 integer                                ::  n1,n2,n3,locn3		! dimension sizes, locn3=n3/numprocs
 integer                                ::  level,n_mg_levels   ! 0 = finest
 integer     		                ::  n1_it,n2_it,n3_it 	! dimension sizes for U,V in matvec
 real			                ::  dxi, deta, dzeta	! grid spacings
 real, dimension(:,:), allocatable	::  x, z 		 	! spatial variables
 real, dimension(:), allocatable	::  y	 		 	! 
 real, dimension(:,:), allocatable	::  x_xi, x_zeta, z_xi, z_zeta	! grid metrics
 real, dimension(:), allocatable	::  y_eta			! grid metric
 real, dimension(:,:,:,:), allocatable	::  J_matrix	! Jacobian matrix
 real, dimension(:,:), allocatable	::  det_J			! det of Jacobian matrix
 real, dimension(:,:), allocatable	::  Cxx,Czz			! pressure coefficients in XZ/13
 real, dimension(:), allocatable	::  Cyy				! pressure coefficients in Y/2

 real, dimension(:,:,:), allocatable	        ::  v, f, scratch		! solution, rhs  and work vectors
 double precision, dimension(:,:), allocatable	::  ddx, ddy, ddz		! differentiation matrices, banded
 integer, dimension(:), allocatable             ::  piv_dx, piv_dy, piv_dz	! corresponding pivot arrays
 double precision, dimension(:,:), allocatable	::  filt_x, filt_y, filt_z	! filtering matrices, banded
 integer, dimension(:), allocatable             ::  piv_fx, piv_fy, piv_fz	! corresponding pivot arrays
 integer, dimension(13)	                        ::  ipar			! PIM integer parameters
 real, dimension(6)	                        ::  spar                        ! PIM real parameters

end type grid_object

! declare an allocatable array of such objects
type(grid_object), dimension(:),allocatable	:: Grid              ! each Grid(j) contains the above
double precision  	                        :: alpha_1, beta_1      ! for compact scheme matrix
double precision                                :: a1, b1, c1           ! for compact scheme (rhs)
real   		                                :: alpha                ! filtering parameter
real, dimension(:), allocatable		        :: wrk		        ! PIM work vector
real, dimension(:,:,:), allocatable             :: GM_uu, GM_uw, GM_ww  ! metrics for transformed eqns of motion
real					        :: t, dt
character(len=80)				:: grid_file,restriction_method
character(len=7)				:: discrete_scheme             ! compact or FD
character(len=5)				:: relax_precon                ! none or diagl
integer                         :: current_grid_level          ! needed by matvec
real, allocatable,dimension(:)  :: global_Czz,global_Cz,global_det_J

end module grid_info
!*****************************************************************

!*****************************************************************
module integration_weights

! weight coefficients in each direction
  real, allocatable             :: W1(:), W2(:), W3(:)

end module integration_weights
!*****************************************************************

!*****************************************************************
module dependent_variables
! Container for dependent variable arrays
  real, allocatable             :: U(:,:,:),v(:,:,:),W(:,:,:),s1(:,:,:),s2(:,:,:)
  real, allocatable             :: pressure(:,:,:,:)
  real, allocatable             :: s1_bar(:,:), s1_bar_xi(:,:), s1_bar_zeta(:,:)
  real, allocatable             :: s2_bar(:,:), s2_bar_xi(:,:), s2_bar_zeta(:,:)
  real, allocatable             :: rho_bar(:,:),pd(:,:,:)
end module dependent_variables
!*****************************************************************

!*****************************************************************
module intermediate_variables
! Container for intermediate variable arrays
  real, allocatable             :: div_u(:,:,:,:),L_of_p(:,:,:)
  real, allocatable             :: b(:,:,:)
  real, allocatable             :: grad_x(:,:,:)
  real, allocatable             :: grad_y(:,:,:)
  real, allocatable             :: grad_z(:,:,:)
  real, allocatable             :: u_star(:,:,:)
  real, allocatable             :: v_star(:,:,:)
  real, allocatable             :: w_star(:,:,:)
  real, allocatable             :: u_star_old(:,:,:)
  real, allocatable             :: v_star_old(:,:,:)
  real, allocatable             :: w_star_old(:,:,:)
end module intermediate_variables
!*****************************************************************

!*****************************************************************
module rhs_variables
! Container for rhs arrays
  real, allocatable             :: rhs1(:,:,:,:),rhs2(:,:,:,:)
  real, allocatable             :: rhs3(:,:,:,:),rhs4(:,:,:,:)
  real, allocatable             :: rhs5(:,:,:,:)
end module rhs_variables
!*****************************************************************


!*****************************************************************
module pde_parameters
! Container for pde parameter data
  real                           :: Ri,Rot,Re,Pr1,Pr2
  real, allocatable              :: K_m(:,:,:)
end module pde_parameters
!*****************************************************************


!*****************************************************************
module dimensional_scales
! Container for scaling data
  real                           :: delta_rho,rho0
  real                           :: g,nu,kappa1,kappa2,f
  real                           :: kappa,Kh
  real                           :: t_scale,L_scale,U_scale
  real                           :: s1_scale,s2_scale
  real                           :: density_scale,pressure_scale
  real                           :: energy_scale,dissipation_scale
  real                           :: DEL,Nsquared,N0
  real                           :: pi
end module dimensional_scales
!*****************************************************************


!*****************************************************************
module mpi_parameters
! Container for MPI data
  integer                         :: myid,nprocs,comm
  integer                         :: init_derived_type,subslice
  integer                         :: oneslice,twoslice
end module mpi_parameters
!*****************************************************************


!*****************************************************************
module boundary_information
! Container for boundary condition arrays
  integer                     :: BC_flag(5,3)  ! 5 variables, 3 directions
  character(len=80)           :: steady_BCS, immersed_bdry_flag

! boundary type
  character(len=80), allocatable     :: boundary_type1(:,:),boundary_type2(:,:)
  character(len=80), allocatable     :: boundary_type3(:,:),boundary_type4(:,:)
  character(len=80), allocatable     :: boundary_type5(:,:),boundary_type6(:,:)
  character(len=80), allocatable     :: BC_pressure1(:,:),BC_pressure2(:,:)
  character(len=80), allocatable     :: BC_pressure3(:,:),BC_pressure4(:,:)
  character(len=80), allocatable     :: BC_pressure5(:,:),BC_pressure6(:,:)
  character(len=80), allocatable     :: BC_vel1(:,:),BC_vel2(:,:)
  character(len=80), allocatable     :: BC_vel3(:,:),BC_vel4(:,:)
  character(len=80), allocatable     :: BC_vel5(:,:),BC_vel6(:,:)
  character(len=80), allocatable     :: BC_scalar1(:,:,:),BC_scalar2(:,:,:)
  character(len=80), allocatable     :: BC_scalar3(:,:,:),BC_scalar4(:,:,:)
  character(len=80), allocatable     :: BC_scalar5(:,:,:),BC_scalar6(:,:,:)

! boundary values
  real, allocatable                  :: BC_values1(:,:,:),BC_values2(:,:,:)
  real, allocatable                  :: BC_values3(:,:,:),BC_values4(:,:,:)
  real, allocatable                  :: BC_values5(:,:,:),BC_values6(:,:,:)
  real, allocatable                  :: phiY(:,:,:,:,:)  
  real, allocatable                  :: inflow(:,:,:)

end module boundary_information
!*****************************************************************

!*****************************************************************
module counters_flags_etc
! Container for misc scalar data
  integer                         :: istep,istart,iend,nsteps,write3D_counter
  integer                         :: i_filter,i_stat,i_full_fields
  integer                         :: i_mean_pressure,i_restart,save_file
  integer                         :: new,old,MM0,MM1,MM2,MM3
  integer                         :: i,j,k,icount,itmp
  integer                         :: ierr,local_rows,local_cols
  integer                         :: global_rows,global_cols
  real                            :: tol,div_rms,div_tol
  integer                         :: maxits,n_mg_cycles,do_pressure
  integer                         :: V_sweeps_pre, V_sweeps_post
  integer, allocatable            :: my_node_indices(:)
  character(len=80)               :: step_flag,psolve_scheme,closure_flag
  character(len=80)               :: relaxation_method
  character(len=80)               :: ics_flag,rho_bar_flag
  character(len=80)               :: restart_flag,user_subs_flag
  character(len=80)               :: user_rhs_flag,user_bcs_flag
  real                            :: inertial_per,BV_per,t_start
  real                            :: kx_val,ky_val
end module counters_flags_etc
!*****************************************************************
module multigrid_parameters
! Container for various constants related to MG preconditioning
  character(len=80)                  :: mg_preconditioning
  integer                            :: num_mg_levels
  integer                            :: dof,s_width
  integer                            :: nx_coarse,ny_coarse,nz_coarse
  integer                            :: refine_x,refine_y,refine_z
  integer,dimension(:),allocatable   :: n1,n2,n3
end module multigrid_parameters

