subroutine dissipation
#define DEBUG_LEVEL 1
 use counters_flags_etc

 if( closure_flag == 'DNS' ) then
  call diss_DNS
 elseif( closure_flag == 'LES' ) then
! call diss_LES
 endif
end subroutine dissipation

subroutine diss_DNS 
#define DEBUG_LEVEL 1
! Routine to calculate the diffusion terms  in the
! transformed governing equations for a DNS.
! Results are added to the partial results stored in rhs1-4/5.
! UPDATE:  MULTIPLICATIONS BY  DLESS VISCOSITY/DIFFUSIVITY 
!          NOW DONE IN grad2_u_cartesion SO Km Kt CAN BE
!          FUNCTIONS OF Z

 use grid_info
 use dependent_variables
 use rhs_variables
 use pde_parameters
 use counters_flags_etc
 use boundary_information
 use mpi_parameters
 use user_parameters
 use dimensional_scales

 implicit none
 include '../input/problem_size.h'
 real, dimension(:,:,:),allocatable     :: g2_u,g2_v,g2_w
 real, dimension(:,:,:),allocatable     :: data,g2_s1,g2_s2
 real                                   :: viscosity,diffusivity1,diffusivity2,xx
 integer                                :: varid

#if DEBUG_LEVEL >=1
  if(myid==0) write(0,*) 'hello world from diss_DNS'
#endif

 allocate( g2_u(nx,ny,locnz), g2_v(nx,ny,locnz) )
 g2_u(:,:,:) = 0.
 g2_v(:,:,:) = 0.
 allocate( g2_w(nx,ny,locnz), g2_s1(nx,ny,locnz) )
 g2_w(:,:,:) = 0.
 g2_s1(:,:,:) = 0.
 if(num_scalars>1) then
   allocate( g2_s2(nx,ny,locnz) )
   g2_s2(:,:,:) = 0.
 endif
 allocate( data(nx,ny,locnz) )
 data(:,:,:) = 0.

! Compute grad^2(total s1),store result in  data
  varid=4
  do j=1,ny
   data(:,j,:)=s1(:,j,:)+s1_bar(1:nx,1:locnz)
  enddo
  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!call grad2_u(data,g2_rho,4)
  call grad2_u_cartesian(data,g2_s1,varid)

if(num_scalars>1) then
! Compute grad^2(total s2),store result in  data
  varid=5
  do j=1,ny
   data(:,j,:)=s2(:,j,:)+s2_bar(1:nx,1:locnz)
  enddo
  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!call grad2_u(data,g2_rho,4)
  call grad2_u_cartesian(data,g2_s2,varid)
endif

! Compute grad^2(u), store in g2_u
  varid=1
  do j=1,ny              ! u = cartesian x velocity
   data(:,j,:)= Grid(0)%x_xi(1:nx,1:locnz)*U(:,j,:)   &
              + Grid(0)%x_zeta(1:nx,1:locnz)*W(:,j,:) 
  enddo
  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!call grad2_u(data,g2_u,1)
  call grad2_u_cartesian(data,g2_u,varid)

! Compute grad^2(v), store in g2_v
  varid=2
  data=v
  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!call grad2_u(data,g2_v,2)
  call grad2_u_cartesian(data,g2_v,varid)

! Compute grad^2(w), store in g2_w
  varid=3
  do j=1,ny              ! w = cartesian z velocity
   data(:,j,:)= Grid(0)%z_xi(1:nx,1:locnz)*U(:,j,:)    &
              + Grid(0)%z_zeta(1:nx,1:locnz)*W(:,j,:) 
  enddo
  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!call grad2_u(data,g2_w,3)
  call grad2_u_cartesian(data,g2_w,varid)

! Add diffusion terms to the rhs of each equation (3 momentum, 1 density)
! note that U = J_matrix(i,k,1,1)*u + J_matrix(i,k,2,1)*w
! note that W = J_matrix(i,k,1,2)*u + J_matrix(i,k,2,2)*w
! UPDATE VISCOSITY?DIFFUSIVITY ALREADY DONE 
! RATHER THAN REMOVING THE STATEMENTS, JUST SET VALUES TO 1.0

  viscosity = 1.
  diffusivity1 = 1.
  diffusivity2 = 1.
  do j=1,ny

   rhs1(:,j,:,MM0) = rhs1(:,j,:,MM0)+                                  &
                viscosity *( g2_u(:,j,:)*Grid(0)%J_matrix(1:nx,1:locnz,1,1)    &
                            +g2_w(:,j,:)*Grid(0)%J_matrix(1:nx,1:locnz,2,1) )

   rhs2(:,j,:,MM0) = rhs2(:,j,:,MM0) + viscosity * g2_v(:,j,:)

   rhs3(:,j,:,MM0) = rhs3(:,j,:,MM0)+                                  &
                viscosity *( g2_u(:,j,:)*Grid(0)%J_matrix(1:nx,1:locnz,1,2)    &
                            +g2_w(:,j,:)*Grid(0)%J_matrix(1:nx,1:locnz,2,2) )

   rhs4(:,j,:,MM0) = rhs4(:,j,:,MM0) + diffusivity1 * g2_s1(:,j,:)
   
   if(num_scalars>1) rhs5(:,j,:,MM0) = rhs5(:,j,:,MM0) + diffusivity2 * g2_s2(:,j,:)

  enddo

 deallocate( g2_u,g2_v,g2_w,g2_s1,data )
 if(num_scalars>1) deallocate(g2_s2)
end subroutine diss_DNS


subroutine grad2_u(u,g2u,id)
#define DEBUG_LEVEL 1
! This routine will calculate grad^2 u where
! the laplacian is defined with respect to
! the cartesion coordinate system, i.e. wrt x,y and z
! and u is an arbitrary scalar of assumed size(nx,ny,locnz)

 use grid_info
 use boundary_information
 use mpi_parameters
 use counters_flags_etc

 implicit none
 include '../input/problem_size.h'
 integer                               :: flag,id     
 real                                  :: h,end_values(2)
 real, dimension(nx,ny,locnz)          :: u,g2u
 real, dimension(:,:,:,:), allocatable :: work
 allocate ( work(nx,ny,locnz,5) )
 work(:,:,:,:) = 0.

#if DEBUG_LEVEL >=1
  if(myid==0) write(0,*) ' hello world from grad2_u'
#endif

! copy u into output array g2u
  g2u=u

! Compute u_xi, u_eta, u_zeta, store in work 1,2,3
  h=Grid(0)%dxi
  flag=1  ! no assumed end conditions
  end_values=-9999999
  call compact_ddx(g2u,work(1,1,1,1),flag,end_values,nx,ny,locnz)

  h=Grid(0)%deta
  call compact_ddy(g2u,work(1,1,1,2),flag,end_values,nx,ny,locnz)

  h=Grid(0)%dzeta
  call compact_ddz_mpi(g2u,work(1,1,1,3),flag,end_values,nx,ny,locnz)

! Store u_x, u_y, u_z in work 4,2,3 respectively
  do j=1,ny
   do i=1,nx
    do k=1,locnz
     work(i,j,k,4)=work(i,j,k,1)*Grid(0)%J_matrix(i,k,1,1)+work(i,j,k,3)*Grid(0)%J_matrix(i,k,1,2)
     work(i,j,k,3)=work(i,j,k,1)*Grid(0)%J_matrix(i,k,2,1)+work(i,j,k,3)*Grid(0)%J_matrix(i,k,2,2)           
    enddo
   enddo
  enddo
  do j=1,ny
   work(:,j,:,2)=work(:,j,:,2)/Grid(0)%y_eta(j)
  enddo
	
! Compute (u_x)_zeta  (u_x)_xi , store in work 1 and 5 respectively
  h=Grid(0)%dzeta
  call compact_ddz_mpi(work(1,1,1,4),work(1,1,1,1),flag,end_values,nx,ny,locnz)
     
  h=Grid(0)%dxi
  call compact_ddx(work(1,1,1,4),work(1,1,1,5),flag,end_values,nx,ny,locnz)

! Store u_xx in g2u
  do j=1,ny
   g2u(:,j,:)=(work(:,j,:,5)*Grid(0)%J_matrix(1:nx,1:locnz,1,1)    &
              +work(:,j,:,1)*Grid(0)%J_matrix(1:nx,1:locnz,1,2))
  enddo

! Compute (u_z)_zeta  (u_z)_xi , store in work 4 and 5 respectively
  h=Grid(0)%dzeta
  call compact_ddz_mpi(work(1,1,1,3),work(1,1,1,4),flag,end_values,nx,ny,locnz)

  h=Grid(0)%dxi
  call compact_ddx(work(1,1,1,3),work(1,1,1,5),flag,end_values,nx,ny,locnz)

! Add u_zz to g2u
  do j=1,ny
   g2u(:,j,:)=g2u(:,j,:)                                  &
             + work(:,j,:,5)*Grid(0)%J_matrix(1:nx,1:locnz,2,1)   &
             + work(:,j,:,4)*Grid(0)%J_matrix(1:nx,1:locnz,2,2)   
  enddo

! Compute (u_y)_eta   store in work 5
  h=Grid(0)%deta
  call compact_ddy(work(1,1,1,2),work(1,1,1,5),flag,end_values,nx,ny,locnz)

! Add u_yy to g2u
  do j=1,ny
   g2u(:,j,:) = g2u(:,j,:) + work(:,j,:,5)/Grid(0)%y_eta(j)
  enddo

! g2u now has u_xx + u_yy + u_zz

 deallocate ( work )
end subroutine grad2_u


subroutine grad2_u_cartesian(u,g2u,varid)
#define DEBUG_LEVEL 1
! This routine will calculate grad^2 u where
! the laplacian is defined with respect to
! the cartesion coordinate system, i.e. wrt x,y and z
! and u is an arbitrary scalar of assumed size(nx,ny,locnz)
!********* SPECIAL CASE ****************
! assume x_xi and y_eta are constants
!        z_zeta varies only with zeta
!***************************************

 use grid_info
 use boundary_information
 use mpi_parameters
 use counters_flags_etc
 use user_parameters       ! Km_of_z,Kt_of_z,Kh 
                           ! HV_METHOD,HV_RATIO
 use dimensional_scales    ! nu/kappa are scaling factors for visc/diff

 implicit none
 include '../input/problem_size.h'
 real                                  :: r_LU
 integer                               :: flag,varid
 real                                  :: h,end_values(2)
 real, dimension(nx,ny,locnz)          :: u,g2u
 real, dimension(:,:,:,:), allocatable :: work
 allocate ( work(nx,ny,locnz,4) )
 work(:,:,:,:) = 0.

#if DEBUG_LEVEL >=1
  if(myid==0) write(0,*) ' hello world from grad2_u_cartesian'
#endif

!(1) Compute u_xi, u_eta, u_zeta, store in work 1,2,3
  h=Grid(0)%dxi
  flag=1  ! no assumed end conditions
  end_values=-9999999
  call compact_ddx(u,work(1,1,1,1),flag,end_values,nx,ny,locnz)

  h=Grid(0)%deta
  call compact_ddy(u,work(1,1,1,2),flag,end_values,nx,ny,locnz)

  h=Grid(0)%dzeta
  call compact_ddz_mpi(u,work(1,1,1,3),flag,end_values,nx,ny,locnz)
  do j=1,ny
   work(:,j,:,3) = (1./Grid(0)%z_zeta(:,:)) * work(:,j,:,3)
  enddo
  
!(1.5) Multiply by (dimensionless) spatially variable eddy viscosity/diffusivity
  r_LU = 1./(L_scale*U_scale)
  if( HV_METHOD==1 ) then
   if( varid <= 3 ) then
    do i=1,nx
     do j=1,ny
      work(i,j,:,1) = HV_RATIO*r_LU*Km_of_z(:) * work(i,j,:,1)
      work(i,j,:,2) = HV_RATIO*r_LU*Km_of_z(:) * work(i,j,:,2)
      work(i,j,:,3) = r_LU*Km_of_z(:) * work(i,j,:,3)
     enddo
    enddo
   elseif( varid >=4 ) then
    do i=1,nx
     do j=1,ny
      work(i,j,:,1) = HV_RATIO*r_LU*Kt_of_z(:) * work(i,j,:,1)
      work(i,j,:,2) = HV_RATIO*r_LU*Kt_of_z(:) * work(i,j,:,2)
      work(i,j,:,3) = r_LU*Kt_of_z(:) * work(i,j,:,3)
     enddo
    enddo
   endif
  elseif( HV_METHOD==2 ) then
   if( varid <= 3 ) then
    do i=1,nx
     do j=1,ny
      work(i,j,:,1) = r_LU*Kh * work(i,j,:,1)
      work(i,j,:,2) = r_LU*Kh * work(i,j,:,2)
      work(i,j,:,3) = r_LU*Km_of_z(:) * work(i,j,:,3)
     enddo
    enddo
   elseif( varid >=4 ) then
    do i=1,nx
     do j=1,ny
      work(i,j,:,1) = r_LU*Kh * work(i,j,:,1)
      work(i,j,:,2) = r_LU*Kh * work(i,j,:,2)
      work(i,j,:,3) = r_LU*Kt_of_z(:) * work(i,j,:,3)
     enddo
    enddo
   endif
  endif
  

!(2) Differentiate again
  h=Grid(0)%dxi
  end_values=-9999999
  call compact_ddx(work(1,1,1,1),work(1,1,1,4),flag,end_values,nx,ny,locnz)

  h=Grid(0)%deta
  call compact_ddy(work(1,1,1,2),work(1,1,1,1),flag,end_values,nx,ny,locnz)

  h=Grid(0)%dzeta
  call compact_ddz_mpi(work(1,1,1,3),work(1,1,1,2),flag,end_values,nx,ny,locnz)
  do j=1,ny
   work(:,j,:,2) = (1./Grid(0)%z_zeta(:,:)) * work(:,j,:,2)
  enddo

!  HV_RATIO is a user-defined parameter defined as nu_horiz/nu where nu is taken as nu_vertical
  do k=1,locnz
   do j=1,ny
    do i=1,nx
     g2u(i,j,k) = ((1./Grid(0)%x_xi(i,k))**2) * work(i,j,k,4)                      &
                + ((1./Grid(0)%y_eta(j))**2) * work(i,j,k,1)  + work(i,j,k,2)
    enddo
   enddo
  enddo


 deallocate ( work )
end subroutine grad2_u_cartesian
