subroutine init3D_netcdf
#define DEBUG_LEVEL 1
 use mpi_parameters
 use grid_info
 use dependent_variables
 use counters_flags_etc
 use dimensional_scales, length_scale=>L_scale
 implicit none

 include 'netcdf.inc'
 include '../input/problem_size.h'

 character(len=5)    ::  cnum
 character(len=3)    ::  cid
 character(len=80)   ::  filename
 integer             ::  ncid,rcode,ntimes 

!Dimensions and dimension variables
 integer             ::  iid, jid, kid, timeid   ! Dimension ids 
 integer             ::  tdimensionID            ! time dimension variable ids
 integer             ::  jdimensionSIZE(1)         
 integer             ::  tdimensionSIZE(1)       ! 1D arrays containing dimension sizes
   
!1D Variables 
 integer             ::  yVarID,y_etaVarID
 integer             ::  All1DVarSIZE(2)         ! 1D array containing j and k dimension sizes
 integer             ::  start1D(1), count1D(1)  ! 1D arrays defining shape of 1D variable

! x,z grid postion and 2d grid metric matrices
 integer             :: xVarID,zVarID            ! x,z grid position ids
 integer             :: x_xiVarID,x_zetaVarID      
 integer             :: z_xiVarID,z_zetaVarID      
 integer             :: s1_barVarID,s2_barVarID  ! reference scalar ids
 integer             :: All2DVarSIZE(2)	         ! 2D array containing variables size
 integer             :: start2D(2),count2D(2)    ! 2D arrays defining shape of 2D variable

!4D Variables (x,y,z,t)
 integer             ::  s1VarID,s2VarID,uVarID,vVarID,divVarID
 integer             ::  wVarId,pVarID,KmVarID   ! 4D Variable ids
 integer             ::  All4DVarSIZE(4)         ! 4D array containing variables size

!Local arrays for holding data
 real, dimension(:), allocatable      :: jdimValues
 real, dimension(:,:), allocatable    :: hold2D 

 allocate ( jdimValues(ny), hold2D(nx,locnz) )

#if DEBUG_LEVEL >=1
 if(myid==0) write(0,*) 'hello world from init3D_netcdf'
#endif

 ntimes=1  ! Each 3D file will contain 1 time slice.
      
!Create the filename: cfd3D_xxxxx_xxx
 write(unit=cnum,fmt=100) istep
100 format(I5.5)
 write(unit=cid,fmt=101) myid
101 format(I3.3)
 if( numprocs > 1 ) then
  filename='../output/cfd3D_'//cnum//'_'//cid   ! 
 else
  filename='../output/cfd3D_'//cnum
 endif

!Open a new file with write access.
 if(myid .eq. 0 ) write(0,*) ' creating 3d netcdf files: ',cnum
 rcode = NF_CREATE(filename, NF_CLOBBER, ncid)
  if(rcode /= NF_NOERR) call HANDLE_ERR(rcode)

!Dimensions Definition
 rcode = NF_DEF_DIM(ncid, 'idimension', nx, iid)
  if(rcode /= NF_NOERR) call HANDLE_ERR(rcode)
 rcode = NF_DEF_DIM(ncid, 'jdimension', ny, jid)
  if(rcode /= NF_NOERR) call HANDLE_ERR(rcode)
 rcode = NF_DEF_DIM(ncid, 'kdimension', locnz, kid)         ! LOCAL PORTION
  if(rcode /= NF_NOERR) call HANDLE_ERR(rcode)
 rcode = NF_DEF_DIM(ncid, 'timedimension', ntimes, timeid)
  if(rcode /= NF_NOERR) call HANDLE_ERR(rcode)

!Initialize size arrays for spatial dimensions
 jdimensionSIZE(1) = jid
 tdimensionSIZE(1) = timeid

!Define dimension variables
 rcode = NF_DEF_VAR(ncid,'timedimension',NF_FLOAT,1,tdimensionSIZE,tdimensionID)
  if(rcode /= NF_NOERR) call HANDLE_ERR(rcode)
 rcode = NF_PUT_ATT_TEXT(ncid,tdimensionID,'long_name',4,'time')
  if(rcode /= NF_NOERR) call HANDLE_ERR(rcode)
 rcode = NF_PUT_ATT_TEXT(ncid,tdimensionID,'units',7,'seconds')
  if(rcode /= NF_NOERR) call HANDLE_ERR(rcode)

!Define 1D time independent variables
!Initialize size arrays for time independent variables
 All1DVarSIZE(1) = jid    ! y related variables
 All1DVarSIZE(2) = kid    ! x-z related variables

 rcode = NF_DEF_VAR(ncid,'yVar',NF_FLOAT,1,All1DVarSIZE,yVarID)
  if(rcode /= NF_NOERR) call HANDLE_ERR(rcode)
 rcode = NF_PUT_ATT_TEXT(ncid,yVarID,'long_name',1,'y')
  if(rcode /= NF_NOERR) call HANDLE_ERR(rcode)
 rcode = NF_PUT_ATT_TEXT(ncid,yVarID,'units',6,'meters')
  if(rcode /= NF_NOERR) call HANDLE_ERR(rcode)

 rcode = NF_DEF_VAR(ncid,'y_etaVar',NF_FLOAT,1,All1DVarSIZE,y_etaVarID)
  if(rcode /= NF_NOERR) call HANDLE_ERR(rcode)
 rcode = NF_PUT_ATT_TEXT(ncid,yVarID,'long_name',7,'dy/deta')
  if(rcode /= NF_NOERR) call HANDLE_ERR(rcode)
 rcode = NF_PUT_ATT_TEXT(ncid,yVarID,'units',6,'meters')
  if(rcode /= NF_NOERR) call HANDLE_ERR(rcode)


!Define s1_bar and x,z grid position variables
!Initialize size arrays for time independent variables
 All2DVarSIZE(1) = iid
 All2DVarSIZE(2) = kid

 rcode = NF_DEF_VAR(ncid,'T_barVar',NF_FLOAT,2,All2DVarSIZE,s1_barVarID)
  if(rcode /= NF_NOERR) call HANDLE_ERR(rcode)
 rcode = NF_PUT_ATT_TEXT(ncid,s1_barVarID,'long_name',5,'T_bar')
  if(rcode /= NF_NOERR) call HANDLE_ERR(rcode)
 rcode = NF_PUT_ATT_TEXT(ncid,s1_barVarID,'units',4,'degK')
  if(rcode /= NF_NOERR) call HANDLE_ERR(rcode)

if(num_scalars>1) then
 rcode = NF_DEF_VAR(ncid,'Sal_barVar',NF_FLOAT,2,All2DVarSIZE,s2_barVarID)
  if(rcode /= NF_NOERR) call HANDLE_ERR(rcode)
 rcode = NF_PUT_ATT_TEXT(ncid,s2_barVarID,'long_name',5,'S_bar')
  if(rcode /= NF_NOERR) call HANDLE_ERR(rcode)
 rcode = NF_PUT_ATT_TEXT(ncid,s2_barVarID,'units',3,'ppt')
  if(rcode /= NF_NOERR) call HANDLE_ERR(rcode)
endif

 rcode = NF_DEF_VAR(ncid,'xVar',NF_FLOAT,2,All2DVarSIZE,xVarID)
  if(rcode /= NF_NOERR) call HANDLE_ERR(rcode)
 rcode = NF_PUT_ATT_TEXT(ncid,xVarID,'long_name',1,'x')
  if(rcode /= NF_NOERR) call HANDLE_ERR(rcode)
 rcode = NF_PUT_ATT_TEXT(ncid,xVarID,'units',6,'meters')
  if(rcode /= NF_NOERR) call HANDLE_ERR(rcode)

 rcode = NF_DEF_VAR(ncid,'zVar',NF_FLOAT,2,All2DVarSIZE,zVarID)
  if(rcode /= NF_NOERR) call HANDLE_ERR(rcode)
 rcode = NF_PUT_ATT_TEXT(ncid,zVarID,'long_name',1,'z')
  if(rcode /= NF_NOERR) call HANDLE_ERR(rcode)
 rcode = NF_PUT_ATT_TEXT(ncid,zVarID,'units',6,'meters')
  if(rcode /= NF_NOERR) call HANDLE_ERR(rcode)

 rcode = NF_DEF_VAR(ncid,'x_xiVar',NF_FLOAT,2,All2DVarSIZE,x_xiVarID)
  if(rcode /= NF_NOERR) call HANDLE_ERR(rcode)
 rcode = NF_PUT_ATT_TEXT(ncid,x_xiVarID,'long_name',4,'x_xi')
  if(rcode /= NF_NOERR) call HANDLE_ERR(rcode)
 rcode = NF_PUT_ATT_TEXT(ncid,x_xiVarID,'units',6,'meters')
  if(rcode /= NF_NOERR) call HANDLE_ERR(rcode)

 rcode = NF_DEF_VAR(ncid,'z_xiVar',NF_FLOAT,2,All2DVarSIZE,z_xiVarID)
  if(rcode /= NF_NOERR) call HANDLE_ERR(rcode)
 rcode = NF_PUT_ATT_TEXT(ncid,z_xiVarID,'long_name',4,'z_xi')
  if(rcode /= NF_NOERR) call HANDLE_ERR(rcode)
 rcode = NF_PUT_ATT_TEXT(ncid,z_xiVarID,'units',6,'meters')
  if(rcode /= NF_NOERR) call HANDLE_ERR(rcode)

 rcode = NF_DEF_VAR(ncid,'x_zetaVar',NF_FLOAT,2,All2DVarSIZE,x_zetaVarID)
  if(rcode /= NF_NOERR) call HANDLE_ERR(rcode)
 rcode = NF_PUT_ATT_TEXT(ncid,x_zetaVarID,'long_name',6,'x_zeta')
  if(rcode /= NF_NOERR) call HANDLE_ERR(rcode)
 rcode = NF_PUT_ATT_TEXT(ncid,x_zetaVarID,'units',6,'meters')
  if(rcode /= NF_NOERR) call HANDLE_ERR(rcode)

 rcode = NF_DEF_VAR(ncid,'z_zetaVar',NF_FLOAT,2,All2DVarSIZE,z_zetaVarID)
  if(rcode /= NF_NOERR) call HANDLE_ERR(rcode)
 rcode = NF_PUT_ATT_TEXT(ncid,z_zetaVarID,'long_name',6,'z_zeta')
  if(rcode /= NF_NOERR) call HANDLE_ERR(rcode)
 rcode = NF_PUT_ATT_TEXT(ncid,z_zetaVarID,'units',6,'meters')
  if(rcode /= NF_NOERR) call HANDLE_ERR(rcode)

!Define 4D variables
 All4DVarSIZE(1) = iid
 All4DVarSIZE(2) = jid
 All4DVarSIZE(3) = kid    ! LOCAL
 All4DVarSIZE(4) = timeid ! 1 slice

!perturbation temperature = s1Var
 rcode = NF_DEF_VAR(ncid,'TempVar',NF_FLOAT,4,All4DVarSIZE,s1VarID)
  if(rcode /= NF_NOERR) call HANDLE_ERR(rcode)
 rcode = NF_PUT_ATT_TEXT(ncid,s1VarID,'long_name',11,'temperature')
  if(rcode /= NF_NOERR) call HANDLE_ERR(rcode)
 rcode = NF_PUT_ATT_TEXT(ncid,s1VarID,'units',4,'degK')
  if(rcode /= NF_NOERR) call HANDLE_ERR(rcode)

if(num_scalars>1) then
!perturbation salinity = s2Var
 rcode = NF_DEF_VAR(ncid,'SalinityVar',NF_FLOAT,4,All4DVarSIZE,s2VarID)
  if(rcode /= NF_NOERR) call HANDLE_ERR(rcode)
 rcode = NF_PUT_ATT_TEXT(ncid,s2VarID,'long_name',8,'salinity')
  if(rcode /= NF_NOERR) call HANDLE_ERR(rcode)
 rcode = NF_PUT_ATT_TEXT(ncid,s2VarID,'units',3,'ppt')
  if(rcode /= NF_NOERR) call HANDLE_ERR(rcode)
endif

!x velocity = uVar
 rcode = NF_DEF_VAR(ncid,'uVar',NF_FLOAT,4,All4DVarSIZE,uVarID)
  if(rcode /= NF_NOERR) call HANDLE_ERR(rcode)
 rcode = NF_PUT_ATT_TEXT(ncid,uVarID,'long_name',1,'u')
  if(rcode /= NF_NOERR) call HANDLE_ERR(rcode)
 rcode = NF_PUT_ATT_TEXT(ncid,uVarID,'units',3,'m/s')
  if(rcode /= NF_NOERR) call HANDLE_ERR(rcode)

!y velocity = vVar
 rcode = NF_DEF_VAR(ncid,'vVar',NF_FLOAT,4,All4DVarSIZE,vVarID)
  if(rcode /= NF_NOERR) call HANDLE_ERR(rcode)
 rcode = NF_PUT_ATT_TEXT(ncid,vVarID,'long_name',1,'v')
  if(rcode /= NF_NOERR) call HANDLE_ERR(rcode)
 rcode = NF_PUT_ATT_TEXT(ncid,vVarID,'units',3,'m/s')
  if(rcode /= NF_NOERR) call HANDLE_ERR(rcode)

!z velocity = wVar
 rcode = NF_DEF_VAR(ncid,'wVar',NF_FLOAT,4,All4DVarSIZE,wVarID)
  if(rcode /= NF_NOERR) call HANDLE_ERR(rcode)
 rcode = NF_PUT_ATT_TEXT(ncid,wVarID,'long_name',1,'w')
  if(rcode /= NF_NOERR) call HANDLE_ERR(rcode)
 rcode = NF_PUT_ATT_TEXT(ncid,wVarID,'units',3,'m/s')
  if(rcode /= NF_NOERR) call HANDLE_ERR(rcode)

!pressure = pVar
 rcode = NF_DEF_VAR(ncid,'pVar',NF_FLOAT,4,All4DVarSIZE,pVarID)
  if(rcode /= NF_NOERR) call HANDLE_ERR(rcode)
 rcode = NF_PUT_ATT_TEXT(ncid,pVarID,'long_name',8,'pressure')
  if(rcode /= NF_NOERR) call HANDLE_ERR(rcode)
 rcode = NF_PUT_ATT_TEXT(ncid,pVarID,'units',7,'kg/ms^2')
  if(rcode /= NF_NOERR) call HANDLE_ERR(rcode)

!Km = KmVar
 rcode = NF_DEF_VAR(ncid,'KmVar',NF_FLOAT,4,All4DVarSIZE,KmVarID)
  if(rcode /= NF_NOERR) call HANDLE_ERR(rcode)
 rcode = NF_PUT_ATT_TEXT(ncid,KmVarID,'long_name',14,'eddy viscosity')
  if(rcode /= NF_NOERR) call HANDLE_ERR(rcode)
 rcode = NF_PUT_ATT_TEXT(ncid,KmVarID,'units',5,'m^2/s')
  if(rcode /= NF_NOERR) call HANDLE_ERR(rcode)

!div(u) = divVar
 rcode = NF_DEF_VAR(ncid,'divVar',NF_FLOAT,4,All4DVarSIZE,divVarID)
  if(rcode /= NF_NOERR) call HANDLE_ERR(rcode)
 rcode = NF_PUT_ATT_TEXT(ncid,divVarID,'long_name',10,'divergence')
  if(rcode /= NF_NOERR) call HANDLE_ERR(rcode)
 rcode = NF_PUT_ATT_TEXT(ncid,divVarID,'units',3,'1/s')
  if(rcode /= NF_NOERR) call HANDLE_ERR(rcode)

!End define mode
 rcode = NF_ENDDEF(ncid)
  if(rcode /= NF_NOERR) call HANDLE_ERR(rcode)

!Close the file
 rcode = NF_CLOSE(ncid)
  if(rcode /= NF_NOERR) call HANDLE_ERR(rcode)

!**************************************************************
!**************************************************************
!**************************************************************

!Reopen the file and write local portion of time independent data
 rcode = NF_OPEN(filename,NF_WRITE,ncid)
  if(rcode /= NF_NOERR) call HANDLE_ERR(rcode)

 rcode = NF_INQ_VARID(ncid,'xVar',xVarID)
  if(rcode /= NF_NOERR) call HANDLE_ERR(rcode)
 rcode = NF_INQ_VARID(ncid,'yVar',yVarID)
  if(rcode /= NF_NOERR) call HANDLE_ERR(rcode)
 rcode = NF_INQ_VARID(ncid,'zVar',zVarID)
  if(rcode /= NF_NOERR) call HANDLE_ERR(rcode)

 rcode = NF_INQ_VARID(ncid,'x_xiVar',x_xiVarID)
  if(rcode /= NF_NOERR) call HANDLE_ERR(rcode)
 rcode = NF_INQ_VARID(ncid,'x_zetaVar',x_zetaVarID)
  if(rcode /= NF_NOERR) call HANDLE_ERR(rcode)
 rcode = NF_INQ_VARID(ncid,'y_etaVar',y_etaVarID)
  if(rcode /= NF_NOERR) call HANDLE_ERR(rcode)
 rcode = NF_INQ_VARID(ncid,'z_xiVar',z_xiVarID)
  if(rcode /= NF_NOERR) call HANDLE_ERR(rcode)
 rcode = NF_INQ_VARID(ncid,'z_zetaVar',z_zetaVarID)
  if(rcode /= NF_NOERR) call HANDLE_ERR(rcode)

 rcode = NF_INQ_VARID(ncid,'T_barVar',s1_barVarID)
  if(rcode /= NF_NOERR) call HANDLE_ERR(rcode)

if(num_scalars>1) then
 rcode = NF_INQ_VARID(ncid,'Sal_barVar',s2_barVarID)
  if(rcode /= NF_NOERR) call HANDLE_ERR(rcode)
endif

!store y values in temp array
 jdimValues(1:ny) = Grid(0)%y(1:ny)*length_scale
!write values to file
 start1D(1) = 1
 count1D(1) = ny
 rcode = NF_PUT_VARA_REAL(ncid,yVarID,start1D,count1D,jdimValues)
  if(rcode /= NF_NOERR) call HANDLE_ERR(rcode)

!store y_eta values in temp array
 jdimValues(1:ny) = Grid(0)%y_eta(1:ny)*length_scale
 rcode = NF_PUT_VARA_REAL(ncid,y_etaVarID,start1D,count1D,jdimValues)
  if(rcode /= NF_NOERR) call HANDLE_ERR(rcode)

!Initialize size arrays for local xz data
 start2D(1) = 1
 start2D(2) = 1
 count2D(1) = nx
 count2D(2) = locnz 

!Write (dimensional) s1_bar values
!!! hold2D(:,:) = s1_bar(:,:)*s1_scale
 hold2D(:,:) = s1_bar(:,:)
!Write values to file
 rcode = NF_PUT_VARA_REAL(ncid,s1_barVarID,start2D,count2D,hold2D)
  if(rcode /= NF_NOERR) call HANDLE_ERR(rcode)

if(num_scalars>1) then
!Write (dimensional) s2_bar values
!!! hold2D(:,:) = s2_bar(:,:)*s2_scale
 hold2D(:,:) = s2_bar(:,:)
!Write values to file
 rcode = NF_PUT_VARA_REAL(ncid,s2_barVarID,start2D,count2D,hold2D)
  if(rcode /= NF_NOERR) call HANDLE_ERR(rcode)
endif

!Write x grid point values
 hold2D(:,:) = Grid(0)%x(:,:)*length_scale
!Write values to file
 rcode = NF_PUT_VARA_REAL(ncid,xVarID,start2D,count2D,hold2D)
  if(rcode /= NF_NOERR) call HANDLE_ERR(rcode)

!Write z grid point values
 hold2D(:,:) = Grid(0)%z(:,:)*length_scale
!Write values to file
 rcode = NF_PUT_VARA_REAL(ncid,zVarID,start2D,count2D,hold2D)
  if(rcode /= NF_NOERR) call HANDLE_ERR(rcode)

!Write x_xi grid point values
 hold2D(:,:) = Grid(0)%x_xi(:,:)*length_scale
!Write values to file
 rcode = NF_PUT_VARA_REAL(ncid,x_xiVarID,start2D,count2D,hold2D)
  if(rcode /= NF_NOERR) call HANDLE_ERR(rcode)

!Write x_zeta grid point values
 hold2D(:,:) = Grid(0)%x_zeta(:,:)*length_scale
!Write values to file
 rcode = NF_PUT_VARA_REAL(ncid,x_zetaVarID,start2D,count2D,hold2D)
  if(rcode /= NF_NOERR) call HANDLE_ERR(rcode)

!Write z_xi grid point values
 hold2D(:,:) = Grid(0)%z_xi(:,:)*length_scale
!Write values to file
 rcode = NF_PUT_VARA_REAL(ncid,z_xiVarID,start2D,count2D,hold2D)
  if(rcode /= NF_NOERR) call HANDLE_ERR(rcode)

!Write z_zeta grid point values
 hold2D(:,:) = Grid(0)%z_zeta(:,:)*length_scale
!Write values to file
 rcode = NF_PUT_VARA_REAL(ncid,z_zetaVarID,start2D,count2D,hold2D)
  if(rcode /= NF_NOERR) call HANDLE_ERR(rcode)


!Close netcdf file
 rcode = NF_CLOSE(ncid)
  if(rcode /= NF_NOERR) call HANDLE_ERR(rcode)
       
 deallocate ( jdimValues, hold2D )
end subroutine init3D_netcdf

subroutine handle_err(status)
 include 'netcdf.inc'
 integer :: status
 if ( status /= NF_NOERR ) then
  write(0,*) NF_STRERROR(status)
  write(0,*) 'stopping after netcdf error'
  STOP
 endif
end subroutine handle_err
