subroutine initialize_grids(testflag)     ! testflag=-999 -> explicitly set simple cartesian grid
#define DEBUG_LEVEL 3

use grid_info
use immersed_boundary
use mpi_parameters
use boundary_information
use counters_flags_etc, only:maxits
implicit none

integer			:: h,i,j,k,testflag
integer         	:: lda,loclen,basis,pret,stopt,ierr
real            	:: tol,xx
real,allocatable        :: scratch1(:),scratch2(:)
                  

include 'mpif.h'
include '../input/problem_size.h'
include '../input/grid_data.h'      ! data statement specifying grid_dimensions(0:nlevels-1,3)

#if DEBUG_LEVEL >= 1
 if(myid == 0 ) then
  write(0,*) ' '
  write(0,*) ' '
  write(0,*) 'hello world from initialize_grids, nlevels=',nlevels
 endif
#endif

if( myid==0 ) open(1,file='../output/grid_level_data')
allocate( Grid(0:nlevels-1) )

do h=0,nlevels-1

  Grid(h)%n_mg_levels = nlevels
  Grid(h)%level=h
  Grid(h)%n1 = grid_dimensions(h,1)
  Grid(h)%n2 = grid_dimensions(h,2)
  Grid(h)%n3 = grid_dimensions(h,3)
  Grid(h)%locn3 = grid_dimensions(h,3)/numprocs

  Grid(h)%dxi    = 1.0/float( Grid(h)%n1-1 )
  Grid(h)%deta  = 1.0/float( Grid(h)%n2-1 )
  Grid(h)%dzeta = 1.0/float( Grid(h)%n3-1 )

#if DEBUG_LEVEL >= 1
 if(myid == 0 ) then
  write(0,*) '**** GRID LEVEL ',h
  write(0,*) '    n1:  ',Grid(h)%n1
  write(0,*) '    n2:  ',Grid(h)%n2
  write(0,*) '    n3:  ',Grid(h)%n3
  write(0,*) '    locn3:  ',Grid(h)%locn3
  write(0,*) '    dxi:  ',Grid(h)%dxi
  write(0,*) '    deta:  ',Grid(h)%deta
  write(0,*) '    dzeta:  ',Grid(h)%dzeta
  write(0,*) '***************************'
  write(0,*) ' '
 endif 
#endif

!allocate arrays for grid level h

  allocate( Grid(h)%x( Grid(h)%n1, Grid(h)%locn3 ), Grid(h)%z( Grid(h)%n1 , Grid(h)%locn3 )  )
  allocate( Grid(h)%x_xi( Grid(h)%n1, Grid(h)%locn3 ), Grid(h)%x_zeta( Grid(h)%n1, Grid(h)%locn3 )  )
  allocate( Grid(h)%z_xi( Grid(h)%n1, Grid(h)%locn3 ), Grid(h)%z_zeta( Grid(h)%n1, Grid(h)%locn3 )  )
  allocate( Grid(h)%y( Grid(h)%n2 )  , Grid(h)%y_eta( Grid(h)%n2 )  )

  allocate( Grid(h)%J_matrix( Grid(h)%n1, Grid(h)%locn3, 2, 2 )  )
  allocate( Grid(h)%det_J( Grid(h)%n1, Grid(h)%locn3 )  )

  allocate( Grid(h)%Cxx( Grid(h)%n1, Grid(h)%locn3 ), Grid(h)%Czz( Grid(h)%n1 , Grid(h)%locn3 )  )
  allocate( Grid(h)%Cyy( Grid(h)%n2 ) )

! allocate( Grid(h)%H( Grid(h)%n1, Grid(h)%n2 )  )
  allocate( Grid(h)%v( Grid(h)%n1, Grid(h)%n2, Grid(h)%locn3 )   )
  allocate( Grid(h)%f( Grid(h)%n1, Grid(h)%n2, Grid(h)%locn3 )   )
  allocate( Grid(h)%scratch( Grid(h)%n1, Grid(h)%n2, Grid(h)%locn3 )   )

  allocate( Grid(h)%ddx(7, Grid(h)%n1), Grid(h)%filt_x(7, Grid(h)%n1) )
  allocate(Grid(h)%ddy(7, Grid(h)%n2), Grid(h)%filt_y(7, Grid(h)%n2) )
  allocate(Grid(h)%ddz(7, Grid(h)%n3), Grid(h)%filt_z(7, Grid(h)%n3) )
  allocate(Grid(h)%piv_dx(Grid(h)%n1), Grid(h)%piv_fx(Grid(h)%n1) )
  allocate(Grid(h)%piv_dy(Grid(h)%n2), Grid(h)%piv_fy(Grid(h)%n2) )
  allocate(Grid(h)%piv_dz(Grid(h)%n3), Grid(h)%piv_fz(Grid(h)%n3) )

! setup & factor differentiation arrays 
  call setup_compact_schemes(h)

! if h=0, read in fine grid data
! Each processor reads and stores its local portion of the grid data:
 if( h == 0 ) then
! these next three arrays are needed in the xyperiodic z stretched case
      allocate( global_Czz( nz ) )
      allocate( global_Cz( nz ) )
      allocate( global_det_J( nz ) )
      allocate( GM_uu(nx,locnz,2), GM_uw(nx,locnz,2), GM_ww(nx,locnz,2) )
       if( testflag == -999 ) then
!        include '../input/test_grid.h'  ! simple cartesian grid, Poisson coefficients
       else  ! read grid data from input file
        do i=0,numprocs-1
         call read_grid_data('../input/grid_coords',i)
        enddo
        call MPI_BARRIER(comm,ierr) ! all procesors have now read the file
       endif
!      for cartesian grids, set values identically to zero
       Grid(0)%z_xi=0.0
       Grid(0)%x_zeta=0.0

! Make Czz,Det_J and d/dzeta Czz available globally
  allocate( scratch1(nz),scratch2(nz) )
  global_Cz(1:nz)=0.
!Czz
  scratch1(1:nz)=0.0
  scratch1( myid*locnz+1:(myid+1)*locnz ) = Grid(0)%Czz(1,1:locnz)  ! assume no variation with i
  call MPI_ALLREDUCE(scratch1,scratch2,nz,MPI_REAL,MPI_SUM,comm,ierr) 
  global_Czz=scratch2
!det_J
  scratch1(1:nz)=0.0
  scratch1( myid*locnz+1:(myid+1)*locnz ) = Grid(0)%det_J(1,1:locnz)  ! assume no variation with i
  call MPI_ALLREDUCE(scratch1,scratch2,nz,MPI_REAL,MPI_SUM,comm,ierr) 
  global_det_J=scratch2

  xx=1./(24.*Grid(0)%dzeta)
  do k=3,nz-2
   global_Cz(k)=( 2.*global_Czz(k-2) -16.*global_Czz(k-1) + 16.*global_Czz(k+1) -2.*global_Czz(k+2) )*xx
  enddo
  global_Cz(1)=(-50.*global_Czz(1) +96.*global_Czz(2) - 72.*global_Czz(3) +32.*global_Czz(4) -6.*global_Czz(5) )*xx
  global_Cz(2)=(-6.*global_Czz(1) -20.*global_Czz(2) + 36.*global_Czz(3) -12.*global_Czz(4) +2.*global_Czz(5) )*xx
  global_Cz(nz)=(50.*global_Czz(nz) -96.*global_Czz(nz-1) +    &
                 72.*global_Czz(nz-2) -32.*global_Czz(nz-3) +6.*global_Czz(nz-4) )*xx
  global_Cz(nz-1)=(6.*global_Czz(nz) +20.*global_Czz(nz-1)     &
                   - 36.*global_Czz(nz-2) +12.*global_Czz(nz-3) -2.*global_Czz(nz-4) )*xx

  if(myid==0) then
   write(0,*)'**********   k, Czz, Cz   ***************'
   do k=1,nz
    write(0,*) k,global_Czz(k),global_Cz(k)
   enddo
   write(0,*)'*****************************************'
  endif

 endif  ! end statements exclusive to h=0 

enddo    ! end loop over h levels

! restrict (h-1) fine grid data to coarse grids (h)
do h=1,nlevels-1   ! loop over coarse grids
 if(myid==0) then
  write(0,*) '  Restricting grid data to level ',h,' of 0:',nlevels-1
  write(0,*) ' Lx,Ly,Lz : ',Grid(0)%x(nx,1),Grid(0)%y(ny),Grid(0)%z(1,locnz)
 endif
 restriction_method = 'injection'
 call restrictXZ( Grid(h-1)%z, Grid(h)%z, h-1 )
 call restrictXZ( Grid(h-1)%z_xi, Grid(h)%z_xi, h-1 )
 call restrictXZ( Grid(h-1)%z_zeta, Grid(h)%z_zeta, h-1 )
 call restrictXZ( Grid(h-1)%x, Grid(h)%x, h-1 )
 call restrictXZ( Grid(h-1)%x_xi, Grid(h)%x_xi, h-1 )
 call restrictXZ( Grid(h-1)%x_zeta, Grid(h)%x_zeta, h-1 )
 call restrictXZ( Grid(h-1)%Cxx, Grid(h)%Cxx, h-1 )
 call restrictXZ( Grid(h-1)%Czz, Grid(h)%Czz, h-1 )
 call restrictXZ( Grid(h-1)%det_J, Grid(h)%det_J, h-1 )
 call restrictXZ( Grid(h-1)%J_matrix(1,1,1,1), Grid(h)%J_matrix(1,1,1,1), h-1 )
 call restrictXZ( Grid(h-1)%J_matrix(1,1,1,2), Grid(h)%J_matrix(1,1,1,2), h-1 )
 call restrictXZ( Grid(h-1)%J_matrix(1,1,2,1), Grid(h)%J_matrix(1,1,2,1), h-1 )
 call restrictXZ( Grid(h-1)%J_matrix(1,1,2,2), Grid(h)%J_matrix(1,1,2,2), h-1 )

 call restrictY( Grid(h-1)%y, Grid(h)%y, h-1 )
 call restrictY( Grid(h-1)%y_eta, Grid(h)%y_eta, h-1 )
 call restrictY( Grid(h-1)%Cyy, Grid(h)%Cyy, h-1 )

enddo
 if(myid==0) then
  write(0,*) '  Done Restricting grid data '
 endif


#if DEBUG_LEVEL >= 2

 do h=0,nlevels-2

 write(1,*) ' RESTRICTION TEST  Z direction, from level',h
 do j=1,Grid(h)%n2
  Grid(h)%v(:,j,:)=Grid(h)%z   ! 3D version of z
 enddo
   call restrict( Grid(h)%v, Grid(h+1)%v, h )  ! 
 do i=1,Grid(h+1)%locn3
  write(1,*) i,(myid*Grid(h+1)%locn3+i-1.)*Grid(h+1)%dzeta, Grid(h+1)%z(1,i), &
               Grid(h+1)%Czz(1,i),Grid(h+1)%det_J(1,i)
 enddo
 write(1,*) ' '

 write(1,*) ' RESTRICTION TEST  X direction, from level',h
 do j=1,Grid(h)%n2
  Grid(h)%v(:,j,:)=Grid(h)%x   ! 3D version of x
 enddo
   call restrict( Grid(h)%v, Grid(h+1)%v, h )  ! 
 do i=1,Grid(h+1)%n1
  write(1,*) i,(i-1.)*Grid(h+1)%dxi, Grid(h+1)%x(i,1)
 enddo
 write(1,*) ' '

 write(1,*) ' RESTRICTION TEST  Y direction, from level',h
 do i=1,Grid(h)%n1
 do k=1,Grid(h)%locn3
  Grid(h)%v(i,:,k)=Grid(h)%y   ! 3D version of y
 enddo
 enddo
   call restrict( Grid(h)%v, Grid(h+1)%v, h )  ! 
 do i=1,Grid(h+1)%n2
  write(1,*) i,(i-1.)*Grid(h+1)%deta, Grid(h+1)%y(i),Grid(h+1)%Cyy(i)
 enddo
 write(1,*) ' '

 enddo

#endif


#if DEBUG_LEVEL >= 2
 do h=0,nlevels-2

 write(1,*) ' INTERPOLATION TEST  Z DIRECTION, to level',h
 do j=1,Grid(h+1)%n2
  Grid(h+1)%v(:,j,:)=Grid(h+1)%z
 enddo
   call interpolate( Grid(h+1)%v, Grid(h)%v, h )
 do k=1,Grid(h)%locn3
  if(myid==0) write(1,*) k,(Grid(h)%locn3*myid+k-1.)*Grid(h)%dzeta, Grid(h)%v(2,3,k), &
               Grid(h)%Czz(1,k),Grid(h)%det_J(1,k)
 enddo
 if(myid==0) write(1,*) ' '
 
 if(myid==0) write(1,*) ' INTERPOLATION TEST  X DIRECTION to level',h
 do j=1,Grid(h+1)%n2
  Grid(h+1)%v(:,j,:)=Grid(h+1)%x
 enddo
   call interpolate( Grid(h+1)%v, Grid(h)%v, h )
 do k=1,Grid(h)%n1
  if(myid==0) write(1,*) k,(k-1.)*Grid(h)%dxi, Grid(h)%v(k,1,1)
 enddo
 if(myid==0) write(1,*) ' '
 
 if(myid==0) write(1,*) ' INTERPOLATION TEST  Y DIRECTION to level',h
 do i=1,Grid(h+1)%n1
 do k=1,Grid(h+1)%locn3
  Grid(h+1)%v(i,:,k)=Grid(h+1)%y
 enddo
 enddo
   call interpolate( Grid(h+1)%v, Grid(h)%v, h )
 do k=1,Grid(h)%n2
  if(myid==0) write(1,*) k,(k-1.)*Grid(h)%deta, Grid(h)%v(1,k,1),Grid(h)%Cyy(k)
 enddo
 if(myid==0) write(1,*) ' '
 
 enddo
 close(1)
#endif

restriction_method='full_weighting'

end subroutine initialize_grids
