subroutine restart
#define DEBUG_LEVEL 1
!routine to restart simulation from save file

 use counters_flags_etc
 use dependent_variables
 use rhs_variables
 use intermediate_variables
 use mpi_parameters
 use boundary_information
 use grid_info
 use dimensional_scales

 implicit none
 include '../input/problem_size.h'
 character(len=5)                  :: step_number
 character(len=4)                  :: processor_id
 character(len=80)                 :: filename
 integer                           :: pid_in,ix,iy,iz,iAB,ntimes

  real                      ::        unoise(nx,ny,locnz)
  real                      ::        vnoise(nx,ny,locnz)

!open save/restart file
 write(step_number,'(I5.5)') save_file
 write(processor_id,'(I4.4)') myid
 filename='../output/save_'//step_number//'_'//processor_id
 open(1,file=filename,status='old',form='unformatted')

!control parameters
 read(1) pid_in,istart,t_start,old,new,MM3,MM2,MM1,MM0,ntimes
  iend = istart+nsteps
  if( ntimes > AB_order ) ntimes=AB_order     ! read/store only what will be used
  if( AB_order == 2 .and. ntimes == 2 ) step_flag='AB2'
  if( AB_order == 3 .and. ntimes == 2 ) step_flag='AB2'
  if( AB_order == 3 .and. ntimes == 3 ) step_flag='AB3'
  if( AB_order == 4 .and. ntimes == 4 ) step_flag='AB4'
 if( pid_in /= myid ) then
  write(0,*) 'myid doesnt match id in save/restart file',myid,pid_in
  stop
 endif
 if(istart /= save_file) then
  if(myid.eq.0) then
   write(0,*) 'FALSE RESTART; istart,save_file=',istart,save_file
  endif
  stop
 endif

!field variables
 do iz=1,locnz
  read(1)((U(ix,iy,iz),ix=1,nx),iy=1,ny)
  read(1)((v(ix,iy,iz),ix=1,nx),iy=1,ny)
  read(1)((W(ix,iy,iz),ix=1,nx),iy=1,ny)
  read(1)((s1(ix,iy,iz),ix=1,nx),iy=1,ny)
  if(num_scalars>1) read(1)((s2(ix,iy,iz),ix=1,nx),iy=1,ny)
 enddo

!!!     call noise(unoise,vnoise)
!!!     do j=1,ny
!!!     U(:,j,:) = U(:,j,:) + unoise(:,j,:) / (U_scale*Grid(0)%x_xi(:,:))
!!!     V(:,j,:) = V(:,j,:) + vnoise(:,j,:) / U_scale
!!!     enddo

!right-hand sides
 do iAB=1,ntimes
  do iz=1,locnz
   read(1)((rhs1(ix,iy,iz,iAB),ix=1,nx),iy=1,ny)
   read(1)((rhs2(ix,iy,iz,iAB),ix=1,nx),iy=1,ny)
   read(1)((rhs3(ix,iy,iz,iAB),ix=1,nx),iy=1,ny)
   read(1)((rhs4(ix,iy,iz,iAB),ix=1,nx),iy=1,ny)
   if(num_scalars>1) read(1)((rhs5(ix,iy,iz,iAB),ix=1,nx),iy=1,ny)
  enddo
 enddo

!pressure
 do k=1,2
  do iz=1,locnz
   read(1)((pressure(ix,iy,iz,k),ix=1,nx),iy=1,ny)
  enddo
 enddo

!divergence arrays
 do iz=1,locnz
  read(1)((u_star(ix,iy,iz),ix=1,nx),iy=1,ny)
  read(1)((v_star(ix,iy,iz),ix=1,nx),iy=1,ny)
  read(1)((w_star(ix,iy,iz),ix=1,nx),iy=1,ny)
 enddo
 do k=1,2
  read(1)((u_star_old(iy,iz,k),iy=1,ny),iz=1,locnz)
  read(1)((v_star_old(ix,iz,k),ix=1,nx),iz=1,locnz)
  read(1)((w_star_old(ix,iy,k),ix=1,nx),iy=1,ny)
  do iz=1,locnz
   read(1)((div_u(ix,iy,iz,k),ix=1,nx),iy=1,ny)
  enddo
 enddo

!ambient scalar arrays
 do iz=1,locnz
  read(1)(s1_bar(ix,iz),ix=1,nx)
  read(1)(s1_bar_xi(ix,iz),ix=1,nx)
  read(1)(s1_bar_zeta(ix,iz),ix=1,nx)
  if(num_scalars>1) read(1)(s2_bar(ix,iz),ix=1,nx)
  if(num_scalars>1) read(1)(s2_bar_xi(ix,iz),ix=1,nx)
  if(num_scalars>1) read(1)(s2_bar_zeta(ix,iz),ix=1,nx)
  read(1)(rho_bar(ix,iz),ix=1,nx)
 enddo

!y radiation bcs data
 do i=1,6
  do j=1,3
   do k=1,4
    do iz=1,locnz
     read(1)(phiY(ix,iz,i,j,k),ix=1,nx)
    enddo
   enddo
  enddo
 enddo

!Y inflow conditions
 do i=1,4
  do iz=1,locnz
   read(1)(inflow(ix,iz,i),ix=1,nx)
  enddo
 enddo

 close(1)
 write3D_counter = 1
end subroutine restart



subroutine save
#define DEBUG_LEVEL 1
!routine to save control and flow variables for restart

 use counters_flags_etc
 use dependent_variables
 use rhs_variables
 use intermediate_variables
 use mpi_parameters
 use boundary_information
 use grid_info

 implicit none
 include '../input/problem_size.h'
 character(len=5)                  :: step_number
 character(len=4)                  :: processor_id
 character(len=80)                 :: filename
 integer                           :: pid_in,ix,iy,iz,iAB

! Execute routine only at designated time steps
 if( istep==istart ) return
!!! if( istep==istart  .and. istart.ne.0 ) return
 if( mod(istep,i_restart) /= 0 ) return

#if DEBUG_LEVEL >=1
 write(0,*) 'hello world from subroutine save'
#endif

!open save/restart file
 write(step_number,'(I5.5)') istep
 write(processor_id,'(I4.4)') myid
 filename='../output/save_'//step_number//'_'//processor_id
 open(1,file=filename,status='new',form='unformatted')

!control parameters
 write(1) myid,istep,t,old,new,MM3,MM2,MM1,MM0,AB_order

!field variables
 do iz=1,locnz
  write(1)((U(ix,iy,iz),ix=1,nx),iy=1,ny)
  write(1)((v(ix,iy,iz),ix=1,nx),iy=1,ny)
  write(1)((W(ix,iy,iz),ix=1,nx),iy=1,ny)
  write(1)((s1(ix,iy,iz),ix=1,nx),iy=1,ny)
  if(num_scalars>1) write(1)((s2(ix,iy,iz),ix=1,nx),iy=1,ny)
 enddo

!right-hand sides
 do iAB=1,AB_order
  do iz=1,locnz
   write(1)((rhs1(ix,iy,iz,iAB),ix=1,nx),iy=1,ny)
   write(1)((rhs2(ix,iy,iz,iAB),ix=1,nx),iy=1,ny)
   write(1)((rhs3(ix,iy,iz,iAB),ix=1,nx),iy=1,ny)
   write(1)((rhs4(ix,iy,iz,iAB),ix=1,nx),iy=1,ny)
   if(num_scalars>1) write(1)((rhs5(ix,iy,iz,iAB),ix=1,nx),iy=1,ny)
  enddo
 enddo

!pressure
 do k=1,2
  do iz=1,locnz
   write(1)((pressure(ix,iy,iz,k),ix=1,nx),iy=1,ny)
  enddo
 enddo

!divergence arrays
 do iz=1,locnz
  write(1)((u_star(ix,iy,iz),ix=1,nx),iy=1,ny)
  write(1)((v_star(ix,iy,iz),ix=1,nx),iy=1,ny)
  write(1)((w_star(ix,iy,iz),ix=1,nx),iy=1,ny)
 enddo
 do k=1,2
  write(1)((u_star_old(iy,iz,k),iy=1,ny),iz=1,locnz)
  write(1)((v_star_old(ix,iz,k),ix=1,nx),iz=1,locnz)
  write(1)((w_star_old(ix,iy,k),ix=1,nx),iy=1,ny)
  do iz=1,locnz
   write(1)((div_u(ix,iy,iz,k),ix=1,nx),iy=1,ny)
  enddo
 enddo

!ambient scalar arrays
 do iz=1,locnz
  write(1)(s1_bar(ix,iz),ix=1,nx)
  write(1)(s1_bar_xi(ix,iz),ix=1,nx)
  write(1)(s1_bar_zeta(ix,iz),ix=1,nx)
  if(num_scalars>1) write(1)(s2_bar(ix,iz),ix=1,nx)
  if(num_scalars>1) write(1)(s2_bar_xi(ix,iz),ix=1,nx)
  if(num_scalars>1) write(1)(s2_bar_zeta(ix,iz),ix=1,nx)
  write(1)(rho_bar(ix,iz),ix=1,nx)
 enddo

!y radiation bcs data
 do i=1,6
  do j=1,3
   do k=1,4
    do iz=1,locnz
     write(1)(phiY(ix,iz,i,j,k),ix=1,nx)
    enddo
   enddo
  enddo
 enddo

!Y inflow conditions
 do i=1,4
  do iz=1,locnz
   write(1)(inflow(ix,iz,i),ix=1,nx)
  enddo
 enddo

 close(1)

end subroutine save
