subroutine psolve
#define DEBUG_LEVEL 1
!Solve the elliptic equation for pressure subject to appropriate
!boundary conditions. Interior and boundary differential operators
!are defined in subroutine matvec.
 
use mpi_parameters
use grid_info
use dependent_variables,    only: pressure   ! soln vector
use intermediate_variables, only: b          ! rhs vector
use counters_flags_etc
use boundary_information
implicit none
include '../input/problem_size.h'
include 'mpif.h'

real                    :: xx,mean_value

#if DEBUG_LEVEL >=1
   if(myid==0) then
    write(0,*) 'hello world from solve_for_pressure'
   endif
#endif

! MULTIGRID SOLVER: Krylov smoothers, compact discrete operators   ****TESTS WORK WELL****
  current_grid_level=0
  discrete_scheme='compact'
  relax_precon='diagl'

  if( istep>=istart ) then
   call mg_solve( b, pressure, xx )   ! any dummy for last arg
  else
   call relax( pressure, b, maxits, current_grid_level ) ! refine soln
  endif

! For x&y periodicity, we solved for the horiz. transform of P,
! i.e. for P(kx,ky,zeta). Now we need to inverse transform
! to recover P(xi,eta,zeta).
 if( boundary_type2(1,1)=='periodic' .and. boundary_type4(1,1)=='periodic') then
  call xy_fft(pressure,pressure,nx,ny,locnz,-1)   ! -1 for inverse direction
  pressure(nx,:,:,1)=pressure(1,:,:,1)   ! carry 2nd periodic endpoints explicitly
  pressure(:,ny,:,1)=pressure(:,1,:,1)   ! in the rest of the cfd algorithm
 endif

!Store updated total pressure in pressure(:,:,:,2).
 if( psolve_scheme .eq. 'total_pressure' ) then
   pressure(:,:,:,2)=pressure(:,:,:,1)
 elseif( psolve_scheme .eq. 'change_in_pressure' ) then
  pressure(:,:,:,2)=pressure(:,:,:,2)+pressure(:,:,:,1)
 endif

!Remove the mean pressure
 xx=sum( pressure(:,:,:,2) )/float(nx*ny*locnz)   ! local mean value
 call MPI_ALLREDUCE(xx,mean_value,1,MPI_REAL,MPI_SUM,comm,ierr)
 pressure(:,:,:,2)=pressure(:,:,:,2)-mean_value/numprocs   ! subtract global mean

end subroutine psolve

subroutine pdumr
! dummy subroutine, can be used as an arg to the PIM routines
end subroutine pdumr
