subroutine psolve_xy_periodic
#define DEBUG_LEVEL 1
!Solve the elliptic equation for pressure subject to appropriate
!boundary conditions. Interior and boundary differential operators
!are defined in subroutine matvec.
 
use mpi_parameters
use grid_info
use dependent_variables,    only: pressure   ! soln vector
use intermediate_variables, only: b          ! rhs vector
use counters_flags_etc
use boundary_information
use user_parameters,        only: LAPACK_FLAG
implicit none
include '../input/problem_size.h'
include 'mpif.h'

!!!double precision                    :: pi,xx,mean_value,kx_max,ky_max,kmax,kstar
!!!double precision                    :: XF,YF,bsum,lambda2
!!!double precision, allocatable       :: kx(:),ky(:)
!!!double precision, allocatable       :: soln(:),tmp(:),C3(:)
!!!double precision, allocatable       :: L_star(:,:)
real                    :: pi,xx,mean_value,kx_max,ky_max,kmax,kstar
real                    :: XF,YF,bsum,lambda2
real, allocatable       :: kx(:),ky(:)
real, allocatable       :: soln(:),tmp(:),C3(:)
real, allocatable       :: L_star(:,:)

real, allocatable                   :: b_col(:,:,:),p_col(:,:,:)
real, allocatable                   :: work(:,:,:)
integer                             :: locnx,N,nrhs,ldb,ii,row
integer, allocatable                :: ipiv(:)

#if DEBUG_LEVEL >=1
   if(myid==0) then
    write(0,*) 'hello world from solve_for_pressure: xy periodic case'
   endif
#endif

current_grid_level=0

!(1) exchange data so that each processor has columns of b~ --> b_col(1:locnx,1:ny,1:nz)
  locnx=(nx-1)/numprocs
  allocate( b_col(locnx,ny,nz),p_col(locnx,ny,nz) )   ! don't keep extra x endpt in col storage
  b_col(:,:,:) = 0.
  p_col(:,:,:) = 0.
  call slabs_to_columns(b,b_col)
  p_col(:,:,:)=0.
  
!(2) each processor compute global arrays kx&ky 
  N=nx-1                 ! nx includes both boundary points, N only 1
  pi=4.*atan(1.)
  allocate(kx(N+1))
  kx(:) = 0.
  do i=1,N/2+1
   kx(i)=2.*pi*(i-1.)    ! FFTW real to half-complex storage
  enddo
  do i=2,N/2
   kx(N-i+2)=kx(i)       ! FFTW real to half-complex storage
  enddo
  kx(N+1)=0.             ! extra location, convenient for alignment of arrays
  kx_max=2.*pi*(N/2.)


  N=ny-1                 ! ny includes both boundary points, N only 1
  allocate(ky(N+1))
  ky(:) = 0.
  do j=1,N/2+1
   ky(j)=2.*pi*(j-1.)    ! FFTW real to half-complex storage
  enddo
  do j=2,N/2
   ky(N-j+2)=ky(j)       ! FFTW real to half-complex storage
  enddo
  ky(N+1)=0.             ! extra location, convenient for alignment of arrays
  ky_max=2.*pi*(N/2.)

  allocate( C3(nz) )    ! global integration weights for zeta direction
  C3(:) = 0.
  if( nz < 9 ) then
   write(0,*) 'nz too small for integration scheme in solve_for_xypressure'
   stop
  endif
  C3(5:nz-4)=1.0
  C3(1)=17./48.
  C3(2)=59./48.
  C3(3)=43./48.
  C3(4)=49./48.
  C3(nz)=17./48.
  C3(nz-1)=59./48.
  C3(nz-2)=43./48.
  C3(nz-3)=49./48.

!(3) each processor  allocate storage for L_star, a FD approximation to L
  allocate( ipiv(nz+1), L_star(nz+1,nz+1) )
  ipiv(:) = 0
  L_star(:,:) = 0.
  allocate( soln(nz),tmp(nz+1) )
  soln(:) = 0.
  tmp(:) = 0.
  nrhs=1
  ldb=nz+1   ! leading dimension of B, i.e. # of rows in augmented rhs vector

!(4) loop over kx,ky space and iteratively solve the perturbed problems
  do i=1,locnx
   ii=i+myid*locnx       ! global kx indexing variable
   do j=1,ny-1

    bsum = sum( b_col(i,j,1:nz)*C3(1:nz) )   ! integral of b dzeta (/dzeta)
    lambda2=  (kx(ii))**2 * (1./Grid(0)%x_xi(1,1)**2)  &
             +(ky(j))**2 * (1./Grid(0)%y_eta(1)**2) 

!   rhs of matrix eqn
    tmp(1:nz)=b_col(i,j,1:nz)

!   Augment the rhs for compatability condition
    if( ii==1 .and. j==1 ) then
     tmp(nz+1)=0.
    else
     tmp(nz+1)=-bsum/lambda2   ! augmented rhs, compatability condition
    endif

    L_star(:,:)=0.

!   set diagonal elements

    do row=3,nz-2
     xx = (global_Czz(row)/global_det_J(row) )*(1./(60*Grid(0)%dzeta**2)) 
     L_star(row,row-2)=-5.*xx
     L_star(row,row-1)=80.*xx
     L_star(row,row)=-150.*xx
     L_star(row,row+1)=80.*xx
     L_star(row,row+2)=-5.*xx
    enddo
!  1st row
    row=1
    xx = (global_Czz(row)/global_det_J(row) )*(1./(60*Grid(0)%dzeta**2)) 
    L_star(row,1)=-(725./3.)*xx
    L_star(row,2)=280.*xx
    L_star(row,3)=-30.*xx
    L_star(row,4)=-(40./3.)*xx
    L_star(row,5)=5.*xx
!  2nd row
    row=2
    xx = (global_Czz(row)/global_det_J(row) )*(1./(60*Grid(0)%dzeta**2)) 
    L_star(row,1)=(290./3.)*xx
    L_star(row,2)=-180.*xx
    L_star(row,3)=90.*xx
    L_star(row,4)=-(20./3.)*xx
!  last row
    row=nz
    xx = (global_Czz(row)/global_det_J(row) )*(1./(60*Grid(0)%dzeta**2)) 
    L_star(nz,nz)=-(725./3.)*xx
    L_star(nz,nz-1)=280.*xx
    L_star(nz,nz-2)=-30.*xx
    L_star(nz,nz-3)=-(40./3.)*xx
    L_star(nz,nz-4)=5.*xx
!  2nd last row
    row=nz-1
    xx = (global_Czz(row)/global_det_J(row) )*(1./(60*Grid(0)%dzeta**2)) 
    L_star(nz-1,nz)=(290./3.)*xx
    L_star(nz-1,nz-1)=-180.*xx
    L_star(nz-1,nz-2)=90.*xx
    L_star(nz-1,nz-3)=-(20./3.)*xx

!Add in the term for horizontal derivatives
    do row=1,nz
     L_star(row,row)=L_star(row,row) - lambda2
    enddo

! add in the terms for  a*b_zeta*p_zeta
! (1st & nz rows don't get anything: p_zeta=0 by BCs, or use approx form, no difference)
    do row=3,nz-2
     xx = (1./(global_det_J(row)))*global_Cz(row)/(24.*Grid(0)%dzeta)  
     L_star(row,row-2)=L_star(row,row-2)+(2.)*xx
     L_star(row,row-1)=L_star(row,row-1)-(16.)*xx
     L_star(row,row+1)=L_star(row,row+1)+(16.)*xx
     L_star(row,row+2)=L_star(row,row+2)-(2.)*xx
    enddo
!  1st row
    row=1
    xx = (1./(global_det_J(row)))*global_Cz(row)/(24.*Grid(0)%dzeta)  
    L_star(row,1)=L_star(row,1)-(50)*xx
    L_star(row,2)=L_star(row,2)+(96.)*xx
    L_star(row,3)=L_star(row,3)-(72.)*xx
    L_star(row,4)=L_star(row,4)+(32.)*xx
    L_star(row,5)=L_star(row,5)-(6.)*xx
!  2nd row
    row=2
    xx = (1./(global_det_J(row)))*global_Cz(row)/(24.*Grid(0)%dzeta)  
    L_star(row,1)=L_star(row,1)-(68./3.)*xx
    L_star(row,2)=L_star(row,2)+(12.)*xx
    L_star(row,3)=L_star(row,3)+(12.)*xx
    L_star(row,4)=L_star(row,4)-(4./3.)*xx
!  nz-1 row
    row=nz-1
    xx = (1./(global_det_J(row)))*global_Cz(row)/(24.*Grid(0)%dzeta) 
    L_star(row,nz)=L_star(row,nz)+(28./3.)*xx
    L_star(row,nz-1)=L_star(row,nz-1)+(12.)*xx
    L_star(row,nz-2)=L_star(row,nz-2)-(20.)*xx
    L_star(row,nz-3)=L_star(row,nz-3)+(8./3.)*xx
!  nz row
    row=nz
    xx = (1./(global_det_J(row)))*global_Cz(row)/(24.*Grid(0)%dzeta)  
    L_star(row,nz)=L_star(row,nz)+(50)*xx
    L_star(row,nz-1)=L_star(row,nz-1)-(96.)*xx
    L_star(row,nz-2)=L_star(row,nz-2)+(72.)*xx
    L_star(row,nz-3)=L_star(row,nz-3)-(32.)*xx
    L_star(row,nz-4)=L_star(row,nz-4)+(6.)*xx

!   Augmented system for unique soln with Neumann BCs:
    L_star(nz+1,1:nz)=C3   ! extra row: compatability condition
    L_star(1:nz,nz+1)=1.   ! xtra col : perturb rhs such that compatability can be satisfied

!   solve the augmented system
    if( LAPACK_FLAG == 'double' ) then
!!!     call DGESV(nz+1,nrhs,L_star,nz+1,ipiv,tmp,ldb,ierr)
      write(0,*) 'LAPACK_FLAG = double instead of single'
      stop
    elseif( LAPACK_FLAG == 'single' ) then
     call SGESV(nz+1,nrhs,L_star,nz+1,ipiv,tmp,ldb,ierr)
    endif

    soln(1:nz)=tmp(1:nz)  ! xGESV overwrites the rhs vector w/ the soln

    p_col(i,j,1:nz)=soln(1:nz)   ! save soln vectors in p_col

   enddo
  enddo

 deallocate( soln,tmp,C3,kx,ky )


!(5) do the data exchanges so that each processor gets slabs of p --> work(1:nx,1:ny,1:locnz)
 allocate( work(nx,ny,locnz) )
 work(:,:,:) = 0.
 call columns_to_slabs(p_col,work)

! Now we need to inverse transform the pressure solution
! to recover P(xi,eta,zeta).
  call xy_fft(work,pressure,nx,ny,locnz,-1)   ! -1 for inverse direction

!Store updated total pressure in pressure(:,:,:,2).
 if( psolve_scheme .eq. 'total_pressure' ) then
   pressure(:,:,:,2)=pressure(:,:,:,1)
 elseif( psolve_scheme .eq. 'change_in_pressure' ) then
  pressure(:,:,:,2)=pressure(:,:,:,2)+pressure(:,:,:,1)
 endif

!release the MPI derived data types  (created in slabs_to_columns)
 call MPI_BARRIER(comm,ierr)
 call MPI_TYPE_FREE(oneslice,ierr)
 call MPI_TYPE_FREE(twoslice,ierr)
 call MPI_TYPE_FREE(subslice,ierr)
 deallocate(b_col,p_col,ipiv,L_star,work)
end subroutine psolve_xy_periodic


 subroutine slabs_to_columns(slab,col)
! In the cfd code, data is partitioned across processors in equal
! sub-blocks in dimension 3, i.e. in horizontal slabs. It is sometimes
! convenient to exchange data so that it is partitioned along dimension
! 1, i.e. in vertical columns. This routine redistributes the data
! array "slab" into column orientation "col".

  use mpi_parameters
  use boundary_information
  include '../input/problem_size.h'
  include 'mpif.h'

  integer              :: tag,ierr,status_array(MPI_STATUS_SIZE,2*numprocs)
  integer              :: dest,source,istart,kstart,reqs(2*numprocs)
  integer              :: sizeofreal,j
  integer              :: iproc,numwords,m1,m2,m3,locnx
  parameter( m1=nx-1,m2=ny,m3=locnz,locnx=m1/numprocs )
  real                 :: slab(nx,ny,locnz) 
  real                 :: col(locnx,m2,nz)    ! condensed, xtra endpts excluded
  real, allocatable    :: work(:)

  if( boundary_type2(1,1) .ne. 'periodic' .or.  boundary_type2(1,1) .ne. 'periodic' ) then
   write(0,*) 'myid,boundary_type2(1,1) = ',myid,boundary_type2(1,1)
   write(0,*) '  USING PERIODIC ENDPOINT LOGIC IN SLABS_TO_COLUMNS W/O PERIODIC BCS !!!'
   stop
  endif

! Create a derived data type for ease of swaps
  call MPI_TYPE_EXTENT(MPI_REAL,sizeofreal,ierr)
! data type for 1d section of basic data slice: m1/nprocs
  call MPI_TYPE_VECTOR(locnx,1,1,MPI_REAL,oneslice,ierr)
! data type for 2d section of basic data slice: m2=ny 1d sections
  call MPI_TYPE_HVECTOR(m2,1,nx*sizeofreal,oneslice,twoslice,ierr)  ! count,blocklength,stride,old_type
! data type for 3d section of basic data slice:m3=nz/numprocs 2d sections
  call MPI_TYPE_HVECTOR(m3,1,nx*ny*sizeofreal,twoslice,subslice,ierr)
  call MPI_TYPE_COMMIT(subslice,ierr)
  call MPI_BARRIER(comm,ierr)
  allocate( work(nx*ny*locnz*2) )
  work(:) = 0.
  call MPI_BUFFER_ATTACH(work,nx*ny*locnz*16,ierr)

!*************   DO THE DATA EXCHANGES  *****************************
!  Ship portions of my local data "slab" needed by other nodes (incl myself)
!  and recieve the remote data required to construct my local column "col"

!  Post non-blocking receives first, establishing buffers for the sends to use.
      do iproc=0,numprocs-1    ! iproc is the processor I will exchange data with
       source=iproc
       kstart=(iproc*locnz)+1         ! start index for receiving data col
       numwords=locnx*m2*locnz        ! # of real words to store in col w/ each send
       tag = 1
       j=iproc+1    ! counter for message request label
       call MPI_IRECV(col(1,1,kstart),numwords,MPI_REAL,source,tag,comm,reqs(j),ierr)
      enddo
      call MPI_BARRIER(comm,ierr)  ! make sure all receive posted before any sends are executed

!  Post non-blocking sends
      do iproc=0,numprocs-1    ! iproc is the processor I will exchange data with
       dest=iproc
       istart=(iproc*locnx)+1         ! start index for sending data slab
       numwords=1  ! i.e. 1 word of type subslice
       tag = 1
       j=numprocs+iproc+1  ! counter for message request label
       call MPI_IBSEND(slab(istart,1,1),numwords,subslice,dest,tag,comm,reqs(j),ierr)
      enddo

!     make sure all communications are done
      call MPI_WAITALL(2*numprocs,reqs,status_array,ierr)

  call MPI_BARRIER(comm,ierr)
  call MPI_BUFFER_DETACH(work,nx*ny*locnz*16,ierr)
  deallocate( work )
 end subroutine slabs_to_columns


 subroutine columns_to_slabs(col,slab)
! In the cfd code, data is partitioned across processors in equal
! sub-blocks in dimension 3, i.e. in horizontal slabs. It is sometimes
! convenient to exchange data so that it is partitioned along dimension
! 1, i.e. in vertical columns. This routine redistributes the data
! array "col" into slab orientation "slab".

  use mpi_parameters
  use boundary_information
  include '../input/problem_size.h'
  include 'mpif.h'

  integer              :: tag,ierr,status_array(MPI_STATUS_SIZE,2*numprocs)
  integer              :: dest,source,istart,kstart,reqs(2*numprocs)
  integer              :: sizeofreal,j
  integer              :: iproc,numwords,m1,m2,m3,locnx
  parameter( m1=nx-1,m2=ny,m3=locnz,locnx=m1/numprocs )
  real                 :: slab(nx,ny,locnz)
  real                 :: col(locnx,m2,nz)  ! condensed, xtra endpts excluded
  real, allocatable    :: work(:)

  if( boundary_type2(1,1) .ne. 'periodic' .or.  boundary_type2(1,1) .ne. 'periodic' ) then
   write(0,*) '  USING PERIODIC ENDPOINT LOGIC IN SLABS_TO_COLUMNS W/O PERIODIC BCS !!!'
   stop
  endif

!*************   DO THE DATA EXCHANGES  *****************************
!     Ship portions of my local data "column" col needed by other nodes
!     and recieve remote data required to construct my local data
!     "slab" .

  allocate( work(nx*ny*locnz*2) )
  work(:) = 0.
  call MPI_BUFFER_ATTACH(work,nx*ny*locnz*16,ierr)

!  Post non-blocking receives first, establishing buffers for the sends to use.
     do iproc=0,numprocs-1    ! iproc is the processor I will exchange data with
      source=iproc
      istart=(iproc*locnx)+1      ! start index for recieving slab data 
      numwords=1   ! i.e. 1 word of type subslice
      tag = 2
      j=iproc+1
      call MPI_IRECV(slab(istart,1,1),numwords,subslice,source,tag,comm,reqs(j),ierr)
     enddo
     call MPI_BARRIER(comm,ierr)

!  Post non-blocking sends
     do iproc=0,numprocs-1    ! iproc is the processor I will exchange data with
      dest=iproc
      kstart=(iproc*locnz)+1      ! start index for sending col data 
      numwords=locnx*m2*locnz     ! # of real words to send from col w/ each send
      tag = 2
      j=numprocs+iproc+1
      call MPI_IBSEND(col(1,1,kstart),numwords,MPI_REAL,dest,tag,comm,reqs(j),ierr)
     enddo

!     make sure all communications are done
      call MPI_WAITALL(2*numprocs,reqs,status_array,ierr)

!     nx&ny locations meaningless in F space, explicitly zero
      slab(nx,:,:)=0.0
      slab(:,ny,:)=0.0
  call MPI_BARRIER(comm,ierr)
  call MPI_BUFFER_DETACH(work,nx*ny*locnz*16,ierr)
  deallocate( work )
 end subroutine columns_to_slabs
