subroutine update_ibdry_rhs
#define DEBUG_LEVEL 2

  use grid_info
  use mpi_parameters
  use immersed_boundary
  use counters_flags_etc
  use boundary_information
  use dependent_variables    !!!!, only:pd,rho_bar
  use rhs_variables
  use pde_parameters
  use user_parameters
  use intermediate_variables
  implicit none

  include '../input/problem_size.h'
  include 'mpif.h'

  real, allocatable,dimension(:,:)		:: Uvals,vvals,Wvals
  real, allocatable,dimension(:,:)		:: rhovals
  real, allocatable,dimension(:,:)		:: rho_x,rho_z
  real, allocatable,dimension(:,:)		:: xgrid,zgrid,Y2,tmpsum
  integer					:: i1,i2,k1,k2,n1,n2,src,dest,tag,nreqs,ii,root
  integer   			                :: status_array(MPI_STATUS_SIZE,48),req(48)
  real						:: xloc,zloc,ans,ans2,dn,dnn,h,V0
  real, allocatable,dimension(:)		:: tmp

#if DEBUG_LEVEL >= 1
  if(myid==0) then
   write(0,*) ' '
   write(0,*) ' '
   write(0,*) 'hello world from update_ibdry'
  endif
#endif

!allocate space for extended data in xz planes
 allocate( Uvals(nx,-2:locnz+3), vvals(nx,-2:locnz+3) )
 allocate( Wvals(nx,-2:locnz+3), rhovals(nx,-2:locnz+3) )
 allocate( rho_x(nx,-2:locnz+3), rho_z(nx,-2:locnz+3) )
 allocate( xgrid(nx,-2:locnz+3), zgrid(nx,-2:locnz+3) )
 allocate( tmp(nx) )

! use local arrays for holding z extended x&z grid data
 xgrid(:,1:locnz)=Grid(0)%x(:,:)
 xgrid(:,-2:0)=Grid(0)%x(:,1:3)   ! all the same
 xgrid(:,locnz+1:locnz+3)=Grid(0)%x(:,1:3)

 h=( Grid(0)%z(1,2) - Grid(0)%z(1,1) )  ! WE ARE ASSUMING UNIFORM Z GRID
 zgrid(:,1:locnz)=Grid(0)%z(:,:)
 zgrid(:,0)=Grid(0)%z(:,1) - h
 zgrid(:,-1)=Grid(0)%z(:,1) - 2*h
 zgrid(:,-2)=Grid(0)%z(:,1) - 3*h
 zgrid(:,locnz+1)=Grid(0)%z(:,locnz) + h
 zgrid(:,locnz+2)=Grid(0)%z(:,locnz) + 2*h
 zgrid(:,locnz+3)=Grid(0)%z(:,locnz) + 3*h

!All processors initialize time level MM0 data to zero so we can use MPI_ALLREDUCE/ADD
!after the owners calculate the required values
 ibdry_data(:,:,MM0)=0.0
 ibdry_integrals(:,:,MM0)=0.0
 ibdry_rhs(:,:)=0.0


!  LOOP THROUGH EACH Y PLANE
 do j=1,ny


! store total density
  rhovals(:,1:locnz) = pd(:,j,1:locnz) + rho_bar(:,1:locnz) 
  rhovals(:,-2:0)=0
  rhovals(:,locnz+1:locnz+3)=0

! similarly for velocity
  Uvals(:,1:locnz) = u(:,j,1:locnz) 
  Uvals(:,-2:0)=0
  Uvals(:,locnz+1:locnz+3)=0
  vvals(:,1:locnz) = v(:,j,1:locnz) 
  vvals(:,-2:0)=0
  vvals(:,locnz+1:locnz+3)=0
  Wvals(:,1:locnz) = w(:,j,1:locnz) 
  Wvals(:,-2:0)=0
  Wvals(:,locnz+1:locnz+3)=0


 if( numprocs > 1 ) then
  tag=1
  ii=1
  
  if( myid < numprocs-1 ) then   ! receive k>locnz values from N neighbor
   src=myid+1
   do k=locnz+1,locnz+3
    call MPI_IRECV(Uvals(1,k),nx,MPI_REAL,src,tag,comm,req(ii),ierr)
    ii=ii+1
    call MPI_IRECV(vvals(1,k),nx,MPI_REAL,src,tag,comm,req(ii),ierr)
    ii=ii+1
    call MPI_IRECV(Wvals(1,k),nx,MPI_REAL,src,tag,comm,req(ii),ierr)
    ii=ii+1
    call MPI_IRECV(rhovals(1,k),nx,MPI_REAL,src,tag,comm,req(ii),ierr)
    ii=ii+1
   enddo
  endif

  if( myid > 0 ) then            ! receive k<1 values from S neighbor
   src=myid-1
   do k=-2,0
    call MPI_IRECV(Uvals(1,k),nx,MPI_REAL,src,tag,comm,req(ii),ierr)
    ii=ii+1
    call MPI_IRECV(vvals(1,k),nx,MPI_REAL,src,tag,comm,req(ii),ierr)
    ii=ii+1
    call MPI_IRECV(Wvals(1,k),nx,MPI_REAL,src,tag,comm,req(ii),ierr)
    ii=ii+1
    call MPI_IRECV(rhovals(1,k),nx,MPI_REAL,src,tag,comm,req(ii),ierr)
    ii=ii+1
   enddo
  endif

  if( myid < numprocs-1 ) then   ! send uppermost values to N neighbor
   dest=myid+1
   do k=locnz-2,locnz
    call MPI_ISEND(u(1,j,k),nx,MPI_REAL,dest,tag,comm,req(ii),ierr)
    ii=ii+1
    call MPI_ISEND(v(1,j,k),nx,MPI_REAL,dest,tag,comm,req(ii),ierr)
    ii=ii+1
    call MPI_ISEND(w(1,j,k),nx,MPI_REAL,dest,tag,comm,req(ii),ierr)
    ii=ii+1
    tmp(1:nx)=pd(:,j,k) + rho_bar(:,k)
    call MPI_ISEND(tmp(1),nx,MPI_REAL,dest,tag,comm,req(ii),ierr)
    ii=ii+1
   enddo
  endif

  if( myid > 0 ) then            ! send lowermost values to S neighbor
   dest=myid-1
   do k=1,3
    call MPI_ISEND(u(1,j,k),nx,MPI_REAL,dest,tag,comm,req(ii),ierr)
    ii=ii+1
    call MPI_ISEND(v(1,j,k),nx,MPI_REAL,dest,tag,comm,req(ii),ierr)
    ii=ii+1
    call MPI_ISEND(w(1,j,k),nx,MPI_REAL,dest,tag,comm,req(ii),ierr)
    ii=ii+1
    tmp(1:nx)=pd(:,j,k) + rho_bar(:,k)
    call MPI_ISEND(tmp(1),nx,MPI_REAL,dest,tag,comm,req(ii),ierr)
    ii=ii+1
   enddo
  endif

!  make sure all communications are done
   nreqs=ii-1
   call MPI_WAITALL(nreqs,req,status_array,ierr)

 endif  ! end numprocs > 1 logic


  call MPI_BARRIER(comm,ierr)
! compute approximations to rho_x and rho_z
  h = Grid(0)%x_xi(1,1)*Grid(0)%dxi    ! x indep of zeta
  do i=3,nx-2
   rho_x(i,:) = ( 2*rhovals(i-2,:) - 16*rhovals(i-1,:) + 16* rhovals(i+1,:) - 2*rhovals(i+2,:) )/(24.*h)
  enddo
  rho_x(1,:) = ( rhovals(2,:) - rhovals(1,:) )/h
  rho_x(2,:) = ( rhovals(3,:) - rhovals(1,:) )/(2.*h)
  rho_x(nx,:) = ( rhovals(nx,:) - rhovals(nx-1,:) )/h
  rho_x(nx-1,:) = ( rhovals(nx,:) - rhovals(nx-2,:) )/(2.*h)

  h = Grid(0)%z_zeta(1,1)*Grid(0)%dzeta    ! z indep of xi
  do k=0,locnz+1
   rho_z(:,k) = ( 2*rhovals(:,k-2) - 16*rhovals(:,k-1) + 16* rhovals(:,k+1) - 2*rhovals(:,k+2) )/(24.*h)
  enddo
  rho_z(:,-1) = ( rhovals(:,0) - rhovals(:,-1) )/h
  rho_z(:,-2) = ( rhovals(:,-1) - rhovals(:,-2) )/h
  rho_z(:,locnz+2) = ( rhovals(:,locnz+2) - rhovals(:,locnz+1) )/h
  rho_z(:,locnz+3) = ( rhovals(:,locnz+3) - rhovals(:,locnz+2) )/h



! loop through each immersed boundary point
  do i=(j-1)*n_j+1, j*n_j    ! indices for the jth plane


  if( ibdry_owner(i)==myid ) then  ! update imersed bdry data for pt i
   xloc = ibdry_locations(1,i) 
   zloc = ibdry_locations(2,i)   

!  interpolate grid values to immersed boundary point
!  compute 2d splines locally
   i1=local_region(1,i)
   i2=local_region(2,i)
   k1=local_region(5,i) - myid*locnz
   k2=local_region(6,i) - myid*locnz
   if(k1 < -2 ) k1=-2                    ! use only local extended data for interpolation
   if(k2 > locnz+3 ) k2=locnz+3

   n1=i2-i1+1
   n2=k2-k1+1
   allocate( Y2(n1,n2) )

   call splie2(xgrid(i1:i2,1),zgrid(1,k1:k2),Uvals(i1:i2,k1:k2),n1,n2,Y2)
   call splin2(xgrid(i1:i2,1),zgrid(1,k1:k2),Uvals(i1:i2,k1:k2),Y2,n1,n2,xloc,zloc,ans)
   ibdry_data(1,i,MM0)=(ans-0.)  ! i.e. target BC of U=0


   call splie2(xgrid(i1:i2,1),zgrid(1,k1:k2),vvals(i1:i2,k1:k2),n1,n2,Y2)
   call splin2(xgrid(i1:i2,1),zgrid(1,k1:k2),vvals(i1:i2,k1:k2),Y2,n1,n2,xloc,zloc,ans)
   ibdry_data(2,i,MM0)= ans - (-V0)  ! i.e. target BC of v=-V0  ( v=pert from V0)

   call splie2(xgrid(i1:i2,1),zgrid(1,k1:k2),Wvals(i1:i2,k1:k2),n1,n2,Y2)
   call splin2(xgrid(i1:i2,1),zgrid(1,k1:k2),Wvals(i1:i2,k1:k2),Y2,n1,n2,xloc,zloc,ans)
   ibdry_data(3,i,MM0)=(ans)

!  overwrite rhovals with d/dn rho
   rhovals(i1:i2,k1:k2) = ibdry_normals(1,i)*rho_x(i1:i2,k1:k2) + ibdry_normals(2,i)*rho_z(i1:i2,k1:k2)

   call splie2(xgrid(i1:i2,1),zgrid(1,k1:k2),rhovals(i1:i2,k1:k2),n1,n2,Y2)
   call splin2(xgrid(i1:i2,1),zgrid(1,k1:k2),rhovals(i1:i2,k1:k2),Y2,n1,n2,xloc,zloc,ans)

   dn = abs( ibdry_normals(1,i)*Grid(0)%x_xi(1,1)*Grid(0)%dxi   &
      + ibdry_normals(2,i)*Grid(0)%z_zeta(1,1)*Grid(0)%dzeta)

   ibdry_data(4,i,MM0) = ( ans )   ! ( d/dn rho )

   deallocate( Y2 )

!  update the time integrals at each immersed boundary point
   if( istep==istart ) then
    ibdry_integrals(:,i,:)=0.0
   elseif( istep==istart+1 ) then
    do k=1,4
     ibdry_integrals(k,i,MM0)=ibdry_integrals(k,i,MM1)   &
     + (dt/2.)*( ibdry_data(k,i,MM1) + ibdry_data(k,i,MM0) )
    enddo
   elseif( istep>=istart+2 ) then
    do k=1,4
     ibdry_integrals(k,i,MM0)=ibdry_integrals(k,i,MM2)    &
     + (dt/3.)*( ibdry_data(k,i,MM2) + 4.*ibdry_data(k,i,MM1) + ibdry_data(k,i,MM0) )
    enddo
   endif
   
!  update the rhs values for each immersed boundary point
   do k=1,3
    ibdry_rhs(k,i) = -xalpha*ibdry_integrals(k,i,MM0) - xbeta*ibdry_data(k,i,MM0)
   enddo

   ibdry_rhs(4,i) = -delta_3 * ibdry_data(4,i,MM0)/(3*dt)

!   SPECIAL TREATMENT NEAR CORNERS  !!!!!!!!!!!!!!
   if( i - (j-1)*n_j >= 23 .and. i -(j-1)*n_j <= 25 ) ibdry_rhs(4,i) = 0.   ! dont alter just above corners
   if( i - (j-1)*n_j >= 538 .and. i -(j-1)*n_j <= 542 ) ibdry_rhs(4,i) = 0.

   endif  ! end logic for owner of immersed boundary point

  enddo  ! end loop through each immersed boundary point

 enddo   ! GOTO NEXT Y PLANE

!write(0,*) ' ALL IBDRY DATA COMPUTED, JUST NEED TO SHARE  ',myid,ny,n_j
 call MPI_BARRIER(comm,ierr)

! Share the computed values by doing an elementwise ALLREDUCE/ADD , only owners' data is nonzero
  allocate( tmpsum(4,ny*n_j) )

 if( myid==1 ) then
  open(1,file='../output/ibdry_rhs0',position='append')
  write(1,*)istep, '******************************************'
  do i=1,n_j
   write(1,*) i,ibdry_data(1,i,MM0),ibdry_data(2,i,MM0),ibdry_data(3,i,MM0),ibdry_data(4,i,MM0)
  enddo
  write(1,*)'******************************************'
  close(1)
 endif

   call MPI_ALLREDUCE(ibdry_data(1,1,MM0),tmpsum,4*ny*n_j,MPI_REAL,MPI_SUM,comm,ierr)
   ibdry_data(:,:,MM0)=tmpsum(:,:)

   call MPI_ALLREDUCE(ibdry_integrals(1,1,MM0),tmpsum,4*ny*n_j,MPI_REAL,MPI_SUM,comm,ierr)
   ibdry_integrals(:,:,MM0)=tmpsum(:,:)

   call MPI_ALLREDUCE(ibdry_rhs(1,1),tmpsum,4*ny*n_j,MPI_REAL,MPI_SUM,comm,ierr)
   ibdry_rhs(:,:)=tmpsum(:,:)

 call MPI_BARRIER(comm,ierr)

 if( myid==1 ) then
  open(1,file='../output/ibdry_rhs1',position='append')
  write(1,*)istep, '******************************************'
  do i=1,n_j
   write(1,*) i,ibdry_rhs(1,i),ibdry_rhs(2,i),ibdry_rhs(3,i),ibdry_rhs(4,i)
  enddo
  write(1,*)'******************************************'
  close(1)
 endif


! some along-boundary, near-neighbor smoothing
!do j=1,ny
! do k = (j-1)*n_j + n_j/2, j*n_j - 1
!  ibdry_rhs(:,k) = ( ibdry_rhs(:,k-1) + 2*ibdry_rhs(:,k) + ibdry_rhs(:,k+1) )/4.
! enddo
! do k = (j-1)*n_j+ n_j/2 - 1,(j-1)*n_j + 2,-1
!  ibdry_rhs(:,k) = ( ibdry_rhs(:,k-1) + 2*ibdry_rhs(:,k) + ibdry_rhs(:,k+1) )/4.
! enddo
!enddo


 deallocate( Uvals, vvals )
 deallocate( Wvals, rhovals )
 deallocate( rho_x, rho_z )
 deallocate( xgrid, zgrid )
 deallocate( tmp, tmpsum  )

 call MPI_BARRIER(comm,ierr)
end subroutine update_ibdry_rhs


!*********************************

      SUBROUTINE splie2(x1a,x2a,ya,m,n,y2a)
      INTEGER m,n,NN
      REAL x1a(m),x2a(n),y2a(m,n),ya(m,n)
      PARAMETER (NN=100)
!U    USES spline
      INTEGER j,k
      REAL y2tmp(NN),ytmp(NN)
      do 13 j=1,m
        do 11 k=1,n
          ytmp(k)=ya(j,k)
11      continue
        call spline(x2a,ytmp,n,1.e30,1.e30,y2tmp)
        do 12 k=1,n
          y2a(j,k)=y2tmp(k)
12      continue
13    continue
      return
      END
      SUBROUTINE splin2(x1a,x2a,ya,y2a,m,n,x1,x2,y)
      INTEGER m,n,NN
      REAL x1,x2,y,x1a(m),x2a(n),y2a(m,n),ya(m,n)
      PARAMETER (NN=100)
!U    USES spline,splint
      INTEGER j,k
      REAL y2tmp(NN),ytmp(NN),yytmp(NN)
      do 12 j=1,m
        do 11 k=1,n
          ytmp(k)=ya(j,k)
          y2tmp(k)=y2a(j,k)
11      continue
        call splint(x2a,ytmp,y2tmp,n,x2,yytmp(j))
12    continue
      call spline(x1a,yytmp,m,1.e30,1.e30,y2tmp)
      call splint(x1a,yytmp,y2tmp,m,x1,y)
      return
      END
      SUBROUTINE spline(x,y,n,yp1,ypn,y2)
      INTEGER n,NMAX
      REAL yp1,ypn,x(n),y(n),y2(n)
      PARAMETER (NMAX=500)
      INTEGER i,k
      REAL p,qn,sig,un,u(NMAX)
      if (yp1.gt..99e30) then
        y2(1)=0.
        u(1)=0.
      else
        y2(1)=-0.5
        u(1)=(3./(x(2)-x(1)))*((y(2)-y(1))/(x(2)-x(1))-yp1)
      endif
      do 11 i=2,n-1
        sig=(x(i)-x(i-1))/(x(i+1)-x(i-1))
        p=sig*y2(i-1)+2.
        y2(i)=(sig-1.)/p
        u(i)=(6.*((y(i+1)-y(i))/(x(i+1)-x(i))-(y(i)-y(i-1))/(x(i)-x(i-1)))/(x(i+1)-x(i-1))-sig*u(i-1))/p
11    continue
      if (ypn.gt..99e30) then
        qn=0.
        un=0.
      else
        qn=0.5
        un=(3./(x(n)-x(n-1)))*(ypn-(y(n)-y(n-1))/(x(n)-x(n-1)))
      endif
      y2(n)=(un-qn*u(n-1))/(qn*y2(n-1)+1.)
      do 12 k=n-1,1,-1
        y2(k)=y2(k)*y2(k+1)+u(k)
12    continue
      return
      END
      SUBROUTINE splint(xa,ya,y2a,n,x,y)
      INTEGER n
      REAL x,y,xa(n),y2a(n),ya(n)
      INTEGER k,khi,klo
      REAL a,b,h
      klo=1
      khi=n
1     if (khi-klo.gt.1) then
        k=(khi+klo)/2
        if(xa(k).gt.x)then
          khi=k
        else
          klo=k
        endif
      goto 1
      endif
      h=xa(khi)-xa(klo)
      if (h.eq.0.) pause 'bad xa input in splint'
      a=(xa(khi)-x)/h
      b=(x-xa(klo))/h
      y=a*ya(klo)+b*ya(khi)+((a**3-a)*y2a(klo)+(b**3-b)*y2a(khi))*(h**2)/6.
      return
      END
