!======================================================================================      
!======================================================================================      
module state_variables
!     Physical space state variables: time t_n
      real(kind=8), allocatable, dimension(:,:,:,:) :: u,v,w,s1,s2
      real(kind=8), allocatable, dimension(:,:,:)   :: pd,ambient_density
      real(kind=8)                                  :: rho_bar,s1_bar,s2_bar
      real(kind=8)                                  :: s1_bar_z,s2_bar_z
      real(kind=8)                                  :: s1_bar_zz,s2_bar_zz
      real(kind=8)                                  :: Temperature,Salinity,pmPa,density
 
!     Wavenumber space state variables: times t_n, t_n-1
      double complex, allocatable, dimension(:,:,:,:)   :: u_cplx,v_cplx,w_cplx,s1_cplx,s2_cplx
end module state_variables
!======================================================================================      
!======================================================================================      

!======================================================================================      
!======================================================================================      
module particles
     character(len=3)    :: init_particle_file = 'yes'
     character(len=80)   :: particle_step_flag='euler'
     character(len=3)    :: do_factor='yes'     
     integer             :: ntrajectories=0
     integer             :: j_particle_time=1
     integer             :: istart_trajectories=0
     integer,allocatable :: owners(:),tmp_owners(:)
     integer,allocatable :: ipiv(:)
     integer             :: lda=4
     real(kind=8),allocatable    :: L(:),D(:),U1(:),U2(:) !! for storing factored interpolation matrices
     real(kind=8),allocatable    :: positions(:,:)
     real(kind=8),dimension(2)   :: my_z_limits
     real(kind=8),allocatable    :: uvels(:,:)   !! (ntrajectories,3) M,MM1,MM2 for stepping
     real(kind=8),allocatable    :: vvels(:,:)
     real(kind=8),allocatable    :: wvels(:,:)
     real(kind=8),allocatable    :: gvelocity(:,:,:,:)  !! 3d + id array of velocity w/ ghost pts
     real(kind=8),dimension(4,4,4) :: u4x4,v4x4,w4x4
end module particles
!======================================================================================      
!======================================================================================      



!======================================================================================      
!======================================================================================      
module independent_variables
!     Spatial positions
      real(kind=8), allocatable, dimension(:)       :: x,y,z,zglobal
      real(kind=8)                                  :: xpos,ypos,zpos 
      real(kind=8)                                  :: dx,dy,dz,kmax
      real(kind=8)                                  :: dt,t_start,t_end,t_secs,t_stat
!     Wavenumber arrays and integrating factors
      real(kind=8), allocatable, dimension(:)       :: wnx,wny,wnz
      real(kind=8), allocatable, dimension(:,:,:,:) :: amp    
end module independent_variables
!======================================================================================      
!======================================================================================      



!======================================================================================      
!======================================================================================      
module dimensional_params
      real(kind=8)                                  :: U0,DGRAD,s1_scale,s2_scale,time_scale
      real(kind=8)                                  :: rho_0,f,g,kappa_1,kappa_2,nu
      real(kind=8)                                  :: lat,Omega,N2,bfreq,Lx,Ly,Lz      
end module dimensional_params
!======================================================================================      
!======================================================================================      



!======================================================================================      
!======================================================================================      
module dimensionless_params
!     Dimensionless parameters appearing in scaled eqns of motion
      real(kind=8)                                  :: Rot,Ri,Re,Pr_1,Pr_2      
end module dimensionless_params

!======================================================================================      
!======================================================================================      
module mpi_params
      integer                                           :: subslice,twoslice,comm,ierr,myid
      integer                                           :: oneslice,status,iproc,nprocs,numwords 
      integer                                           :: source,dest,tag,nreqs
end module mpi_params
!======================================================================================      
!======================================================================================      



!======================================================================================      
!======================================================================================      
module flags_counters_etc
      character (len=80)                                :: runlabel
      character (len=80)                                :: bc_flag,step_flag='euler'
      character (len=80)                                :: diss_flag,pressure_flag,force_flag
      character (len=3)                                 :: vort_flag
      character (len=11)                                :: ctype,stype,ftype
      character (len=2)                                 :: scalars       ! TS, TP, RP, R0, PP
      integer                                           :: nx,ny,nz,numprocs
      integer                                           :: i,j,k,istep,nyplanes,n3,locnx,locnz
      integer, allocatable                              :: numzplanes(:),itmp1d(:),numzplanes_below(:)
      integer                                           :: N,NM1,M,MM1,MM2,itmp,junk,isign,m1,m2,m3
      integer                                           :: istart,iend,i_stat,dir,maxprocs
      integer                                           :: clock_start(100)=0,rate=0,numcalls(100)=0
      integer                                           :: runtime_initialization_id=1,vort_fcs_id=2,ifft_id=3
      integer                                           :: equation_of_state_id=4,advance_particles_id=5
      integer                                           :: nonlin_id=6,write_results_id=7,fft_id=8,rhs_id=9
      integer                                           :: step_id=10, forcing_id=11, fforcing_id=12
      real(kind=8)                                      :: secs(100)=0.,cumsecs(100)=0.
      real(kind=8)                                      :: pi=3.14159265358979,umax,vmax,wmax
      double complex                                    :: cmplxi=(0.0,1.0)
end module flags_counters_etc
!======================================================================================      
!======================================================================================      



!======================================================================================      
!======================================================================================      
module intermediate_variables
!     3 copies of transfer terms: t_n, t_n-1, t_n-2
      double complex, allocatable, dimension(:,:,:,:)   :: Tr1,Tr2,Tr3,Tr4,Tr5 
      
      double complex,   allocatable, dimension(:,:,:)   :: vortx,vorty,vortz
      real(kind=8), allocatable, dimension(:,:,:)       :: zetax,zetay,zetaz
      
!     Fourier amplitudes of forcing terms
      double complex, allocatable, dimension(:,:,:,:)   :: force_cplx 
      
!     
      real(kind=8), allocatable, dimension(:,:,:)       :: s1_times_u,s1_times_v,s1_times_w,   &
                                                           s2_times_u,s2_times_v,s2_times_w,   &
                                                           T1,T2,T3,dz_s1_bar,dz_s2_bar,       &
                                                           dzz_s1_bar,dzz_s2_bar,              &
                                                           u_dot_grad_s1_bar,u_dot_grad_s2_bar 
      double complex, allocatable, dimension(:,:,:)     :: T1_cplx,T2_cplx,T3_cplx,                          &
	                                                       u_dot_grad_s1_bar_cplx,u_dot_grad_s2_bar_cplx,    &
	                                                       s1_times_u_cplx,s1_times_v_cplx,s1_times_w_cplx,  &
	                                                       s2_times_u_cplx,s2_times_v_cplx,s2_times_w_cplx,  &
	                                                       dz_s1_bar_cplx,dz_s2_bar_cplx,                    &
	                                                       dzz_s1_bar_cplx,dzz_s2_bar_cplx
                                                           
end module intermediate_variables
!======================================================================================      
!======================================================================================      



!======================================================================================      
!======================================================================================      
module io_params
       integer, parameter                    :: maxfiles=20
       integer                               :: num_netcdf_files=0
       integer                               :: start(4,maxfiles),count(4,maxfiles)  !! (dimension,file id)
       integer                               :: dimid_x(maxfiles),dimid_y(maxfiles)
       integer                               :: dimid_z(maxfiles),dimid_t(maxfiles)
       integer                               :: time_counter(maxfiles)=1
       integer                               :: ilocs(3,maxfiles)    !! (start,end,inc,file id)
       integer                               :: jlocs(3,maxfiles)    !! (start,end,inc,file id)
       integer                               :: klocs(3,maxfiles)    !! (start,end,inc,file id)  -- global indexing
       integer                               :: my_kstart(maxfiles)
       integer                               :: my_kend(maxfiles)
       integer                               :: my_kinc(maxfiles)
       character(len=80)                     :: filename_root(maxfiles),fullname(maxfiles),mode(maxfiles)='new'
       integer                               :: nsteps(maxfiles),variable_key(9,maxfiles) 
       integer                               :: file_id=0,idim,jdim,ncid,rcode,nspace(maxfiles)
       character(len=6)                      :: cnum
       character(len=3)                      :: cid,dimstr,do_x(maxfiles),do_y(maxfiles),do_z(maxfiles)
end module io_params
!======================================================================================      
!======================================================================================      
