SHELL=/bin/sh
BASE=${PWD}

#=====================================================
MY_MODULE=-I/$(BASE)/src/class
PETSC_DIR=$(PETSC_DIR_X)
include $(PETSC_DIR)/bmake/common/variables
include $(PETSC_DIR)/bmake/petscconf


#==================================================================
#=== f90 optimization flags
#==================================================================
 FOPTFLAGS = -C -O3          # -C -qarch=auto -qextname -qsuffix=cpp=f90

#==================================================================
#=== the following can be set as environment 
#=== variables (the preferred approach) or defined 
#=== explicitly here
#==================================================================

 F77        = pgf95          # how to compile fixed format .f files
 F90        = mpif90 -g -Mpreprocess -Mbounds     # f90 compiler that is mpi aware

PACKAGES_LIBS= $(HYPRE_LIB_X) $(SCALAPACK_LIB_X) $(LAPACK_LIB_X) $(BLAS_LIB_X) $(BLACS_LIB_X) $(SUPERLUDIST_LIB_X) $(X11_LIB_X) 
PETSC_LIBS = ${PACKAGES_LIBS} $(PETSC_LIB_X) $(PETSC_FORTRAN_LIB_BASIC) $(PETSC_LIB_BASIC)

XTRA_LIBS = $(HYPRE_LIB_X) $(SCALAPACK_LIB_X) $(LAPACK_LIB_X) $(BLAS_LIB_X) $(BLACS_LIB_X) $(SUPERLUDIST_LIB_X) $(NETCDF_LIB_X) $(FFTW3_LIB_X) $(MPI_LIB_X)

#==================================================================
# combine  variables for (PETSC + EXTERNAL PACKAGES), NETCDF and FFTW3
#==================================================================

ALL_INCLUDES = $(MY_MODULE) $(PETSC_INC_X) $(NETCDF_INC_X) $(FFTW3_INC_X) $(MPI_INC_X)
ALL_EXT_LIBS = $(PETSC_LIBS) $(XTRA_LIBS) 

showconfig:
	@echo
	-@echo  "host:          " `hostname -s`
	-@echo  "               " `uname -sr`
	-@echo  "user:          " `whoami`
	-@echo  "compile date:  " `date`
	-@echo  "PETSC_DIR:     " ${PETSC_DIR}
	-@echo  "PETSC_ARCH:    " ${PETSC_ARCH}
	-@echo  "OUTPUT_ROOT:   " ${OUTPUT_ROOT}
	-@echo  "F90:           " ${F90}
	-@echo  "F77:           " ${F77}
	-@echo  "FOPTFLAGS:     " ${FOPTFLAGS}
	-@echo  "FLINKER:       " ${FLINKER}
	-@echo  "NETCDF_INC:    " ${NETCDF_INC}
	-@echo  "NETCDF_LIB:    " ${NETCDF_LIB}
	-@echo  "FFTW_INC:      " ${FFTW_INC}
	-@echo  "FFTW_LIB:      " ${FFTW_LIB}
	-@echo  "LAPACK_LIB:    " ${LAPACK_LIB}
	-@echo  "BLAS_LIB:      " ${BLAS_LIB}
	-@echo  "PETSC_LIBS:    " ${PETSC_LIBS}
	-@echo  "XTRA_LIBS:     " ${XTRA_LIBS}
