subroutine read_userdata
 use mpi_params
 use user_data
 use io_params
 use particles
 use etc

 implicit none  
 include 'mpif.h'
 integer           :: processor_doing_reads
 integer           :: id
 integer           :: endpt
 character(len=80) :: file1,file2,file3,file4,file5,runlabel2

 file1='input/problem_params'
 file2='input/num_filesets'
 file3='input/io_params'
 file4='input/BoundaryConditions'
 file5='input/DifferentiationMethods'
    
do processor_doing_reads = 0, numprocs-1

 if( processor_doing_reads == myid ) then  !! my turn
  
  open(1,file=file1,position='rewind') 
  read(1,*) runlabel
  read(1,*) nx
  read(1,*) ny
  read(1,*) nz
  read(1,*) nparticles
  read(1,*) dt
  read(1,*) t_start
  read(1,*) t_end
  read(1,*) Lx
  read(1,*) Ly
  read(1,*) Lz
  read(1,*) scalar_kind(1)
  read(1,*) scalar_kind(2)
  read(1,*) do_forcing
  read(1,*) do_immersed_boundary
  read(1,*) g
  read(1,*) f
  read(1,*) beta_dimensional
  read(1,*) rho_0
  read(1,*) nu
  read(1,*) kappa_1
  read(1,*) kappa_2
  read(1,*) dgrad
  read(1,*) s1_scale
  read(1,*) s2_scale
  read(1,*) u0
 close(1)
 
 !! these are hardwired here for now
 DNS=.TRUE.
 LES=.FALSE. !! should be read in
 if( LES ) DNS=.FALSE.
 
 if(myid==0) write(0,*) '... ',trim(file1),' read'

 open(1,file=file2,position='rewind')
  read(1,*) num_netcdf_files
 close(1)
 if(myid==0) write(0,*) '... ',trim(file2),' read'
 
 open(1,file=file3,position='rewind')
 do i=1,num_netcdf_files
  read(1,*) filename_root(i)
  read(1,*) mode(i)
  read(1,*) nsteps(i)
  read(1,*) ilocs(1,i)
  read(1,*) ilocs(2,i)
  read(1,*) ilocs(3,i)
  read(1,*) jlocs(1,i)
  read(1,*) jlocs(2,i)
  read(1,*) jlocs(3,i)
  read(1,*) klocs(1,i)
  read(1,*) klocs(2,i)
  read(1,*) klocs(3,i)
  do j=1,5 !! u,v,w,s1,s2
   read(1,*) variable_key(j,i)
  enddo
  if(myid==0) write(0,*) '... fileset ',i,' read'
  
  !! set these to 1 during testing
  variable_key(6,i)=1   ! divustar
  variable_key(7,i)=1   ! phi
  
  !! not pd though
  variable_key(8,i)=0   ! pd
  
  !! quick sanity check ....
  if(ilocs(2,i) > nx ) stop 'io_params data > nx'
  if(jlocs(2,i) > ny ) stop 'io_params data > ny'
  if(klocs(2,i) > nz ) stop 'io_params data > nz'
 enddo
 
 if(myid==0) write(0,*) '... ',trim(file3),' read'
 close(1)

 open(1,file=file4,position='rewind')
 do id=1,5
  do idim=1,3
   do endpt=1,2
    read(1,*) BCtype(id,idim,endpt)
   enddo
  enddo
 enddo
 if(myid==0) write(0,*) '... ',trim(file4),' read'
 close(1)
 
 open(1,file=file5,position='rewind')
  read(1,*) u_deriv_types(1)
  read(1,*) u_deriv_types(2)
  read(1,*) u_deriv_types(3)

  read(1,*) v_deriv_types(1)
  read(1,*) v_deriv_types(2)
  read(1,*) v_deriv_types(3)

  read(1,*) w_deriv_types(1)
  read(1,*) w_deriv_types(2)
  read(1,*) w_deriv_types(3)

  read(1,*) s1_deriv_types(1)
  read(1,*) s1_deriv_types(2)
  read(1,*) s1_deriv_types(3)

  read(1,*) s2_deriv_types(1)
  read(1,*) s2_deriv_types(2)
  read(1,*) s2_deriv_types(3)

  read(1,*) phi_deriv_types(1)
  read(1,*) phi_deriv_types(2)
  read(1,*) phi_deriv_types(3)
  if(myid==0) write(0,*) '... ',trim(file5),' read'
 close(1)
 endif
 
 idim=1
 if( BCtype(1,idim,1) == 'periodic' ) then
  BCtype(:,idim,:)='periodic'
  xdim_periodic=.TRUE.
 else
  xdim_periodic=.FALSE.
 endif
 
 idim=2
 if( BCtype(1,idim,1) == 'periodic' ) then
  BCtype(:,idim,:)='periodic'
  ydim_periodic=.TRUE.
 else
  ydim_periodic=.FALSE.
 endif
 
 idim=3
 if( BCtype(1,idim,1) == 'periodic' ) then
  BCtype(:,idim,:)='periodic'
  zdim_periodic=.TRUE.
 else
  zdim_periodic=.FALSE.
 endif

 !! not my turn, just wait
 call mpi_barrier(comm,ierr)
 
enddo !! on to next processor


end subroutine read_userdata
