program test_driver
!implicit none
integer             :: ierr,myid,numprocs,ncid
integer(kind=8)     :: forward_plan ,inverse_plan 
integer             :: rank_trans,howmany_rank
integer             :: n,nb2,n_trans,n_loop
integer             :: fft_kind,myflag
integer             :: is_trans,is_loop
integer             :: os_trans,os_loop

real(kind=8), allocatable  :: in(:,:,:),out(:,:,:)
character(len=80)          :: optionfile

include 'fftw3.f'     
include 'netcdf.inc'
#include "finclude/petsc.h"  
#include "finclude/petscvec.h"

Vec   petsc_vec

write(0,*) ' ................   =========================================== '
write(0,*) ' ................   Saludos de OOFlowSolve installacion prueba. '
write(0,*) ' ................   =========================================== '


!! (1) MPI
write(0,*) ' ................   '
write(0,*) ' ................   Prueba de MPI .....'
write(0,*) ' ................   '
call mpi_init(ierr)
call mpi_comm_rank(MPI_COMM_WORLD,myid,ierr)
call mpi_comm_size(MPI_COMM_WORLD,numprocs,ierr)

!read user parameter file or use test values
if( myid .eq. 0 .and. ierr==MPI_SUCCESS ) then
write(0,*) ' ................        Soy MPI y vivo! '
write(0,*) ' ................        Tengo ',numprocs,' processero(s).'
else
write(0,*) ' ................        Houston, tenemos una problema con MPI... '
write(0,*) ' ................         ierr   '    ,ierr
write(0,*) ' ................         myid   '    ,myid
write(0,*) ' ................         numprocs   ',numprocs
endif   

!! (2) NETCDF
write(0,*) ' ................   '
write(0,*) ' ................   Prueba de NETCDF .....'
write(0,*) ' ................   '
ierr=NF_CREATE('install_test.nc',NF_CLOBBER,ncid)
if(ierr.eq.NF_NOERR) then
 write(0,*) ' ................        creating a netcdf file: install_test.nc'
else
 write(0,*) ' ................        problem creating a netcdf file: ',ierr
endif

ierr=NF_ENDDEF(ncid)
if(ierr.eq.NF_NOERR) then
 write(0,*) ' ................        exit define mode ok'
else
 write(0,*) ' ................        error while leaving define mode'
endif

ierr=NF_CLOSE(ncid)
if(ierr.ne.NF_NOERR) &
write(0,*) ' ................        problems closing a netcdf file'

!! (3) FFTW3
write(0,*) ' ................   '
write(0,*) ' ................   Prueba de FFTW3 .....'
write(0,*) ' ................   '

rank_trans=1          !! Transforms work on 1 dimension at a time.
howmany_rank=1        !! Loop over the other 2 dimensions treated as 1d in memory
n = 32                !! Data size in transform direction, measured in real words.
nb2 = n/2
n_trans=n             !! length of transform in real words     
is_trans=1            !! stride of 1d transform --> in x
os_trans=is_trans
!! describe the loops over dims 2,3
n_loop=n*n            !! 
is_loop=n             !! stride between transforms
os_loop=is_loop
myflag=FFTW_PATIENT
fft_kind=FFTW_R2HC
allocate( in(n,n,n),out(n,n,n) )

call dfftw_plan_guru_r2r(forward_plan,         &
rank_trans,n_trans,is_trans,os_trans,          &
howmany_rank,n_loop,is_loop,os_loop,           &
in,out,fft_kind,myflag)
write(0,*) ' ................        creating dfftw_plan_guru_r2r ok ..'
!execute an fftw3 plan
in(:,:,:) = 0.0
out(:,:,:)= 0.0
call dfftw_execute_r2r(forward_plan,in,out)
write(0,*) ' ................         executing dfftw_execute_r2r ok ..'


!! (4) PETSC
write(0,*) ' ................   '
write(0,*) ' ................   Initializacion de PETSC .....'
write(0,*) ' ................   '
optionfile='PetscOptions'
call PetscInitialize(optionfile,ierr)
if(ierr==0) write(0,*) ' ................        opened PETSC options file '
if(ierr==0) write(0,*) ' ................        PetscInitialize ok .....'


!! (5) Read User Specified Parameters
write(0,*) ' ................   '
write(0,*) ' ................   Leer datos del usuario .....'
write(0,*) ' ................   '
call read_userdata

!! Shut down MPI
call mpi_finalize(ierr)
write(0,*) ' ................   '
write(0,*) ' ................   Shutting down MPI ..... '
write(0,*) ' ................        MPI esta muerte ahora. ',ierr
write(0,*) ' ................   '

deallocate(in,out)
write(0,*) ' ................   =========================================== '
write(0,*) ' ................   Adios '
write(0,*) ' ................   =========================================== '
end program test_driver
