subroutine toggle
 use user_data
 use etc
 use mpi_params
 use operators
 implicit none
  
 integer  :: itmp
  
 if( myid .eq. 0 ) then
  write(0,*) '..........................................................'
  write(0,*) '............................     Toggling, istep  ',istep
  write(0,*) '..........................................................'
 endif
  
 t_secs = t_secs + dt*time_scale
 
!!Toggle 2-cycle pointer for AM solves (N,NM1).
 if( AM_order .eq. 4 ) then
  itmp=NM1
  NM1=N  
  N=itmp
  N_oldest=NM1
 elseif( AM_order .eq. 3 ) then
  N=1
  N_oldest=N
 endif
 
!!Toggle M cycle pointers for AB rhs fields
 if( AB_order .eq. 4 ) then
  itmp=MM3
  MM3=MM2
  MM2=MM1
  MM1=MM0
  MM0=itmp
  M_oldest=MM3
 elseif( AB_order .eq. 3 ) then
  itmp=MM2
  MM2=MM1
  MM1=MM0
  MM0=itmp
  M_oldest=MM2
 elseif( AB_order .eq. 2 ) then
  itmp=MM1
  MM1=MM0
  MM0=itmp
  M_oldest=MM1
 endif
 
!! Counters for Particle Trajectories
!! (always 4th order AB)
  itmp=PM3
  PM3=PM2
  PM2=PM1
  PM1=PM0
  PM0=itmp
  P_oldest=PM3

end subroutine toggle



subroutine finalize
  use dependent_variables
  use intermediate_variables
  use user_data
  use etc
  use operators
  use mpi_params
  use mpi_times
 
!!! xlv no skipping deallocations
!!! if(myid==0) STOP 'skipping deallocations...just quit'

!!! xlv smart stop job 
!===============================!
 call PetscFinalize(ierr)
 if ( myid == 0 ) then
      print*,' '
      print*,'   ... Good Job !'
      print*,' '
 endif
 call MPI_TIME(1)
 call quit_mpi(ierr)
!===============================!

  if(myid==0) STOP 'skipping deallocations...just quit'
  
  call DeleteVector(velocity)
  
  do i=0,mglevels-1
   call DeleteScalarVariable(Phi(i))
  enddo
  
  call DeleteScalarVariable(pd)
  call DeleteScalarVariable(rhs_b)
  
  do i=1,5
   do j=1,AB_order
    call DeleteScalarVariable(explicit_rhs(i,j))
   enddo
   call DeleteScalarVariable(implicit_rhs(i,1))
   if(AM_order>3) call DeleteScalarVariable(implicit_rhs(i,2))
  enddo
 
  do i=1,4
   call DeleteScalarVariable(scratch(i))
  enddo
  
  if(myid==0) write(0,*) '... ScalarVariables etc deallocated... skipping PetscFinalize'
  !call PetscFinalize(ierr)

end subroutine finalize
 
 
!!===========================================================
!!===========================================================
 subroutine LogMessage(msg,logfile)
  character(len=80)  :: msg,logfile
  open(1,file=logfile,position='append')
   write(1,*) msg
  close(1)
 end subroutine LogMessage
!!===========================================================
!!===========================================================



!=============================================================== 
!===============================================================
        !   "rho.f"

        !   Parameter   :   equation of state
        !   Source      :   MILLERO et.al. (1980) DEEP-SEA RES.
        !                   Vol 27A, pp 255-264
        !   Units       :   Kg / m**3
        !   Input       :   s (concentration units), t (deg C), p (MPa)
        !   Range       :   s (0 to 0.040), t (-2 to 40), p (MPa)
        !   Ck value    :   K (0, 0, 100)      = 2.297721e4
        !                   K (0.035, 0, 100)  = 2.499200e4
        !                   K (0, 25, 100)     = 2.54051e4
        !                   K (0.035, 25, 100) = 2.710895e4
        !   Coded       :   Ngoc Dang, July 1982

        subroutine rho (s, t, p, density)

        real(kind=8) s, t, p
        real(kind=8) a, b, c, d, e, k, t1, t2, t3, t4, ta, tb
        real(kind=8) k0, k0w, alpha0, alpha, rho0, aw, bw, density 


        !  convert input units

        s = 1000.0d0 * s
        p = 10.0d0 * p

        !  compute alpha0 = alpha (s, t, 0)

        t1 = 9.99841594d-1 + t * (6.793952d-5 + t * (-9.095290d-6 + t * &
             (1.001685d-7 + t * (-1.120083d-9 + t * 6.536332d-12))))
     
        t2 = 8.25917d-4 + t * (-4.4490d-6 + t * (1.0485d-7 + t *    &   
             (-1.258d-9 + t * 3.315d-12)))
             
        t3 = -6.33761d-6 + t * (2.8441d-7 + t * (-1.6871d-8 + t *   &   
             2.83258d-10))
             
        t4 = 5.4705d-7 + t * (-1.97975d-8 + t * (1.6641d-9 - t *    &   
             3.1203d-11))
     
        rho0 = t1 + t2 * s + t3 * s**(1.5) + t4 * s**2
        alpha0 = 1.0d0 / rho0

        !  compute the pressure coefficient

        !   the first term
        k0w = 1.965221d4 + t * (1.484206d2 + t * (-2.327105d0 + t *   &  
              (1.360477d-2 - t * 5.155288d-5)))
              
        ta = 5.46746d1 + t * (-6.03459d-1 + t * (1.09987d-2 - t *     &    
             6.167d-5))
             
        tb = 7.944d-2 + t * (1.6483d-2 - t * 5.3009d-4)
        k0 = k0w + ta * s + tb * s**(1.5)

        !   the second term
        aw = 3.239908d0 + t * (1.43713d-3 + t * (1.16092d-4 - t *  &     
             5.77905d-7))
     
        c = 2.2838d-3 + t * (-1.0981d-5 - t * 1.6078d-6)
        d = 1.9107d-4
        a = aw + c * s + d * s**(1.5)

        !   the third term
        bw = 8.50935d-5 + t * (-6.12293d-6 + t * 5.2787d-8)
        e = -9.9348d-7 + t * (2.0816d-8 + t * 9.1697d-10)
        b = bw + e * s

        k = k0 + a * p + b * p**2

        !  compute density
        alpha = alpha0 * (1.0d0 - p / k)
        density = 1.0d0 / alpha

        !  convert to kg / m**3
        density = 1000.0d0 * density
        s = s / 1000.0d0
        p = p / 10.0d0

        return
        end
!=============================================================== 
!===============================================================

 

!================================================================== 
!==================================================================
     subroutine eos
!!=================================================================
!!=================================================================
!     Routine to calculate d'less pd from values of d'less s1 and s2
!     and their reference profiles s1_bar(x,z) and s2_bar(x,z)
!     The "perturbation" density is defined relative to the reference
!     profile rho_bar(z) which is determined by s1_bar and s2_bar.
!======================================================================
!     scalars='TS'
!     EOS from MILLERO et.al. (1980) DEEP-SEA RES.
!     The scalar s1=temparature in deg C and 
!     the scalar s2 is salinity in psu (i.e. ~ 36)
!======================================================================
      !use user_routines
      use user_data
      use independent_variables
      use dependent_variables
      use intermediate_variables
      use mpi_params
      use etc
      
      implicit none
      real(kind=8)          :: xx,yy,PMpa
      real(kind=8)          :: rho_bar,total_density
      real(kind=8)          :: temperature,salinity
      
      PMpa = 0.0   !   compute potential density, not in situ density

      if(        trim(scalar_kind(1)) .eq. 't'              &
           .and. trim(scalar_kind(2)) .eq. 's' ) then
           
            xx=1.0    ! use this to keep  Salinity
            yy=1.0    ! use this to keep Temperature
            
      elseif(    trim(scalar_kind(1)) .eq. 't'              &
           .and. trim(scalar_kind(2)) .ne. 's' ) then
           
            yy=1.0    ! use this to keep Temperature
            xx=0.0    ! use this to neglect Salinity
            
      elseif(    trim(scalar_kind(1)) .ne. 't'              &
           .and. trim(scalar_kind(2)) .eq. 's' ) then
           
            xx=1.0    ! use this to keep  Salinity
            yy=0.0    ! use this to neglect Temperature
            
      elseif( trim(scalar_kind(1)) .eq. 'r' ) then
      
            pd%SlabVals = scalars(1)%SlabVals
            return
        
      elseif( trim(scalar_kind(2)) .eq. 'r' ) then
      
            pd%SlabVals = scalars(2)%SlabVals
            return
            
      elseif(       trim(scalar_kind(2)) .eq. 'p'         &
              .and. trim(scalar_kind(1)) .eq. 'p' ) then
              
            return    ! pd remains zero
            
      else
      
            stop 'problem parsing scalars code: eos in flow_solve.f90'
       
      endif

      xx = xx/1000.d0  ! for units consistency w/ fortran routine rho
      
      do k=1,scalars(1)%DataMap%locnz
         
        Temperature = yy*(s1_bar(k)*s1_scale)         !! [deg C]
        Salinity =    xx*(s2_bar(k)*s2_scale)         !! [psu]
        call rho(Salinity,Temperature,Pmpa,rho_bar)   !! ambient density [kg/m3]                    
        
        do j=1,ny
          do i=1,nx
           
           !!now repeat the calculations for total s1 & s2(x,y,z)
           Temperature = yy*( (s1_bar(k)  +  &
                               scalars(1)%SlabVals(i,j,k))*s1_scale ) ! dimensional, total Temp.
 
           Salinity    = xx*( (s2_bar(k)  +  &
                               scalars(2)%SlabVals(i,j,k))*s2_scale ) ! dimensional, total Salinity                        

           call rho(Salinity,Temperature,PMpa,total_density) ! total density [kg/m3]
           

           pd%SlabVals(i,j,k) = (total_density - rho_bar)/(Lz*DGRAD)
           

         enddo
        enddo
       enddo  

      end subroutine eos


function acosh2 ( x )

!*******************************************************************************
!
!! ACOSH2 returns the inverse hyperbolic cosine of a number.
!
!  Discussion:
!
!    Since a library function ACOSH may be available on some systems,
!    this routine is named ACOSH2 to avoid name conflicts.
!
!    One formula is:
!
!      ACOSH2 = LOG ( X + SQRT ( X**2 - 1.0 ) )
!
!    but this formula suffers from roundoff and overflow problems.
!    The formula used here was recommended by W Kahan, as discussed
!    by Moler.
!
!    Applying the inverse function
!
!      Y = ACOSH2(X)
!
!    implies that
!
!      X = COSH(Y) = 0.5 * ( EXP(Y) + EXP(-Y) ).
!
!    For every X greater than or equal to 1, there are two possible
!    choices Y such that X = COSH(Y), differing only in sign.  It
!    is usual to resolve this choice by taking the value of ACOSH2(X)
!    to be nonnegative.
!
!  Modified:
!
!    06 April 1999
!
!  Author:
!
!    John Burkardt
!
!  Reference:
!
!    Cleve Moler,
!    Trigonometry is a Complex Subject,
!    MATLAB News and Notes,
!    Summer 1998.
!
!  Parameters:
!
!    Input, real ( kind = 8 ) X, the number whose inverse hyperbolic 
!    cosine is desired.  X should be greater than or equal to 1.
!
!    Output, real ( kind = 8 ) ACOSH2, the inverse hyperbolic cosine of 
!    X.  The principal value (that is, the positive value of the two ) 
!    is returned.
!
  implicit none

  real ( kind = 8 ) acosh2
  real ( kind = 8 ) x

  if ( x < 1.0D+00 ) then
    write ( *, '(a)' ) ' '
    write ( *, '(a)' ) 'ACOSH2 - Fatal error!'
    write ( *, '(a)' ) '  Argument X must satisfy 1 <= X.'
    write ( *, '(a,g14.6)' ) '  The input X = ', x
    stop
  end if

  acosh2 = 2.0D+00 * log ( &
    sqrt ( 0.5D+00 * ( x + 1.0D+00 ) ) + sqrt ( 0.5D+00 * ( x - 1.0D+00 ) ) )

  return
end





subroutine AdjustDerivTypes(id,SV)
 use Class_ScalarVariable
 use dependent_variables
 use etc
 implicit none
 integer                :: id
 type( ScalarVariable ) :: SV(3)
 
 !! Reset the deriv types for vector SV
 !! On input, the three components of SV are:
 !!   u_x, u_y, u_z   if id=1
 !!   v_x, v_y, v_z   if id=2
 !!   w_x, w_y, w_z   if id=3
 !! the derivatives need to be set properly
 !! so that div(SV) can be done correctly
 
 if( id > 0 .and. id < 4 ) then
 
  do i=1,3
   if( velocity(id)%DataMap%DerivTypes(i)=='sin' ) then
     SV(id)%DataMap%DerivTypes(i)='cos'
   elseif( velocity(id)%DataMap%DerivTypes(i)=='cos' ) then
     SV(id)%DataMap%DerivTypes(i)='sin'
   else
     SV(id)%DataMap%DerivTypes(i)=velocity(id)%DataMap%DerivTypes(i)
   endif
  enddo
   
  elseif( id == 4 ) then
  
   do i=1,3
    if( scalars(1)%DataMap%DerivTypes(i)=='sin' ) then
      SV(id)%DataMap%DerivTypes(i)='cos'
    elseif( scalars(1)%DataMap%DerivTypes(i)=='cos' ) then
      SV(id)%DataMap%DerivTypes(i)='sin'
    else
      SV(id)%DataMap%DerivTypes(i)=scalars(1)%DataMap%DerivTypes(i)
    endif
   enddo
   
  elseif( id == 5 ) then
  
   do i=1,3
    if( scalars(2)%DataMap%DerivTypes(i)=='sin' ) then
      SV(id)%DataMap%DerivTypes(i)='cos'
    elseif( scalars(2)%DataMap%DerivTypes(i)=='cos' ) then
      SV(id)%DataMap%DerivTypes(i)='sin'
    else
      SV(id)%DataMap%DerivTypes(i)=scalars(2)%DataMap%DerivTypes(i)
    endif
   enddo
   
  endif
end subroutine AdjustDerivTypes

!===================================================    
!===================================================
integer function factorial(k)
integer::ans,n,k
 ans=1
 n=k
 do while (n.GT.1)
  ans=ans*n
  n=n-1
 enddo
 factorial=ans
 return
end function factorial
!===================================================    
!===================================================



!======================================================================
!======================================================================
function delta(x,eps)
 implicit none
 real(kind=8) :: delta,x,eps,recip
 
 !!=================================
 !! delta = (1/2eps) sech^2(x/eps)
 !!  (a) in lim eps-->0, delta-->inf
 !!  (b) int delta(x;eps) dx = 1
 !!=================================
 recip = 2.*eps*cosh(x/eps)**2
 delta = 1.d0/recip
                                 
 return
end function delta  
!======================================================================
!======================================================================


!======================================================================
!======================================================================
function step(x,eps)
 implicit none
 real(kind=8) :: step,x,eps
 
 !!=================================
 !! step = (1/2){ tanh(x/eps) + 1. }
 !!  [H] = 1. 
 !!  d/dx(H) = step 
 !!=================================
 step = (0.5d0)*( tanh(x/eps) + 1.d0 )
                                 
 return
end function step  
!======================================================================
!======================================================================
