module InterfacePetsc
 use class_ScalarVariable 
  
 public  ::  ScalarVariable2PetscVec
 public  ::  PetscVec2Scalar
 
 Contains
 
subroutine ScalarVariable2PetscVec(f,petsc_vector)
use class_ScalarVariable
implicit none
#include "finclude/petsc.h"
#include "finclude/petscvec.h"

Vec                       petsc_vector
type( ScalarVariable ) :: f
integer,allocatable, SAVE :: indices(:)
integer                :: ihi,ilo
integer                :: icount
integer                :: i,j,k
integer                :: jerr,myid
integer, SAVE          :: nlocal
logical, SAVE          :: first_entry = .TRUE.

if( first_entry ) then
 call mpi_comm_rank(MPI_COMM_WORLD,myid,jerr)
  if(jerr .ne. 0 ) &
  stop 'ScalarVariable2PetscVec: error calling mpi_comm_rank'
     
  nlocal = f%DataMap%SlabDims(1,myid)  &
         * f%DataMap%SlabDims(2,myid)  &
         * f%DataMap%SlabDims(3,myid)
   
  !! set up an "indices" array corresponding to 
  !! normal fortran array indexing but starting at 0
  allocate( indices(nlocal) )
   icount=0
   do k=1,f%DataMap%SlabDims(3,myid)
    do j=1,f%DataMap%SlabDims(2,myid)
     do i=1,f%DataMap%SlabDims(1,myid)
      indices(icount + 1)=icount
      icount=icount+1
    enddo
   enddo
  enddo
  
  call VecGetOwnershipRange(petsc_vector,ilo,ihi,jerr)
  if(jerr .ne. 0 ) &
  stop 'ScalarVariable2PetscVec: error calling VecGetOwnershipRange'
  indices=indices + (ilo)
    
 first_entry = .FALSE.
endif

!!=================================================
!!store the values in the petsc vector
!!=================================================
 
 call VecSetValues(petsc_vector,nlocal,indices,    &
                   f%SlabVals,Insert_Values,jerr)
                   
  if(jerr .ne. 0 ) &
  stop 'ScalarVariable2PetscVec: error calling VecSetValues'
  
 call VecAssemblyBegin(petsc_vector,jerr)
  if(jerr .ne. 0 ) &
  stop 'ScalarVariable2PetscVec: error calling VecAssemblyBegin'
  
 call VecAssemblyEnd(petsc_vector,jerr) 
  if(jerr .ne. 0 ) &
  stop 'ScalarVariable2PetscVec: error calling VecAssemblyEnd'
 
!!=================================================
!!=================================================
end subroutine ScalarVariable2PetscVec








!!=================================================
!!=================================================
subroutine PetscVec2Scalar(petsc_vector,f)
use class_ScalarVariable

implicit none
#include "finclude/petsc.h"
#include "finclude/petscvec.h"

type( ScalarVariable ) :: f
Vec                       petsc_vector

integer                   :: ilo,ihi
integer                   :: i,j,k,icount
integer,allocatable, SAVE :: indices(:)
integer, SAVE             :: myid,jerr,nlocal
logical, SAVE             :: first_entry = .TRUE.

if( first_entry ) then
 call mpi_comm_rank(MPI_COMM_WORLD,myid,jerr)
  if(jerr .ne. 0 ) &
  stop 'PetscVec2Scalar: error calling mpi_comm_rank'
     
  nlocal = f%DataMap%SlabDims(1,myid)  &
         * f%DataMap%SlabDims(2,myid)  &
         * f%DataMap%SlabDims(3,myid)
   
  !! set up an "indices" array corresponding to 
  !! normal fortran array indexing but starting at 0
  allocate( indices(nlocal) )
   icount=0
   do k=1,f%DataMap%SlabDims(3,myid)
    do j=1,f%DataMap%SlabDims(2,myid)
     do i=1,f%DataMap%SlabDims(1,myid)
      indices(icount + 1)=icount
      icount=icount+1
    enddo
   enddo
  enddo
  
  call VecGetOwnershipRange(petsc_vector,ilo,ihi,jerr)
  if(jerr .ne. 0 ) &
  stop 'PetscVec2Scalar: error calling VecGetOwnershipRange'
  indices=indices + (ilo)
    
 first_entry = .FALSE.
endif

 
 call VecGetValues(petsc_vector,nlocal,indices,f%SlabVals,jerr)
  if(jerr .ne. 0 ) &
  stop 'PetscVec2Scalar: error calling VecGetValues'
  
 call VecAssemblyBegin(petsc_vector,jerr)
  if(jerr .ne. 0 ) &
  stop 'PetscVec2Scalar: error calling VecAssemblyBegin'
  
 call VecAssemblyEnd(petsc_vector,jerr)
  if(jerr .ne. 0 ) &
  stop 'PetscVec2Scalar: error calling VecAssemblyEnd' 

end subroutine PetscVec2Scalar
!!=================================================
!!=================================================

end module InterfacePetsc 
