subroutine write_results
 use user_data
 use etc
 use mpi_params
 use particles
 use io_params
  
 implicit none
 integer       :: ifile
 integer       :: inc_particle_write=16
 
 if( nparticles > 0 .and. mod(istep,inc_particle_write)==0 ) then
  call write_particle_data
 endif
   
 do ifile=1,num_netcdf_files
   
  if( mod( (istep-istart), nsteps(ifile) ) == 0    &
       .or. istep == istart ) then
    
   if( trim(mode(ifile)) == 'new' .or. istep == istart) then      
    call init_netcdf(ifile) 
    call write_netcdf(ifile)         
   elseif( trim(mode(ifile)) == 'append') then
    call write_netcdf(ifile)
   endif
               
  endif
    
 enddo
       
 call mpi_barrier(comm,ierr)
 return
end subroutine write_results
  
  
 subroutine init_netcdf(fid)
 use user_data
 use intermediate_variables,            &
        global_scratch_space=>scratch
 use dependent_variables
 use independent_variables
 use mpi_params
 use io_params
 use etc
      
 implicit none
 include 'netcdf.inc'

 character(len=80)         :: filename,topdir
 integer                   :: npts,ndims,All_nD_VarSIZE(4)
 integer                   :: start1D(1),count1D(1),iwrk
 integer                   :: k0,kend,kinc,kcount,counter
 real(kind=4),allocatable,save  :: scratch(:)
     
 integer      :: fid,iid,jid,kid,timeid
 integer      :: xVarID,yVarID,zVarID,tVarID,s1VarID,s2VarID   
 integer      :: divustarVarID,phiVarID,pdVarID
 integer      :: rhoVarID,uVarID,vVarID,wVarID,pVarID

 character(len=80)   :: s1_name, s2_name, s1_units, s2_units
 integer             :: s1_name_len, s2_name_len, s1_units_len, s2_units_len
 
 integer             :: count_vals,mymod
 logical,save        :: ROCKS_STORAGE=.FALSE.
 logical,save        :: first_entry=.TRUE.
 external  count_vals
!!============================================================================ 
  if( first_entry ) then
   iwrk=maxval( (/nx,ny,nz/) )
   allocate( scratch(iwrk+1), stat=rcode )
   if( rcode /= 0 ) stop 'error allocating scratch in init_netcdf'
   first_entry=.FALSE.
  endif
  
  npts=numzplanes_below(myid)
  
  !! count the number of spatially varying indices up from 1
  !! if not varying, count down from 4
  !! nspace is the number of spatially varying indices
  !! these values are stored in dimid_x,dimid_y,dimid_z,dimid_t
  
  idim=1  !! count up for spatially varying dimensions
  jdim=4  !! count down for dimensions w/ indices held constant
  count(:,fid)=1
  nspace(fid)=0    
  
   if( ilocs(2,fid).ne.ilocs(1,fid) ) then
    nspace(fid)=nspace(fid)+1
    dimid_x(fid) = idim
    idim = idim + 1
   else
    dimid_x(fid) = jdim
    jdim = jdim - 1
   endif
   
   if( jlocs(2,fid).ne.jlocs(1,fid) ) then
    nspace(fid)=nspace(fid)+1
    dimid_y(fid) = idim
    idim = idim + 1
   else
    dimid_y(fid) = jdim
    jdim = jdim - 1
   endif
   
   if( klocs(2,fid).ne.klocs(1,fid) ) then
    nspace(fid)=nspace(fid)+1
    dimid_z(fid) = idim
    idim = idim + 1
   else
    dimid_z(fid) = jdim
    jdim = jdim - 1
   endif
   dimid_t(fid) = idim
   ndims=idim   !! time + counted number of space dims

   !! given # of space dimensions, decide which directory to write to
   if( ndims-1==0 ) then
    dimstr='TS/'
   elseif( ndims-1==1 ) then
    dimstr='1D/'
   elseif( ndims-1==2 ) then
    dimstr='2D/'
   elseif( ndims-1==3 ) then
    dimstr='3D/'
   endif
   
   if( ROCKS_STORAGE ) then
    mymod=mod(myid,16)
    if( mymod==0 ) topdir='/home/opti00/output/'
    if( mymod==1 ) topdir='/home/opti01/output/'
    if( mymod==2 ) topdir='/home/opti02/output/'
    if( mymod==3 ) topdir='/home/opti03/output/'
    if( mymod==4 ) topdir='/home/opti04/output/'
    if( mymod==5 ) topdir='/home/opti05/output/'
    if( mymod==6 ) topdir='/home/opti06/output/'
    if( mymod==7 ) topdir='/home/opti07/output/'
    if( mymod==8 ) topdir='/home/opti08/output/'
    if( mymod==9 ) topdir='/home/opti09/output/'
    if( mymod==10 ) topdir='/home/opti10/output/'
    if( mymod==11 ) topdir='/home/opti11/output/'
    if( mymod==12 ) topdir='/home/opti12/output/'
    if( mymod==13 ) topdir='/home/opti13/output/'
    if( mymod==14 ) topdir='/home/opti14/output/'
    if( mymod==15 ) topdir='/home/opti15/output/'
   else
    topdir='output/'
   endif
   
  !!Create the filename
    write(unit = cnum, fmt = 100) istep
100 format(I6.6)
    write(unit = cid, fmt = 101) myid
101 format(I3.3)

  if( trim(mode(fid)) == 'new' ) then
    if(numprocs .gt. 1) then
      filename = trim(topdir)//dimstr//trim(filename_root(fid))//'_'//cnum//'_'//cid//'.nc'
    else
      filename = trim(topdir)//dimstr//trim(filename_root(fid))//'_'//cnum//'.nc'
    endif
  elseif( trim(mode(fid)) == 'append' ) then
    if(numprocs .gt. 1) then
      filename = trim(topdir)//dimstr//trim(filename_root(fid))//'_'//cid//'.nc'
    else
      filename = trim(topdir)//dimstr//trim(filename_root(fid))//'.nc'
    endif
  endif
  !!save the current full filename
  fullname(fid)=filename
 
 
   !! Store start, end and inc for myid, 
   !! for this file, and for each direction
   start(dimid_x(fid),fid)=ilocs(1,fid)
   count(dimid_x(fid),fid)=count_vals(ilocs(1,fid),ilocs(2,fid),ilocs(3,fid))
   
   start(dimid_y(fid),fid)=jlocs(1,fid)                     
   count(dimid_y(fid),fid)=count_vals(jlocs(1,fid),jlocs(2,fid),jlocs(3,fid))

   !! Calculate k start&ends for
   !! each processor using Local indexing
   
   !! requested data COMPLETELY EXCLUDES myid
   if( klocs(2,fid) <= numzplanes_below(myid) ) then  !! request ends below my bottom slice
    my_kstart(fid)=-999  !! use as indicator that myid just waits
    my_kend(fid)=-999
    my_kinc(fid)=-999
    goto 999
   endif
   if( klocs(1,fid) > numzplanes(myid)+numzplanes_below(myid) ) then !!starts above my top slice
    my_kstart(fid)=-999  !! use as indicator that myid just waits
    my_kend(fid)=-999
    my_kinc(fid)=-999
    goto 999
   endif

   !! if I get this far, I know myid has SOME data, 
   !! several possibilities for this, deal with them individually
   
   !! requested data SPANS all of myid
   if( klocs(1,fid) <= numzplanes_below(myid) .and.  &
       klocs(2,fid) > numzplanes_below(myid)+numzplanes(myid) ) then
    k0=1
    kend=numzplanes(myid)   !! local indexing for k0/k1
    kinc=klocs(3,fid)
    kcount=count_vals(k0,kend,kinc)
    start(dimid_z(fid),fid)=k0  !! local indexing
    count(dimid_z(fid),fid)=kcount
    
   !! requested data STARTS on myid and ends above (already know myid not excluded)
   elseif( klocs(1,fid) > numzplanes_below(myid) .and.  &
           klocs(2,fid) > numzplanes_below(myid)+numzplanes(myid) ) then
    k0=klocs(1,fid)-numzplanes_below(myid)
    kend=numzplanes(myid)   !! local indexing for k0/k1
    kinc=klocs(3,fid)
    kcount=count_vals(k0,kend,kinc)
    start(dimid_z(fid),fid)=k0  !! local indexing
    count(dimid_z(fid),fid)=kcount

   !! requested data ENDS on myid and STARTS somewhere below
   elseif( klocs(1,fid) < numzplanes_below(myid) .and.  &
           klocs(2,fid) <= numzplanes_below(myid)+numzplanes(myid) ) then
    k0=1
    kend=numzplanes(myid)   !! local indexing for k0/k1
    kinc=klocs(3,fid)
    kcount=count_vals(k0,kend,kinc)
    start(dimid_z(fid),fid)=k0  !! local indexing
    count(dimid_z(fid),fid)=kcount

   !! requested data STARTS on myid and ENDS on myid
   elseif( klocs(1,fid) > numzplanes_below(myid) .and. &
           klocs(2,fid) <= numzplanes_below(myid)+numzplanes(myid) ) then
    k0=(klocs(1,fid)-numzplanes_below(myid))
    kend=(klocs(2,fid)-numzplanes_below(myid))
    kinc=klocs(3,fid)
    kcount=count_vals(k0,kend,kinc)
    start(dimid_z(fid),fid)=k0  !! local indexing
    count(dimid_z(fid),fid)=kcount
   endif
  
  !! keep track of start,end,inc 
  !! index k LOCALLY i.e. in [1,numzplanes(myid)]
   my_kstart(fid)=k0
   my_kend(fid)=kend
   my_kinc(fid)=kinc
  
   !iwrk=maxval( count(:,fid) )

!! Open a netcdf file
  rcode=NF_CREATE(filename,NF_NOCLOBBER,ncid)
  if(rcode.ne.NF_NOERR) then
   write(0,*) '... ERROR OPENING NETCDF FILE: init_netcdf ',trim(filename)
   write(0,*) '... myid, rcode ',myid, rcode
   stop
  endif

!! Spatial Dimensions Definitions
   rcode=NF_DEF_DIM(ncid,'idimension',count(dimid_x(fid),fid),iid)
   if (rcode.ne.NF_NOERR) write(0,*) myid,  &
           ': NetCDF Error: NF_DEF_DIM: idimension', rcode
    All_nD_VarSIZE(dimid_x(fid))=iid

   rcode=NF_DEF_DIM(ncid,'jdimension',count(dimid_y(fid),fid),jid)
   if (rcode.ne.NF_NOERR) write(0,*) myid,  &
           ': NetCDF Error: NF_DEF_DIM: jdimension', rcode
   All_nD_VarSIZE(dimid_y(fid))=jid

   rcode=NF_DEF_DIM(ncid,'kdimension',count(dimid_z(fid),fid),kid)
   if (rcode.ne.NF_NOERR) write(0,*) myid,   &
           ': NetCDF Error: NF_DEF_DIM: kdimension', rcode
   All_nD_VarSIZE(dimid_z(fid))=kid
   
  rcode=NF_DEF_DIM(ncid,'timedimension',NF_UNLIMITED,timeid)
  if (rcode.ne.NF_NOERR) write(0,*) myid,   &
          ': NetCDF Error: NF_DEF_DIM: timedimension', rcode
  All_nD_VarSIZE(dimid_t(fid))=timeid


!! Define x,y,z grid position  and time variables

!!**X*****
      rcode=NF_DEF_VAR(ncid,'xVar',NF_FLOAT,1,iid,xVarID)
       if (rcode.ne.NF_NOERR) print *,myid,   &
           ': NetCDF Error: NF_DEF_VAR: xVar', rcode
      rcode=NF_PUT_ATT_TEXT(ncid,xVarID,'long_name',1,'x')
       if (rcode.ne.NF_NOERR) print *,myid,   &
           ': NetCDF Error: NF_PUT_ATT_TEXT: xVar', rcode
      rcode=NF_PUT_ATT_TEXT(ncid,xVarID,'units',1,'m')
       if (rcode.ne.NF_NOERR) print *,myid,   &
           ': NetCDF Error: NF_PUT_ATT_TEXT: xVar', rcode

!!**Y*****
      rcode=NF_DEF_VAR(ncid,'yVar',NF_FLOAT,1,jid,yVarID)
       if (rcode.ne.NF_NOERR) print *,myid,   &
           ': NetCDF Error: NF_DEF_VAR: yVar', rcode
      rcode=NF_PUT_ATT_TEXT(ncid,yVarID,'long_name',1,'y')
       if (rcode.ne.NF_NOERR) print *,myid,   &
           ': NetCDF Error: NF_PUT_ATT_TEXT: yVar', rcode
      rcode=NF_PUT_ATT_TEXT(ncid,yVarID,'units',1,'m')
       if (rcode.ne.NF_NOERR) print *,myid,   &
           ': NetCDF Error: NF_PUT_ATT_TEXT: yVar', rcode
           
!!**Z*****
      rcode=NF_DEF_VAR(ncid,'zVar',NF_FLOAT,1,kid,zVarID)
       if (rcode.ne.NF_NOERR) print *,myid,   &
           ': NetCDF Error: NF_DEF_VAR: zVar', rcode
      rcode=NF_PUT_ATT_TEXT(ncid,zVarID,'long_name',1,'z')
       if (rcode.ne.NF_NOERR) print *,myid,   &
           ': NetCDF Error: NF_PUT_ATT_TEXT: zVar', rcode
      rcode=NF_PUT_ATT_TEXT(ncid,zVarID,'units',1,'m')
       if (rcode.ne.NF_NOERR) print *,myid,   &
           ': NetCDF Error: NF_PUT_ATT_TEXT: zVar', rcode

!!**time*****
      rcode=NF_DEF_VAR(ncid,'tVar',NF_FLOAT,1,timeid,tVarID)
       if (rcode.ne.NF_NOERR) print *,myid,   &
           ': NetCDF Error: NF_DEF_VAR: tVar', rcode
      rcode=NF_PUT_ATT_TEXT(ncid,tVarID,'long_name',4,'time')
       if (rcode.ne.NF_NOERR) print *,myid,   &
           ': NetCDF Error: NF_PUT_ATT_TEXT: tVar', rcode
      rcode=NF_PUT_ATT_TEXT(ncid,tVarID,'units',7,'seconds')
       if (rcode.ne.NF_NOERR) print *,myid,   &
           ': NetCDF Error: NF_PUT_ATT_TEXT: tVar', rcode     


!!     Set the appropriate labels for s1 and s2.
      if(trim(scalar_kind(1)) == 't') then
        s1_name = 'Temperature'
        s1_name_len = 11
        s1_units = 'deg C'
        s1_units_len = 5
       elseif(trim(scalar_kind(2)) == 't') then
        s2_name = 'Temperature'
        s2_name_len = 11
        s2_units = 'deg C'
        s2_units_len = 5
       endif

       if(trim(scalar_kind(1)) == 's') then
        s1_name = 'Salinity'
        s1_name_len = 8
        s1_units = 'psu'
        s1_units_len = 3
       elseif(trim(scalar_kind(2)) == 's') then
        s2_name = 'Salinity'
        s2_name_len = 8
        s2_units = 'psu'
        s2_units_len = 3
       endif
       
       if(trim(scalar_kind(1)) == 'p') then
        s1_name = 'Passive Tracer'
        s1_name_len = 14
        s1_units = 'Concentration'
        s1_units_len = 13
       elseif(trim(scalar_kind(2)) == 'p') then
        s2_name = 'Passive Tracer'
        s2_name_len = 14
        s2_units = 'Concentration'
        s2_units_len = 13
       endif
       
      
      if(trim(scalar_kind(1)) == 'r') then
        s1_name = 'Density'
        s1_name_len = 7
        s1_units = 'kg/m3'
        s1_units_len = 5
       elseif(trim(scalar_kind(2)) == 'r') then
        s1_name = 'Density'
        s1_name_len = 7
        s1_units = 'kg/m3'
        s1_units_len = 5
       endif

!!
! scalar (1)
!!   
    if( variable_key(4,fid) == 1 ) then
      rcode=NF_DEF_VAR(ncid,'s1Var',NF_FLOAT,ndims,All_nD_VarSIZE(1:ndims),s1VarID)
       if (rcode.ne.NF_NOERR) print *,myid,   &
           ': NetCDF Error: NF_DEF_VAR: s1Var', rcode
      rcode=NF_PUT_ATT_TEXT(ncid,s1VarID,'long_name',s1_name_len,s1_name)
       if (rcode.ne.NF_NOERR) print *,myid,   &
           ': NetCDF Error: NF_PUT_ATT_TEXT: s1Var', rcode
      rcode=NF_PUT_ATT_TEXT(ncid,s1VarID,'units',s1_units_len,s1_units)
       if (rcode.ne.NF_NOERR) print *,myid,   &
           ': NetCDF Error: NF_PUT_ATT_TEXT: s1Var', rcode
    endif

!!
!scalar (2)
!!
    if( variable_key(5,fid) == 1 ) then
      rcode=NF_DEF_VAR(ncid,'s2Var',NF_FLOAT,ndims,All_nD_VarSIZE,s2VarID)
       if (rcode.ne.NF_NOERR) print *,myid,   &
           ': NetCDF Error: NF_DEF_VAR: s2Var', rcode
      rcode=NF_PUT_ATT_TEXT(ncid,s2VarID,'long_name',s2_name_len,s2_name)                           
       if (rcode.ne.NF_NOERR) print *,myid,   &
           ': NetCDF Error: NF_PUT_ATT_TEXT: s2Var', rcode
      rcode=NF_PUT_ATT_TEXT(ncid,s2VarID,'units',s2_units_len,s2_units)
       if (rcode.ne.NF_NOERR) print *,myid,   &
           ': NetCDF Error: NF_PUT_ATT_TEXT: s2Var', rcode
     endif

!!
!x velocity = uVar
!!
    if( variable_key(1,fid) == 1 ) then
      rcode=NF_DEF_VAR(ncid,'uVar',NF_FLOAT,ndims,All_nD_VarSIZE,uVarID)
       if (rcode.ne.NF_NOERR) print *,myid,   &
           ': NetCDF Error: NF_DEF_VAR: uVar', rcode
      rcode=NF_PUT_ATT_TEXT(ncid,uVarID,'long_name',1,'u')
       if (rcode.ne.NF_NOERR) print *,myid,   &
           ': NetCDF Error: NF_PUT_ATT_TEXT: uVar', rcode
      rcode=NF_PUT_ATT_TEXT(ncid,uVarID,'units',3,'m/s')
       if (rcode.ne.NF_NOERR) print *,myid,   &
           ': NetCDF Error: NF_PUT_ATT_TEXT: uVar', rcode
    endif
!!
!y velocity = vVar
!! 
    if( variable_key(2,fid) == 1 ) then
      rcode=NF_DEF_VAR(ncid,'vVar',NF_FLOAT,ndims,All_nD_VarSIZE,vVarID)
       if (rcode.ne.NF_NOERR) print *,myid,   &
           ': NetCDF Error: NF_DEF_VAR: vVar', rcode
      rcode=NF_PUT_ATT_TEXT(ncid,vVarID,'long_name',1,'v')
       if (rcode.ne.NF_NOERR) print *,myid,   &
           ': NetCDF Error: NF_PUT_ATT_TEXT: vVar', rcode
      rcode=NF_PUT_ATT_TEXT(ncid,vVarID,'units',3,'m/s')
       if (rcode.ne.NF_NOERR) print *,myid,   &
           ': NetCDF Error: NF_PUT_ATT_TEXT: vVar', rcode
    endif
!!
!z velocity = wVar
!!
    if( variable_key(3,fid) == 1 ) then
      rcode=NF_DEF_VAR(ncid,'wVar',NF_FLOAT,ndims,All_nD_VarSIZE,wVarID)
       if (rcode.ne.NF_NOERR) print *,myid,   &
           ': NetCDF Error: NF_DEF_VAR: wVar', rcode
      rcode=NF_PUT_ATT_TEXT(ncid,wVarID,'long_name',1,'w')
       if (rcode.ne.NF_NOERR) print *,myid,   &
           ': NetCDF Error: NF_PUT_ATT_TEXT: wVar', rcode
      rcode=NF_PUT_ATT_TEXT(ncid,wVarID,'units',3,'m/s')
       if (rcode.ne.NF_NOERR) print *,myid,   &
           ': NetCDF Error: NF_PUT_ATT_TEXT: wVar', rcode
    endif
    
!!
!pressure = pVar
!! 
    if( variable_key(9,fid) == 1 ) then
      rcode=NF_DEF_VAR(ncid,'pVar',NF_FLOAT,ndims,All_nD_VarSIZE,pVarID)
       if (rcode.ne.NF_NOERR) print *,myid,   &
           ': NetCDF Error: NF_DEF_VAR: pVar', rcode
      rcode=NF_PUT_ATT_TEXT(ncid,pVarID,'long_name',8,'pressure')
       if (rcode.ne.NF_NOERR) print *,myid,   &
           ': NetCDF Error: NF_PUT_ATT_TEXT: pVar', rcode
      rcode=NF_PUT_ATT_TEXT(ncid,pVarID,'units',6,'kg/ms2')
       if (rcode.ne.NF_NOERR) print *,myid,   &
           ': NetCDF Error: NF_PUT_ATT_TEXT: pVar', rcode
    endif
    
!!
!divustar = divustarVar
!! 
    if( variable_key(6,fid) == 1 ) then
      rcode=NF_DEF_VAR(ncid,'divustarVar',NF_FLOAT,ndims,All_nD_VarSIZE,divustarVarID)
       if (rcode.ne.NF_NOERR) print *,myid,   &
           ': NetCDF Error: NF_DEF_VAR: divustarVar', rcode
      rcode=NF_PUT_ATT_TEXT(ncid,divustarVarID,'long_name',9,'div ustar')                            
       if (rcode.ne.NF_NOERR) print *,myid,   &
           ': NetCDF Error: NF_PUT_ATT_TEXT: divustarVar', rcode
      rcode=NF_PUT_ATT_TEXT(ncid,divustarVarID,'units',4,'1/s')
       if (rcode.ne.NF_NOERR) print *,myid,   &
           ': NetCDF Error: NF_PUT_ATT_TEXT: divustarVar', rcode
    endif
    
!!
!pressure variable = phiVar
!! 
    if( variable_key(7,fid) == 1 ) then
      rcode=NF_DEF_VAR(ncid,'phiVar',NF_FLOAT,ndims,All_nD_VarSIZE,phiVarID)
       if (rcode.ne.NF_NOERR) print *,myid,   &
           ': NetCDF Error: NF_DEF_VAR: phiVar', rcode
      rcode=NF_PUT_ATT_TEXT(ncid,phiVarID,'long_name',8,'pressure')                            
       if (rcode.ne.NF_NOERR) print *,myid,   &
           ': NetCDF Error: NF_PUT_ATT_TEXT: phiVar', rcode
      rcode=NF_PUT_ATT_TEXT(ncid,phiVarID,'units',6,'kg/ms2')
       if (rcode.ne.NF_NOERR) print *,myid,   &
           ': NetCDF Error: NF_PUT_ATT_TEXT: phiVar', rcode
    endif
    
!!
!perterbation density = pdVar
!! 
    if( variable_key(8,fid) == 1 ) then
      rcode=NF_DEF_VAR(ncid,'pdVar',NF_FLOAT,ndims,All_nD_VarSIZE,pdVarID)
       if (rcode.ne.NF_NOERR) print *,myid,   &
           ': NetCDF Error: NF_DEF_VAR: pdVar', rcode
      rcode=NF_PUT_ATT_TEXT(ncid,pdVarID,'long_name',20,'perturbation density')                            
       if (rcode.ne.NF_NOERR) print *,myid,   &
           ': NetCDF Error: NF_PUT_ATT_TEXT: pdVar', rcode
      rcode=NF_PUT_ATT_TEXT(ncid,pdVarID,'units',5,'kg/m3')
       if (rcode.ne.NF_NOERR) print *,myid,   &
           ': NetCDF Error: NF_PUT_ATT_TEXT: pdVar', rcode
    endif
    
!!
!End define mode
!!
    rcode=NF_ENDDEF(ncid)
    if(rcode.ne.NF_NOERR) print *,myid,'ERROR  LEAVING DEFINE MODE'


!    store x values in temp array
      counter = 1
      do i = ilocs(1,fid),ilocs(2,fid),ilocs(3,fid)
       scratch(counter) = Grid(0)%SpaceDims(1)%x(i)*Lz ! [m]
       counter = counter + 1
      enddo

!    Put dimension data in netcdf file
      start1D(1) = 1
      count1D(1) = count_vals(ilocs(1,fid),ilocs(2,fid),ilocs(3,fid))
      rcode=NF_PUT_VARA_REAL(ncid,xVarID,start1D,count1D,scratch)
      if(rcode.ne.NF_NOERR) print *,myid,'NetCDF ERROR xVarID: ',rcode


!    store y values in temp array
      counter = 1
      do j = jlocs(1,fid),jlocs(2,fid),jlocs(3,fid)
       scratch(counter) = Grid(0)%SpaceDims(2)%x(j)*Lz ! [m]
       counter = counter + 1
      enddo

!    Put dimension data in netcdf file
      start1D(1) = 1
      count1D(1) = count_vals( jlocs(1,fid),jlocs(2,fid),jlocs(3,fid) )
      rcode=NF_PUT_VARA_REAL(ncid,yVarID,start1D,count1D,scratch)
      if(rcode.ne.NF_NOERR) print *,myid,'NetCDF ERROR yVarID: ',rcode,filename,count1D(1)

    
     k0=k0+numzplanes_below(myid)
     kend=kend+numzplanes_below(myid)
!    store z values in temp array
      counter = 1
      do k = k0,kend,kinc
       scratch(counter) = Grid(0)%SpaceDims(3)%x(k)*Lz ! [m]
       counter = counter + 1
      enddo

!    Put dimension data in netcdf file
      start1D(1) = 1
      count1D(1) = count_vals(k0,kend,kinc)  !! had kcount
      rcode=NF_PUT_VARA_REAL(ncid,zVarID,start1D,count1D,scratch)
      if(rcode.ne.NF_NOERR) print *,myid,'NetCDF ERROR zVarID: ',rcode
        
      rcode=NF_CLOSE(ncid)
      if(rcode.ne.NF_NOERR) print *,myid,'ERROR CLOSING NETCDF FILE'
        
999   call mpi_barrier(comm,ierr)
     
      return
     end



 subroutine write_netcdf(fid)
 use user_routines
 use user_data
 use etc
 use dependent_variables
 use independent_variables
 use intermediate_variables,    &
   global_scratch_space=>scratch
 use mpi_params
 use io_params
      
 implicit none
 include 'netcdf.inc'
 !include 'mpif.h'

 character(len=80)         :: filename,err_msg
 integer                   :: Nd_start(4)=0,Nd_count(4)=0
 integer                   :: ndims,ii,jj,kk,kend,kinc,my_k
 integer                   :: icount,jcount,kcount,kstart
 real(kind=4),allocatable  :: scratch(:,:,:)
 real(kind=4)              :: xx_4
 real(kind=8)              :: zpos
     
 integer      :: fid,iid,jid,kid,timeid   
 integer      :: xVarID,yVarID,zVarID,tVarID,s1VarID,s2VarID   
 integer      :: divustarVarID,phiVarID,pdVarID
 integer      :: rhoVarID,uVarID,vVarID,wVarID,pVarID

 if( my_kstart(fid) == -999 ) then  !! no data to write
  goto 999
 endif

 !!Create the filename, init overwrote filename_root(fid) w/ full name
 filename = fullname(fid)
 
! Each processor opens netcdf file and gets variable id's.
  if( myid .eq. 0 ) then
  endif

  rcode=NF_OPEN(filename,NF_WRITE,ncid)
  if(rcode.ne.NF_NOERR) then
   write(0,*) '... ERROR OPENING NETCDF FILE: write_netcdf ',trim(filename)
   write(0,*) '... myid, rcode ',myid, rcode
   stop
  endif
   
       rcode=NF_INQ_VARID(ncid,'tVar',tVarID)
       if (rcode.ne.NF_NOERR) print *,myid,   &
         'NetCDF ERROR INQ_VARID -> tVar: ',rcode

      if( variable_key(4,fid) == 1 ) then
       rcode=NF_INQ_VARID(ncid,'s1Var',s1VarID)
       if (rcode.ne.NF_NOERR) print *,myid,   &
         'NetCDF ERROR INQ_VARID -> s1Var: ',rcode
      endif
      
      if( variable_key(5,fid) == 1 ) then
       rcode=NF_INQ_VARID(ncid,'s2Var',s2VarID)
       if (rcode.ne.NF_NOERR) print *,myid,   &
         'NetCDF ERROR INQ_VARID -> s2Var: ',rcode
      endif
      
      if( variable_key(1,fid) == 1 ) then
       rcode=NF_INQ_VARID(ncid,'uVar',uVarID)
       if (rcode.ne.NF_NOERR) print *,myid,   &
         'NetCDF ERROR INQ_VARID -> uVar: ',rcode
      endif
      
      if( variable_key(2,fid) == 1 ) then
       rcode=NF_INQ_VARID(ncid,'vVar',vVarID)
       if (rcode.ne.NF_NOERR) print *,myid,   &
         'NetCDF ERROR INQ_VARID -> vVar: ',rcode
      endif
      
      if( variable_key(3,fid) == 1 ) then
       rcode=NF_INQ_VARID(ncid,'wVar',wVarID)
       if (rcode.ne.NF_NOERR) print *,myid,   &
         'NetCDF ERROR INQ_VARID -> wVar: ',rcode
      endif
      
      if( variable_key(6,fid) == 1 ) then        
        rcode=NF_INQ_VARID(ncid,'divustarVar',divustarVarID)
        if (rcode.ne.NF_NOERR) print *,myid,   &
          'NetCDF ERROR INQ_VARID -> divustarVar: ',rcode
      endif
      
      if( variable_key(7,fid) == 1 ) then        
        rcode=NF_INQ_VARID(ncid,'phiVar',phiVarID)
        if (rcode.ne.NF_NOERR) print *,myid,   &
         'NetCDF ERROR INQ_VARID -> phiVar: ',rcode
      endif
      
      if( variable_key(8,fid) == 1 ) then        
        rcode=NF_INQ_VARID(ncid,'pdVar',pdVarID)
        if (rcode.ne.NF_NOERR) print *,myid,   &
          'NetCDF ERROR INQ_VARID -> pdVar: ',rcode
      endif
        
            
!     Write data to netcdf file. *******************************
       Nd_start(dimid_x(fid))=1
       Nd_count(dimid_x(fid))=count(dimid_x(fid),fid)
       
       Nd_start(dimid_y(fid))=1
       Nd_count(dimid_y(fid))=count(dimid_y(fid),fid)
       
       Nd_start(dimid_z(fid))=1
       Nd_count(dimid_z(fid))=count(dimid_z(fid),fid)
       
       Nd_start(dimid_t(fid))=time_counter(fid)
       Nd_count(dimid_t(fid))=1   !! always 1 time slice written per call

!     time in seconds
      xx_4 = t_secs !! convert to float
      rcode=NF_PUT_VARA_REAL(ncid,tVarID,Nd_start(dimid_t(fid)),        &
                                           Nd_count(dimid_t(fid)),xx_4)
      if(rcode.ne.NF_NOERR) print *,myid,'NetCDF ERROR tVarID: ',rcode
      if( trim(mode(fid)) == 'append' ) time_counter(fid)=time_counter(fid)+1

      allocate( scratch( Nd_count(dimid_x(fid)), &
                         Nd_count(dimid_y(fid)), &
                         Nd_count(dimid_z(fid)) ), stat=rcode )
      if(rcode /=0 ) stop 'problem allocating scratch in write_netcdf'
      
      !write(0,*) '========================='
      !write(0,*) '========================='
      !write(0,*) 'myid kstart,kend,kinc ',myid,kstart,kend,kinc
      !write(0,*) 'Nd_start  ',Nd_start
      !write(0,*) 'Nd_count  ',Nd_count

!     Scalar 1  in dimensional units.
      if( variable_key(4,fid)==1 ) then
       ii=1 ;  
       do i=ilocs(1,fid),ilocs(2,fid),ilocs(3,fid)
        jj=1;
        do j=jlocs(1,fid),jlocs(2,fid),jlocs(3,fid)
         kk=1;
         do k=my_kstart(fid),my_kend(fid),my_kinc(fid)
          
           scratch(ii,jj,kk) = (s1_bar(k) + scalars(1)%SlabVals(i,j,k))*s1_scale
          
          kk=kk+1
         enddo
         jj=jj+1
        enddo
        ii=ii+1
       enddo

!      Each processor writes its local data to 
!      the appropriate locations in the netcdf file.
       rcode=NF_PUT_VARA_REAL(ncid,s1VarID,Nd_start,Nd_count,scratch)
       if(rcode.ne.NF_NOERR) then
         print *,myid,'NetCDF ERROR: PUT_VARA_DOUBLE scalar (1)',rcode
         err_msg=NF_STRERROR(rcode)
         print*,myid, err_msg
        endif
      endif
      
!     Scalar 2  in dimensional units.
      if( variable_key(5,fid)==1 ) then
       ii=1 ;  
       do i=ilocs(1,fid),ilocs(2,fid),ilocs(3,fid)
        jj=1;
        do j=jlocs(1,fid),jlocs(2,fid),jlocs(3,fid)
         kk=1;
         do k=my_kstart(fid),my_kend(fid),my_kinc(fid)
         
          scratch(ii,jj,kk) = (s2_bar(k) + scalars(2)%SlabVals(i,j,k))*s2_scale
          
          kk=kk+1
         enddo
         jj=jj+1
        enddo
        ii=ii+1
       enddo

!      Each processor writes its local data to 
!      the appropriate locations in the netcdf file.
       rcode=NF_PUT_VARA_REAL(ncid,s2VarID,Nd_start,Nd_count,scratch)
       if(rcode.ne.NF_NOERR) then
        print *,myid,'NetCDF ERROR: PUT_VARA_DOUBLE scalar (2)',rcode
        err_msg=NF_STRERROR(rcode)
        print*,myid, err_msg
       endif
      endif
      
!     u velocity component in dimensional units.
      if( variable_key(1,fid)==1 ) then
       ii=1 ;  
       do i=ilocs(1,fid),ilocs(2,fid),ilocs(3,fid)
        jj=1;
        do j=jlocs(1,fid),jlocs(2,fid),jlocs(3,fid)
         kk=1;
         do k=my_kstart(fid),my_kend(fid),my_kinc(fid)
         
          scratch(ii,jj,kk) = velocity(1)%SlabVals(i,j,k)*U0
          
          kk=kk+1
         enddo
         jj=jj+1
        enddo
        ii=ii+1
       enddo

!      Each processor writes its local data to 
!      the appropriate locations in the netcdf file.
       rcode=NF_PUT_VARA_REAL(ncid,uVarID,Nd_start,Nd_count,scratch)
       if(rcode.ne.NF_NOERR) print *,myid,   &
         'NetCDF ERROR: PUT_VARA_DOUBLE u',rcode
      endif
      
!      v velocity component in dimensional units.
      if( variable_key(2,fid)==1 ) then
       ii=1 ;  
       do i=ilocs(1,fid),ilocs(2,fid),ilocs(3,fid)
        jj=1;
        do j=jlocs(1,fid),jlocs(2,fid),jlocs(3,fid)
         kk=1;
         do k=my_kstart(fid),my_kend(fid),my_kinc(fid)
         
          scratch(ii,jj,kk) = velocity(2)%SlabVals(i,j,k)*U0
          
          kk=kk+1
         enddo
         jj=jj+1
        enddo
        ii=ii+1
       enddo

!      Each processor writes its local data to 
!      the appropriate locations in the netcdf file.
       rcode=NF_PUT_VARA_REAL(ncid,vVarID,Nd_start,Nd_count,scratch)
       if(rcode.ne.NF_NOERR) print *,myid,   &
         'NetCDF ERROR: PUT_VARA_DOUBLE v',rcode
      endif
      
!      w velocity component in dimensional units.
      if( variable_key(3,fid)==1 ) then
       ii=1 ;  
       do i=ilocs(1,fid),ilocs(2,fid),ilocs(3,fid)
        jj=1;
        do j=jlocs(1,fid),jlocs(2,fid),jlocs(3,fid)
         kk=1;
         do k=my_kstart(fid),my_kend(fid),my_kinc(fid)
         
          scratch(ii,jj,kk) = velocity(3)%SlabVals(i,j,k)*U0
           if(fid==8) write(0,*) i,j,k,ii,jj,kk
                    
          kk=kk+1
         enddo
         jj=jj+1
        enddo
        ii=ii+1
       enddo

!      Each processor writes its local data to 
!      the appropriate locations in the netcdf file.
       
       rcode=NF_PUT_VARA_REAL(ncid,wVarID,Nd_start,Nd_count,scratch)
       if(rcode.ne.NF_NOERR) print *,myid,   &
         'NetCDF ERROR PUT_VARA_DOUBLE: w ',rcode
      endif

!      div ustar in dimensional units.
      if( variable_key(6,fid)==1 ) then
       ii=1 ;  
       do i=ilocs(1,fid),ilocs(2,fid),ilocs(3,fid)
        jj=1;
        do j=jlocs(1,fid),jlocs(2,fid),jlocs(3,fid)
         kk=1;
         do k=my_kstart(fid),my_kend(fid),my_kinc(fid)
         
          scratch(ii,jj,kk) = div_u%SlabVals(i,j,k)/(time_scale)
          
          kk=kk+1
         enddo
         jj=jj+1
        enddo
        ii=ii+1
       enddo

!      Each processor writes its local data to 
!      the appropriate locations in the netcdf file.
       rcode=NF_PUT_VARA_REAL(ncid,divustarVarID,Nd_start,Nd_count,scratch)
       if(rcode.ne.NF_NOERR) print *,myid,   &
         'NetCDF ERROR PUT_VARA_DOUBLE: div ustar ',rcode
      endif

!     pressure soln in dimensional units.
      if( variable_key(7,fid)==1 ) then
       ii=1 ;  
       do i=ilocs(1,fid),ilocs(2,fid),ilocs(3,fid)
        jj=1;
        do j=jlocs(1,fid),jlocs(2,fid),jlocs(3,fid)
         kk=1;
         do k=my_kstart(fid),my_kend(fid),my_kinc(fid)
         
          scratch(ii,jj,kk) = phi(0)%SlabVals(i,j,k)*p_scale
          
          kk=kk+1
         enddo
         jj=jj+1
        enddo
        ii=ii+1
       enddo

!      Each processor writes its local data to 
!      the appropriate locations in the netcdf file.
       rcode=NF_PUT_VARA_REAL(ncid,phiVarID,Nd_start,Nd_count,scratch)
       if(rcode.ne.NF_NOERR) print *,myid,   &
         'NetCDF ERROR PUT_VARA_DOUBLE: phi ',rcode
      endif
      
!     perturbation density in dimensional units.
      if( variable_key(8,fid)==1 ) then
       ii=1 ;  
       do i=ilocs(1,fid),ilocs(2,fid),ilocs(3,fid)
        jj=1;
        do j=jlocs(1,fid),jlocs(2,fid),jlocs(3,fid)
         kk=1;
         do k=my_kstart(fid),my_kend(fid),my_kinc(fid)
         
          scratch(ii,jj,kk) = pd%SlabVals(i,j,k)*Lz*DGRAD
          
          kk=kk+1
         enddo
         jj=jj+1
        enddo
        ii=ii+1
       enddo

!      Each processor writes its local data to 
!      the appropriate locations in the netcdf file.
       rcode=NF_PUT_VARA_REAL(ncid,pdVarID,Nd_start,Nd_count,scratch)
       if(rcode.ne.NF_NOERR) print *,myid,   &
         'NetCDF ERROR PUT_VARA_DOUBLE: pd ',rcode
      endif

!      Each processor closes the netcdf file.
       rcode=NF_CLOSE(ncid)
       if(rcode.ne.NF_NOERR) print *,myid,   &
         'NetCDF ERROR: closing file in write3D_mpi_netcdf',rcode
      deallocate( scratch )

999   call mpi_barrier(comm,rcode)
      return
      end

subroutine write_particle_data
  use particles
  use etc,                   only: t_secs
  use user_data,             only: Lz
  use mpi_params,            only: myid,comm
      
  implicit none
  include 'netcdf.inc'
  
  logical, SAVE      :: first_entry=.TRUE.
  character(len=80)  :: filename
  integer            :: iid  !!dimension id for particles
  integer            :: jid  !!dimension id for time
  integer            :: idimensionSIZE(1) !!arrays for  dimension sizes
  integer            :: jdimensionSIZE(1) 
  integer            :: idimensionID !! for dimension variable "float id"
  integer            :: jdimensionID !! for dimension variable "time"
  integer            :: rcode, ierr
  integer            :: ncid
      
!!Variables: x(particle id, time),y(particle id, time),z(particle id, time)
  integer                  :: All2DVarSIZE(2),start2D(2),count2D(2)
  integer                  :: xVarID,yVarID,zVarID   !! 2D Variable ids
  integer                  :: istart,jstart,icount,jcount,i
  integer,allocatable,save      :: idata(:)
  real(kind=4),allocatable,save :: xdata(:)
  real(kind=4)                  :: xx_4
  LOGICAL                       :: ROCKS_STORAGE=.FALSE.
  
  call mpi_barrier(comm,ierr)
  
  if(myid>0) goto 999  !! and wait there

  if( ROCKS_STORAGE ) THEN
   filename='/home/opti15/output/particles.nc'
  else
   filename='output/particles.nc'
  endif
    
  
 if( first_entry ) then
  
  allocate( idata(nparticles), stat=rcode )
  if( rcode /= 0 ) &
   stop 'problem allocating idata in write_particle_data'
  allocate( xdata(nparticles), stat=rcode  )
  if( rcode /= 0 ) &
   stop 'problem allocating idata in write_particle_data'
 
  !!write(0,*) 'creating particle file ',t_secs,init_particle_file
  rcode = NF_CREATE(filename, NF_NOCLOBBER, ncid) 
  if(rcode.ne.NF_NOERR) then
   write(0,*) 'ERROR CREATING NETCDF FILE FOR PARTICLES ',filename
   stop
  endif

!!Dimensions Definition
  rcode = NF_DEF_DIM(ncid, 'idimension', nparticles, iid)
  if(rcode.ne.NF_NOERR) write(0,*) 'init for particles NetCDF ERROR: ',rcode

  rcode = NF_DEF_DIM(ncid, 'jdimension', NF_UNLIMITED, jid)
  if(rcode.ne.NF_NOERR) print *,'init for particles NetCDF ERROR: ',rcode

!!Initialize size arrays for dimensions
  idimensionSIZE(1) = iid
  jdimensionSIZE(1) = jid

!!Define dimension variables
  rcode = NF_DEF_VAR(ncid,'idimension',NF_INT,1,idimensionSIZE,idimensionID)
  if(rcode.ne.NF_NOERR) print *,'NetCDF ERROR: ',rcode
  rcode=NF_PUT_ATT_TEXT(ncid,idimensionID,'long_name',11,'particle id')
  !!rcode=NF_PUT_ATT_TEXT(ncid,idimensionID,'units',3,'[1]')

  rcode = NF_DEF_VAR(ncid,'jdimension',NF_FLOAT,1,jdimensionSIZE,jdimensionID)
  if(rcode.ne.NF_NOERR) print *,'NetCDF ERROR: ',rcode
  rcode=NF_PUT_ATT_TEXT(ncid,jdimensionID,'long_name',4,'time')
  rcode=NF_PUT_ATT_TEXT(ncid,jdimensionID,'units',7,'seconds')

!! Define 2D position variables
!!Initialize size arrays for id/time varying variables
  All2DVarSIZE(1) = iid
  All2DVarSIZE(2) = jid


!! x particle position = xVar
  rcode = NF_DEF_VAR(ncid,'xVar',NF_FLOAT,2,All2DVarSIZE,xVarID)
  if(rcode.ne.NF_NOERR) print *,'NetCDF ERROR: ',rcode
  rcode=NF_PUT_ATT_TEXT(ncid,xVarID,'long_name',19,'x particle position')                      
  rcode=NF_PUT_ATT_TEXT(ncid,xVarID,'units',1,'m')
  
!! y particle position = yVar
  rcode = NF_DEF_VAR(ncid,'yVar',NF_FLOAT,2,All2DVarSIZE,yVarID)
  if(rcode.ne.NF_NOERR) print *,'NetCDF ERROR: ',rcode
  rcode=NF_PUT_ATT_TEXT(ncid,yVarID,'long_name',19,'y particle position')                      
  rcode=NF_PUT_ATT_TEXT(ncid,yVarID,'units',1,'m')
  
!! z particle position = zVar
  rcode = NF_DEF_VAR(ncid,'zVar',NF_FLOAT,2,All2DVarSIZE,zVarID)
  if(rcode.ne.NF_NOERR) print *,'NetCDF ERROR: ',rcode
  rcode=NF_PUT_ATT_TEXT(ncid,zVarID,'long_name',19,'z particle position')                      
  rcode=NF_PUT_ATT_TEXT(ncid,zVarID,'units',1,'m')
     
!!End define mode
  rcode=NF_ENDDEF(ncid)
  if(rcode.ne.NF_NOERR) print *,'ERROR  LEAVING DEFINE MODE'
  

!!Write out dimension variable: particle id
!!store values in temp array 
  do i = 1,nparticles
   idata(i) = i   !!--> integer
  enddo

!! Put idimension data, i.e. particle id, into netcdf file
  istart = 1
  icount = nparticles
  rcode=NF_PUT_VARA_INT(ncid,idimensionID,istart,icount,idata)
  if(rcode.ne.NF_NOERR) print *,'NetCDF ERROR storing particle ids: ',rcode

!!Close netcdf file
  rcode=NF_CLOSE(ncid)
  if(rcode.ne.NF_NOERR) print *,'ERROR CLOSING particle NETCDF FILE'
  
  first_entry=.FALSE.

endif  ! end file initialization 


!!=========================================================================

!! Open the previously created netcdf data file
  rcode=NF_OPEN(filename,NF_WRITE,ncid)
  if(rcode.ne.NF_NOERR) then 
   print *,'ERROR OPENING NETCDF FILE: write_netcdf ',filename,ncid
   stop
  endif
   


!!Write out current value of dimension variable time
  jstart = j_particle_time
  jcount = 1
  rcode=NF_INQ_VARID(ncid,'jdimension',jdimensionID)
  if(rcode.ne.NF_NOERR) print *,'NetCDF ERROR: NF_INQ_VARID ',rcode
  
  xx_4=t_secs  !! convert to float
  rcode=NF_PUT_VARA_REAL(ncid,jdimensionID,jstart,jcount,xx_4)
  if(rcode.ne.NF_NOERR) print *,'NetCDF ERROR: NF_PUT_VARA_REAL',rcode

!!Write 2D data to netcdf file

!!Initialize size arrays
  start2D(1) = 1
  start2D(2) = j_particle_time
  count2D(1) = nparticles
  count2D(2) = 1

!!Store x float position variable xVar
  do  i=1,nparticles
   xdata(i) = positions(i,1)*Lz
  enddo

!!Write variable to netcdf file
  rcode=NF_INQ_VARID(ncid,'xVar',xVarID)
  rcode=NF_PUT_VARA_REAL(ncid,xVarID,start2D,count2D,xdata)
  if(rcode.ne.NF_NOERR) print *,'NetCDF ERROR: ',rcode

!!Store y float position variable yVar
  do  i=1,nparticles
   xdata(i) = positions(i,2)*Lz
  enddo

!!Write variable to netcdf file
   rcode=NF_INQ_VARID(ncid,'yVar',yVarID)
   rcode=NF_PUT_VARA_REAL(ncid,yVarID,start2D,count2D,xdata)
   if(rcode.ne.NF_NOERR) print *,'NetCDF ERROR: ',rcode

!!Store z float position variable zVar
  do  i=1,nparticles
   xdata(i) = positions(i,3)*Lz
  enddo

!!Write variable to netcdf file
  rcode=NF_INQ_VARID(ncid,'zVar',zVarID)
  rcode=NF_PUT_VARA_REAL(ncid,zVarID,start2D,count2D,xdata)
  if(rcode.ne.NF_NOERR) print *,'NetCDF ERROR: ',rcode

!! Close netcdf file
  rcode=NF_CLOSE(ncid)
  if(rcode.ne.0) print *,'ERROR CLOSING NETCDF FILE'
  
!!increment integer time index
  j_particle_time = j_particle_time + 1 

999 call mpi_barrier(comm,ierr)  !! sync up here, only 0 did the writing
 return
end subroutine write_particle_data



subroutine nondimensionalize_s1(s1,s1_z,s1_zz)
 use user_data
 implicit none
 real(kind=8)    :: s1,s1_z,s1_zz       
 s1=s1/s1_scale
 s1_z=s1_z/(s1_scale/Lz)
 s1_zz=s1_zz/(s1_scale/Lz**2)       
end subroutine nondimensionalize_s1
      
subroutine nondimensionalize_s2(s2,s2_z,s2_zz)
 use user_data
 implicit none
 real(kind=8)    :: s2,s2_z,s2_zz       
 s2=s2/s2_scale
 s2_z=s2_z/(s2_scale/Lz)
 s2_zz=s2_zz/(s2_scale/Lz**2)       
end subroutine nondimensionalize_s2

integer function count_vals(i0,i1,inc) result(numvals)
  integer, intent(in)  :: i0,i1,inc
  integer              :: n
  numvals = floor( float(i1-i0)/inc ) + 1
end function count_vals
 
 subroutine tic( start,rate )
  implicit none
  integer   :: start  ! current value of system clock
  integer   :: rate   ! system clock counts/sec
  call system_clock(start,rate)
 end subroutine tic
 
 subroutine toc( start,rate,sec )
  implicit none
  integer           :: start  ! original value of system clock
  integer           :: finish ! current value of system clock
  integer           :: rate   ! system clock counts/sec
  real(kind=8)      :: sec
  call system_clock(finish)
  sec=0.
  sec=float(finish-start)/float(rate)
 end subroutine toc

