subroutine apply_BCs
#define DEBUG_LEVEL 1

! apply BCs to intermediate velocities and pd
 use grid_info
 use dependent_variables
 use intermediate_variables
 use mpi_parameters
 use boundary_information
 use counters_flags_etc
 use user_parameters   ! for surface_V_of_x
 use dimensional_scales
 implicit none
 include '../input/problem_size.h'
 include 'mpif.h'

 real, dimension(:,:), allocatable            :: f0,f1,f2,f3,f4
 real                                         :: h, xval,F_of_t,thresh

#if DEBUG_LEVEL >= 1
 if(myid==0) write(0,*) 'hello world from apply_BCs'
#endif

! FACE 1: xi=0
#if DEBUG_LEVEL >= 2
 write(0,*) 'apply_BCs, FACE 1',myid
#endif

 h=Grid(0)%dxi
 allocate (f0(ny,locnz),f1(ny,locnz),f2(ny,locnz),f3(ny,locnz),f4(ny,locnz))
 i=1
 if( BC_vel1(1,1) == 'specified_value' ) then
  U(i,1:ny,1:locnz)=BC_values1(1:ny,1:locnz,1)
  v(i,1:ny,1:locnz)=BC_values1(1:ny,1:locnz,2)
  W(i,1:ny,1:locnz)=BC_values1(1:ny,1:locnz,3)
 elseif( BC_vel1(1,1) == 'specified_stress' ) then
  if( boundary_type1(1,1) == 'solid' ) then
   U(i,1:ny,1:locnz)=BC_values1(1:ny,1:locnz,1)
  elseif( boundary_type1(1,1) == 'permeable' ) then
   f1=U(2,:,:)
   f2=U(3,:,:)
   f3=U(4,:,:)
   f4=U(5,:,:)
   f0=(-24.*h*BC_values1(:,:,1) + 96.*f1 -72.*f2 +32.*f3 -6.*f4)/50.
   U(i,1:ny,1:locnz)=f0
  endif

  f1=v(2,:,:)
  f2=v(3,:,:)
  f3=v(4,:,:)
  f4=v(5,:,:)
  f0=(-24.*h*BC_values1(:,:,2) + 96.*f1 -72.*f2 +32.*f3 -6.*f4)/50.
  v(i,1:ny,1:locnz)=f0
  f1=w(2,:,:)
  f2=w(3,:,:)
  f3=w(4,:,:)
  f4=w(5,:,:)
  f0=(-24.*h*BC_values1(:,:,3) + 96.*f1 -72.*f2 +32.*f3 -6.*f4)/50.
  w(i,1:ny,1:locnz)=f0
 endif

! bcs for scalar 1 at face 1
 if( BC_scalar1(1,1,1) == 'specified_value' ) then
  s1(i,1:ny,1:locnz)=BC_values1(1:ny,1:locnz,4)
 elseif( BC_scalar1(1,1,1) == 'specified_gradient' ) then
  f1=s1(2,:,:)
  f2=s1(3,:,:)
  f3=s1(4,:,:)
  f4=s1(5,:,:)
  f0=(-24.*h*BC_values1(:,:,4) + 96.*f1 -72.*f2 +32.*f3 -6.*f4)/50.
  s1(i,1:ny,1:locnz)=f0
 endif

! bcs for scalar 2 at face 1
if(num_scalars>1) then
 if( BC_scalar1(1,1,2) == 'specified_value' ) then
  s2(i,1:ny,1:locnz)=BC_values1(1:ny,1:locnz,5)
 elseif( BC_scalar1(1,1,2) == 'specified_gradient' ) then
  f1=s2(2,:,:)
  f2=s2(3,:,:)
  f3=s2(4,:,:)
  f4=s2(5,:,:)
  f0=(-24.*h*BC_values1(:,:,5) + 96.*f1 -72.*f2 +32.*f3 -6.*f4)/50.
  s2(i,1:ny,1:locnz)=f0
 endif
endif


! FACE 2: xi=1
#if DEBUG_LEVEL >= 2
 write(0,*) 'apply_BCs, FACE 2',myid
#endif

 i=nx
 if( BC_vel2(1,1) == 'specified_value' ) then
  U(i,1:ny,1:locnz)=BC_values2(1:ny,1:locnz,1)
  v(i,1:ny,1:locnz)=BC_values2(1:ny,1:locnz,2)
  W(i,1:ny,1:locnz)=BC_values2(1:ny,1:locnz,3)
 elseif( BC_vel2(1,1) == 'specified_stress' ) then
  if( boundary_type2(1,1) == 'solid' ) then
   U(i,1:ny,1:locnz)=BC_values2(1:ny,1:locnz,1)
  elseif( boundary_type2(1,1) == 'permeable' ) then
   f0=U(nx-4,:,:)
   f1=U(nx-3,:,:)
   f2=U(nx-2,:,:)
   f3=U(nx-1,:,:)
   f4=(24.*h*BC_values2(:,:,1) -6.*f0 +32.*f1 -72.*f2 +96.*f3)/50.
   U(i,1:ny,1:locnz)=f0
  endif

   f0=v(nx-4,:,:)
   f1=v(nx-3,:,:)
   f2=v(nx-2,:,:)
   f3=v(nx-1,:,:)
   f4=(24.*h*BC_values2(:,:,2) -6.*f0 +32.*f1 -72.*f2 +96.*f3)/50.
   v(i,1:ny,1:locnz)=f4
   f0=w(nx-4,:,:)
   f1=w(nx-3,:,:)
   f2=w(nx-2,:,:)
   f3=w(nx-1,:,:)
   f4=(24.*h*BC_values2(:,:,3) -6.*f0 +32.*f1 -72.*f2 +96.*f3)/50.
   w(i,1:ny,1:locnz)=f4
 endif

 if( BC_scalar2(1,1,1) == 'specified_value' ) then
  s1(i,1:ny,1:locnz)=BC_values2(1:ny,1:locnz,4)
 elseif( BC_scalar2(1,1,1) == 'specified_gradient' ) then
  f0=s1(nx-4,:,:)
  f1=s1(nx-3,:,:)
  f2=s1(nx-2,:,:)
  f3=s1(nx-1,:,:)
  f4=(24.*h*BC_values2(:,:,4) -6.*f0 +32.*f1 -72.*f2 +96.*f3)/50.
  s1(i,1:ny,1:locnz)=f4
 endif

if(num_scalars>1) then
 if( BC_scalar2(1,1,2) == 'specified_value' ) then
  s2(i,1:ny,1:locnz)=BC_values2(1:ny,1:locnz,5)
 elseif( BC_scalar2(1,1,2) == 'specified_gradient' ) then
  f0=s2(nx-4,:,:)
  f1=s2(nx-3,:,:)
  f2=s2(nx-2,:,:)
  f3=s2(nx-1,:,:)
  f4=(24.*h*BC_values2(:,:,5) -6.*f0 +32.*f1 -72.*f2 +96.*f3)/50.
  s2(i,1:ny,1:locnz)=f4
 endif
endif

 deallocate (f0,f1,f2,f3,f4)


! FACE 3: eta=0
#if DEBUG_LEVEL >= 2
 write(0,*) 'apply_BCs, FACE 3',myid
#endif

 h=Grid(0)%deta
 allocate (f0(nx,locnz),f1(nx,locnz),f2(nx,locnz),f3(nx,locnz),f4(nx,locnz))
 j=1
 if( BC_vel3(1,1) == 'specified_value' ) then
  U(1:nx,j,1:locnz)=BC_values3(1:nx,1:locnz,1)
  v(1:nx,j,1:locnz)=BC_values3(1:nx,1:locnz,2)
  W(1:nx,j,1:locnz)=BC_values3(1:nx,1:locnz,3)
 elseif( BC_vel3(1,1) == 'specified_stress' ) then
  if( boundary_type3(1,1) == 'solid' ) then
   v(1:nx,j,1:locnz)=BC_values3(1:nx,1:locnz,2)
  elseif( boundary_type3(1,1) == 'permeable' ) then
   f1=v(1:nx,2,1:locnz)
   f2=v(1:nx,3,1:locnz)
   f3=v(1:nx,4,1:locnz)
   f4=v(1:nx,5,1:locnz)
   f0=(-24.*h*BC_values3(:,:,2) + 96.*f1 -72.*f2 +32.*f3 -6.*f4)/50.
   v(1:nx,j,1:locnz)=f0
  endif

   f1=U(1:nx,2,1:locnz)
   f2=U(1:nx,3,1:locnz)
   f3=U(1:nx,4,1:locnz)
   f4=U(1:nx,5,1:locnz)
   f0=(-24.*h*BC_values3(:,:,1) + 96.*f1 -72.*f2 +32.*f3 -6.*f4)/50.
   U(1:nx,j,1:locnz)=f0
   f1=W(1:nx,2,1:locnz)
   f2=W(1:nx,3,1:locnz)
   f3=W(1:nx,4,1:locnz)
   f4=W(1:nx,5,1:locnz)
   f0=(-24.*h*BC_values3(:,:,3) + 96.*f1 -72.*f2 +32.*f3 -6.*f4)/50.
   W(1:nx,j,1:locnz)=f0
 endif

 if( BC_scalar3(1,1,1) == 'specified_value' ) then
  s1(1:nx,j,1:locnz)=BC_values3(1:nx,1:locnz,4)
 elseif( BC_scalar3(1,1,1) == 'specified_gradient' ) then
  f1=s1(1:nx,2,1:locnz)
  f2=s1(1:nx,3,1:locnz)
  f3=s1(1:nx,4,1:locnz)
  f4=s1(1:nx,5,1:locnz)
  f0=(-24.*h*BC_values3(:,:,4) + 96.*f1 -72.*f2 +32.*f3 -6.*f4)/50.
  s1(1:nx,j,1:locnz)=f0
 endif

if(num_scalars>1) then
 if( BC_scalar3(1,1,2) == 'specified_value' ) then
  s2(1:nx,j,1:locnz)=BC_values3(1:nx,1:locnz,5)
 elseif( BC_scalar3(1,1,2) == 'specified_gradient' ) then
  f1=s2(1:nx,2,1:locnz)
  f2=s2(1:nx,3,1:locnz)
  f3=s2(1:nx,4,1:locnz)
  f4=s2(1:nx,5,1:locnz)
  f0=(-24.*h*BC_values3(:,:,5) + 96.*f1 -72.*f2 +32.*f3 -6.*f4)/50.
  s2(1:nx,j,1:locnz)=f0
 endif
endif

! FACE 4: eta=1
#if DEBUG_LEVEL >= 2
 write(0,*) 'apply_BCs, FACE 4',myid
#endif

 j=ny
 if( BC_vel4(1,1) == 'specified_value' ) then
  U(1:nx,j,1:locnz)=BC_values4(1:nx,1:locnz,1)
  v(1:nx,j,1:locnz)=BC_values4(1:nx,1:locnz,2)
  W(1:nx,j,1:locnz)=BC_values4(1:nx,1:locnz,3)
 elseif( BC_vel4(1,1) == 'specified_stress' ) then
  if( boundary_type4(1,1) == 'solid' ) then
   v(1:nx,j,1:locnz)=BC_values4(1:nx,1:locnz,2)
  elseif( boundary_type4(1,1) == 'permeable' ) then
   f0=v(1:nx,ny-4,1:locnz)
   f1=v(1:nx,ny-3,1:locnz)
   f2=v(1:nx,ny-2,1:locnz)
   f3=v(1:nx,ny-1,1:locnz)
   f4=(24.*h*BC_values4(:,:,2) -6.*f0 +32.*f1 -72.*f2 +96.*f3)/50.
   v(1:nx,j,1:locnz)=f4
  endif

   f0=U(1:nx,ny-4,1:locnz)
   f1=U(1:nx,ny-3,1:locnz)
   f2=U(1:nx,ny-2,1:locnz)
   f3=U(1:nx,ny-1,1:locnz)
   f4=(24.*h*BC_values4(:,:,1) -6.*f0 +32.*f1 -72.*f2 +96.*f3)/50.
   U(1:nx,j,1:locnz)=f4
   f0=W(1:nx,ny-4,1:locnz)
   f1=W(1:nx,ny-3,1:locnz)
   f2=W(1:nx,ny-2,1:locnz)
   f3=W(1:nx,ny-1,1:locnz)
   f4=(24.*h*BC_values4(:,:,3) -6.*f0 +32.*f1 -72.*f2 +96.*f3)/50.
   W(1:nx,j,1:locnz)=f4
 endif

 if( BC_scalar4(1,1,1) == 'specified_value' ) then
  s1(1:nx,j,1:locnz)=BC_values4(1:nx,1:locnz,4)
 elseif( BC_scalar4(1,1,1) == 'specified_gradient' ) then
  f0=s1(1:nx,ny-4,1:locnz)
  f1=s1(1:nx,ny-3,1:locnz)
  f2=s1(1:nx,ny-2,1:locnz)
  f3=s1(1:nx,ny-1,1:locnz)
  f4=(24.*h*BC_values4(:,:,4) -6.*f0 +32.*f1 -72.*f2 +96.*f3)/50.
  s1(1:nx,j,1:locnz)=f4
 endif

if(num_scalars>1) then
 if( BC_scalar4(1,1,2) == 'specified_value' ) then
  s2(1:nx,j,1:locnz)=BC_values4(1:nx,1:locnz,5)
 elseif( BC_scalar4(1,1,2) == 'specified_gradient' ) then
  f0=s2(1:nx,ny-4,1:locnz)
  f1=s2(1:nx,ny-3,1:locnz)
  f2=s2(1:nx,ny-2,1:locnz)
  f3=s2(1:nx,ny-1,1:locnz)
  f4=(24.*h*BC_values4(:,:,5) -6.*f0 +32.*f1 -72.*f2 +96.*f3)/50.
  s2(1:nx,j,1:locnz)=f4
 endif
endif

 deallocate (f0,f1,f2,f3,f4)


! FACE 5: zeta=0
#if DEBUG_LEVEL >= 2
 write(0,*) 'apply_BCs, FACE 5',myid
#endif

 if( myid == 0 ) then
  h=Grid(0)%dzeta
  allocate (f0(nx,ny),f1(nx,ny),f2(nx,ny),f3(nx,ny),f4(nx,ny))
  k=1
  if( BC_vel5(1,1) == 'specified_value' ) then
   U(1:nx,1:ny,k)=BC_values5(1:nx,1:ny,1)
   v(1:nx,1:ny,k)=BC_values5(1:nx,1:ny,2)
   W(1:nx,1:ny,k)=BC_values5(1:nx,1:ny,3)
  elseif( BC_vel5(1,1) == 'specified_stress' ) then
   if( boundary_type5(1,1) == 'solid' ) then
    W(1:nx,1:ny,k)=BC_values5(1:nx,1:ny,3)
   elseif( boundary_type5(1,1) == 'permeable' ) then
!!!    f1=W(:,:,2)
!!!    f2=W(:,:,3)
!!!    f3=W(:,:,4)
!!!    f4=W(:,:,5)
!!!    f0=(-24.*h*BC_values5(:,:,3) + 96.*f1 -72.*f2 +32.*f3 -6.*f4)/50.
!!!    W(:,:,k)=f0
      W(:,:,k)=-9999.
   endif

    f1=U(:,:,2)
    f2=U(:,:,3)
    f0 = (-2.*h*BC_values5(:,:,1) + 4.*f1 - f2) / 3.
!!!    f3=U(:,:,4)
!!!    f4=U(:,:,5)
!!!    f0=(-24.*h*BC_values5(:,:,1) + 96.*f1 -72.*f2 +32.*f3 -6.*f4)/50.
    U(:,:,k)=f0

    f1=v(:,:,2)
    f2=v(:,:,3)
    f0 = (-2.*h*BC_values5(:,:,2) + 4.*f1 - f2) / 3.
!!!    f3=v(:,:,4)
!!!    f4=v(:,:,5)
!!!    f0=(-24.*h*BC_values5(:,:,2) + 96.*f1 -72.*f2 +32.*f3 -6.*f4)/50.
    v(:,:,k)=f0
  endif

  if( BC_scalar5(1,1,1) == 'specified_value' ) then
   s1(1:nx,1:ny,k)=BC_values5(1:nx,1:ny,4)
  elseif( BC_scalar5(1,1,1) == 'specified_gradient' ) then
   f1=s1(:,:,2)
   f2=s1(:,:,3)
   f0 = (-2.*h*BC_values5(:,:,4) + 4.*f1 - f2) / 3.
!!!   f3=s1(:,:,4)
!!!   f4=s1(:,:,5)
!!!   f0=(-24.*h*BC_values5(:,:,4) + 96.*f1 -72.*f2 +32.*f3 -6.*f4)/50.
   s1(:,:,k)=f0
  endif

 if(num_scalars>1) then
  if( BC_scalar5(1,1,2) == 'specified_value' ) then
   s2(1:nx,1:ny,k)=BC_values5(1:nx,1:ny,5)
  elseif( BC_scalar5(1,1,2) == 'specified_gradient' ) then
   f1=s2(:,:,2)
   f2=s2(:,:,3)
   f0 = (-2.*h*BC_values5(:,:,5) + 4.*f1 - f2) / 3.
!!!   f3=s2(:,:,4)
!!!   f4=s2(:,:,5)
!!!   f0=(-24.*h*BC_values5(:,:,5) + 96.*f1 -72.*f2 +32.*f3 -6.*f4)/50.
   s2(:,:,k)=f0
  endif
 endif

  deallocate (f0,f1,f2,f3,f4)
 endif ! myid==0 block

! FACE 6: zeta=1
#if DEBUG_LEVEL >= 2
 write(0,*) 'apply_BCs, FACE 6',myid
#endif

 if( myid == numprocs-1 ) then
  h=Grid(0)%dzeta
  allocate (f0(nx,ny),f1(nx,ny),f2(nx,ny),f3(nx,ny),f4(nx,ny))
  k=locnz

!**** ENFORCE STRESS FREE CONDITION OVER ENTIRE UPPER SURFACE ****************
  if( BC_vel6(1,1) == 'specified_value' ) then
   U(1:nx,1:ny,k)=BC_values6(1:nx,1:ny,1)
   v(1:nx,1:ny,k)=BC_values6(1:nx,1:ny,2)
   W(1:nx,1:ny,k)=BC_values6(1:nx,1:ny,3)
  elseif( BC_vel6(1,1) == 'specified_stress' ) then
   if( boundary_type6(1,1) == 'solid' ) then
    W(1:nx,1:ny,k)=BC_values6(1:nx,1:ny,3)
   elseif( boundary_type6(1,1) == 'permeable' ) then
!!!    f0=W(:,:,k-4)
!!!    f1=W(:,:,k-3)
!!!    f2=W(:,:,k-2)
!!!    f3=W(:,:,k-1)
!!!    f4=(24.*h*BC_values6(:,:,3) -6.*f0 +32.*f1 -72.*f2 +96.*f3)/50.
!!!    W(:,:,k)=f4
    W(:,:,k) = -9999.
   endif

!!!    f0=U(:,:,k-4)
!!!    f1=U(:,:,k-3)
!!!    f2=U(:,:,k-2)
!!!    f3=U(:,:,k-1)
!!!    f4=(24.*h*BC_values6(:,:,1) -6.*f0 +32.*f1 -72.*f2 +96.*f3)/50.
!!!    U(:,:,k)=f4
    f0=U(:,:,k-2)
    f1=U(:,:,k-1)
    f2 = (2.*h*BC_values6(:,:,1) + 4.*f1 - f0) / 3.
    U(:,:,k)=f2

!!!    f0=v(:,:,k-4)
!!!    f1=v(:,:,k-3)
!!!    f2=v(:,:,k-2)
!!!    f3=v(:,:,k-1)
!!!    f4=(24.*h*BC_values6(:,:,2) -6.*f0 +32.*f1 -72.*f2 +96.*f3)/50.
!!!    v(:,:,k)=f4      ! set dV/dz everywhere on top surface
    f0=v(:,:,k-2)
    f1=v(:,:,k-1)
    f2 = (2.*h*BC_values6(:,:,2) + 4.*f1 - f0) / 3.
    v(:,:,k)=f2
   endif
!**********************************************************************

!*** NOW, OVER-WRITE THE VALUES IN [0,x_1] and [x_2,L_x] with the ICE BCS***
   F_of_t = 1.0    ! dimensional time in [s] stored in the scalar variable "t"
   thresh=1.e-4    ! dimensionless value
   do i=1,nx
    xval = Grid(0)%x(i,locnz)*L_scale
    if( abs(surface_V_of_x(i)) >= thresh .or. xval <= x_1 ) then ! SHEARED ICE MOTION
     u(i,:,k) = 0.
     v(i,:,k) = F_of_t*surface_V_of_x(i)
    endif
    if( xval >= x_2 ) then  ! stationary ice
     u(i,:,k) = 0.
     v(i,:,k) = 0.
    endif
   enddo
!**************************************************************************


  if( BC_scalar6(1,1,1) == 'specified_value' ) then
   s1(1:nx,1:ny,k)=BC_values6(1:nx,1:ny,4)
  elseif( BC_scalar6(1,1,1) == 'specified_gradient' ) then
!!!   f0=s1(:,:,k-4)
!!!   f1=s1(:,:,k-3)
!!!   f2=s1(:,:,k-2)
!!!   f3=s1(:,:,k-1)
!!!   f4=(24.*h*BC_values6(:,:,4) -6.*f0 +32.*f1 -72.*f2 +96.*f3)/50.
!!!   s1(:,:,k)=f4
   f0=s1(:,:,k-2)
   f1=s1(:,:,k-1)
   f2 = (2.*h*BC_values6(:,:,4) + 4.*f1 - f0) / 3.
   s1(:,:,k)=f2
  endif

 if(num_scalars>1) then
  if( BC_scalar6(1,1,2) == 'specified_value' ) then
   s2(1:nx,1:ny,k)=BC_values6(1:nx,1:ny,5)
  elseif( BC_scalar6(1,1,2) == 'specified_gradient' ) then
!!!   f0=s2(:,:,k-4)
!!!   f1=s2(:,:,k-3)
!!!   f2=s2(:,:,k-2)
!!!   f3=s2(:,:,k-1)
!!!   f4=(24.*h*BC_values6(:,:,5) -6.*f0 +32.*f1 -72.*f2 +96.*f3)/50.
!!!   s2(:,:,k)=f4
   f0=s2(:,:,k-2)
   f1=s2(:,:,k-1)
   f2 = (2.*h*BC_values6(:,:,5) + 4.*f1 - f0) / 3.
   s2(:,:,k)=f2
  endif
 endif

  deallocate (f0,f1,f2,f3,f4)
 endif

end subroutine apply_BCs
