subroutine apply_ibdry_rhs
#define DEBUG_LEVEL 2

  use grid_info
  use mpi_parameters
  use immersed_boundary
  use counters_flags_etc
  use rhs_variables
  use dependent_variables
  implicit none

  include '../input/problem_size.h'
  include 'mpif.h'

  integer		:: ipt,npts,i1,i2,j1,j2,k1,k2,update,m1,m2
  real			:: delta,delta_n,xloc,zloc,n1,n2

#if DEBUG_LEVEL >= 1
  if(myid==0) then
   write(0,*) ' '
   write(0,*) ' '
   write(0,*) 'hello world from apply_ibdry_rhs'
  endif
#endif

  ! damp the flow below immersed boundary
  do j=1,ny
   rhs1(:,j,:,MM0) = rhs1(:,j,:,MM0) - xbeta*U(:,j,:)*(1-mask2(:,:))
   rhs2(:,j,:,MM0) = rhs2(:,j,:,MM0) - xbeta*v(:,j,:)*(1-mask2(:,:))
   rhs3(:,j,:,MM0) = rhs3(:,j,:,MM0) - xbeta*W(:,j,:)*(1-mask2(:,:))
  enddo


  npts=n_j*ny
  do ipt=1,npts

   i1=local_region(1,ipt)
   i2=local_region(2,ipt)

   j1=local_region(3,ipt)
   j2=local_region(4,ipt)

   m1=local_region(5,ipt)    ! global k indexing scheme
   m2=local_region(6,ipt)

   if( m1 >= myid*locnz+1 .and. m2 <= (myid+1)*locnz ) then     ! entire region of influence on myid
    k1 = m1-myid*locnz  ! global to local indexing
    k2 = m2-myid*locnz  ! global to local indexing
    update=1
   elseif( m1 < myid*locnz+1 .and. m2 > myid*locnz+1 .and. m2 <= (myid+1)*locnz  ) then  ! region shared w/ S neighbor
    k1 = 1
    k2 = m2-myid*locnz  ! global to local indexing
    update=1
   elseif( m1 < (myid+1)*locnz .and. m1 >= myid*locnz+1 .and. m2 > (myid+1)*locnz ) then  ! region shared w/ N neighbor
    k1 = m1-myid*locnz  ! global to local indexing
    k2 = locnz
    update=1
   else
    update=0
   endif

   xloc = ibdry_locations(1,ipt)
   zloc = ibdry_locations(2,ipt)

   if( ibdry_normals(1,ipt) == -1 ) then   ! left vertical slice
    n1 = xloc - delta_1
    n2 = zloc
   elseif( ibdry_normals(1,ipt) == 1 ) then   ! right vertical slice
    n1 = xloc + delta_1
    n2 = zloc
   else
    n1 = xloc 
    n2 = zloc - delta_3/2.
   endif


! if processor myid owns part of ipts region of influence, update rhs on myid
  if( update==1 ) then 
   do i=i1,i2    !  loop over grid points in local region of influence
    do j=j1,j2
     do k=k1,k2

      if( i<1 .or. i>nx .or. j<1 .or. j>ny .or. k<1 .or. k>locnz ) then
       write(0,*) '**** RANGE ERROR:  ',i,j,k,ipt
       write(0,*) '**********  ',i1,i2,j1,j2,k1,k2
       STOP
      endif

      delta = exp( -(                &
            ( (Grid(0)%x(i,k)-xloc)/(delta_1))**2     &
           +( (Grid(0)%z(i,k)-zloc)/(delta_3))**2  ) )

      delta_n = exp( -(                &
            ( 10*(Grid(0)%x(i,k)-n1)/(delta_1))**2     &
           +( (Grid(0)%z(i,k)-n2)/(delta_3))**2  ) )

      rhs1(i,j,k,MM0) = rhs1(i,j,k,MM0) + delta*ibdry_rhs(1,ipt)
      rhs2(i,j,k,MM0) = rhs2(i,j,k,MM0) + delta*ibdry_rhs(2,ipt)
      rhs3(i,j,k,MM0) = rhs3(i,j,k,MM0) + delta*ibdry_rhs(3,ipt)
      rhs4(i,j,k,MM0) = rhs4(i,j,k,MM0) + delta_n*ibdry_rhs(4,ipt)

     enddo
    enddo
   enddo     ! end loop over grid points in local region of influence
  endif

  enddo   ! end loop over all immersed bdry pts

 call MPI_BARRIER(comm,ierr)    ! all processors leave routine together
end subroutine apply_ibdry_rhs
