subroutine gradient
#define DEBUG_LEVEL 1
!     Routine to calculate the "gradient" of the scalar field pressure
!     where the "gradient is defined as:
!
!     "grad" P = [ (x_zeta^2 + z_zeta^2)/(J^2) P_xi,
!                                   (1/y_eta) P_eta,
!                  (x_xi^2 +  z_xi^2 )/(J^2) P_zeta,
!     where   det_J = x_xi*z_zeta - x_zeta*z_xi

use mpi_parameters
use grid_info
use dependent_variables
use intermediate_variables
use boundary_information
use counters_flags_etc
implicit none
include '../input/problem_size.h'

integer                               :: flag
real                                  :: h,end_values(2)
real, dimension(:,:), allocatable     :: C_1,C_3
allocate( C_1(nx,locnz), C_3(nx,locnz) )

#if DEBUG_LEVEL >= 1
 if(myid==0) write(0,*) 'hello world from gradient'
#endif

! Take xi derivative, store result in grad_x
  flag=BC_flag(5,1)
  end_values=-9999999   !  dummy value, not used
  h=Grid(0)%dxi
  call compact_ddx(pressure(:,:,:,2),grad_x,flag,end_values,nx,ny,locnz)

! multiply p_xi by the appropriate factor (see comments above)
  C_1 = (Grid(0)%x_zeta**2 + Grid(0)%z_zeta**2)/(Grid(0)%det_J)**2
  do j=1,ny
   grad_x(:,j,:) = C_1*grad_x(:,j,:)
  enddo

! Take eta derivative, store result in grad_y
  flag=BC_flag(5,2)
  h=Grid(0)%deta
  call compact_ddy(pressure(:,:,:,2),grad_y,flag,end_values,nx,ny,locnz)
! divide by y_eta to get d/dy p
  do j=1,ny
   grad_y(:,j,:)=grad_y(:,j,:)/Grid(0)%y_eta(j)
  enddo

! Take zeta derivative, store result in p3
  flag=BC_flag(5,3)
  h=Grid(0)%dzeta
  call compact_ddz_mpi(pressure(:,:,:,2),grad_z,flag,end_values,nx,ny,locnz)

! multiply p_zeta by the appropriate factor (see comments above)
  C_3 = (Grid(0)%x_xi**2  + Grid(0)%z_xi**2)/(Grid(0)%det_J)**2
  do j=1,ny
   grad_z(:,j,:) = C_3*grad_z(:,j,:)
  enddo

 deallocate( C_1, C_3 )
end subroutine gradient
