subroutine initialize_cfd
#define DEBUG_FLAG 3

  use  grid_info
  use  mpi_parameters
  use  integration_weights
  use  dimensional_scales
  use  user_parameters
  use  pde_parameters
  use  dependent_variables
  use  intermediate_variables
  use  rhs_variables
  use  boundary_information
  use  counters_flags_etc

  integer               :: rank,flag
  real                  :: end_values(2),gamma,drdz
!!!  real                  :: maxu,minu,maxv,minv

  include '../input/problem_size.h'
  include 'mpif.h'
  include '../input/params.h'

!module integration_weights
  allocate (  W1(nx), W2(ny), W3(locnz) )

!module dependent_variables
  allocate ( U(nx,ny,locnz),v(nx,ny,locnz) )
  allocate ( W(nx,ny,locnz),pd(nx,ny,locnz) )
  allocate ( pressure(nx,ny,locnz,2) )
  allocate ( s1(nx,ny,locnz),s2(nx,ny,locnz) )
  allocate ( s1_bar(nx,locnz) )          ! not a dep. variable, but goes w/ s1
  allocate ( s1_bar_xi(nx,locnz) )      
  allocate ( s1_bar_zeta(nx,locnz) )
  allocate ( s2_bar(nx,locnz) )          ! not a dep. variable, but goes w/ s2
  allocate ( s2_bar_xi(nx,locnz) )      
  allocate ( s2_bar_zeta(nx,locnz) )
  allocate ( rho_bar(nx,locnz) )         ! not a dep. variable, but goes w/ pd
  

!module intermediate_variables
  allocate ( div_u(nx,ny,locnz,2),b(nx,ny,locnz) )
  allocate ( L_of_p(nx,ny,locnz) )
  allocate ( grad_x(nx,ny,locnz) )
  allocate ( grad_y(nx,ny,locnz) )
  allocate ( grad_z(nx,ny,locnz) )
  allocate ( u_star(nx,ny,locnz) )
  allocate ( v_star(nx,ny,locnz) )
  allocate ( w_star(nx,ny,locnz) )
  allocate ( u_star_old(ny,locnz,2) )
  allocate ( v_star_old(nx,locnz,2) )
  allocate ( w_star_old(nx,ny,2) )

!module rhs_variables
  allocate ( rhs1(nx,ny,locnz,AB_order) )
  allocate ( rhs2(nx,ny,locnz,AB_order) )
  allocate ( rhs3(nx,ny,locnz,AB_order) )
  allocate ( rhs4(nx,ny,locnz,AB_order) )
  allocate ( rhs5(nx,ny,locnz,AB_order) )

!module pde_parameters
  allocate ( K_m(nx,ny,locnz) )

! module boundary_information
!!!  allocate ( boundary_type1(ny,locnz),boundary_type2(ny,locnz) )
!!!  allocate ( boundary_type3(nx,locnz),boundary_type4(nx,locnz) )
!!!  allocate ( boundary_type5(nx,ny),boundary_type6(nx,ny) )
!!!  allocate ( BC_pressure1(ny,locnz),BC_pressure2(ny,locnz) )
!!!  allocate ( BC_pressure3(nx,locnz),BC_pressure4(nx,locnz) )
!!!  allocate ( BC_pressure5(nx,ny),BC_pressure6(nx,ny) )
!!!  allocate ( BC_vel1(ny,locnz),BC_vel2(ny,locnz) )
!!!  allocate ( BC_vel3(nx,locnz),BC_vel4(nx,locnz) )
!!!  allocate ( BC_vel5(nx,ny),BC_vel6(nx,ny) )
!!!  allocate ( BC_scalar1(ny,locnz,num_scalars),BC_scalar2(ny,locnz,num_scalars) )
!!!  allocate ( BC_scalar3(nx,locnz,num_scalars),BC_scalar4(nx,locnz,num_scalars) )
!!!  allocate ( BC_scalar5(nx,ny,num_scalars),BC_scalar6(nx,ny,num_scalars) )
  allocate ( BC_values1(ny,locnz,3+num_scalars),BC_values2(ny,locnz,3+num_scalars) )
  allocate ( BC_values3(nx,locnz,3+num_scalars),BC_values4(nx,locnz,3+num_scalars) )
  allocate ( BC_values5(nx,ny,3+num_scalars),BC_values6(nx,ny,3+num_scalars) )
  allocate ( phiY(nx,locnz,6,3,5) )
  allocate ( inflow(nx,locnz,3+num_scalars) )

!*******************************************************************
  write(0,*) 'IN INITIALIZE CFD, DEBUG_FLAG, myid = ', DEBUG_FLAG,myid
!!!#if DEBUG_FLAG >= 1
!!!  if(myid .eq. 0 ) then
!!!   write(0,*) '   '
   write(0,*) 'hello world from initialize_cfd, myid =', myid
!!!  endif
!!!#endif

! Check sizing parameters for obvious problems.
  if( mod(nz,numprocs) /= 0 ) then
   write(0,*) 'nz must be divisible by numprocs'
   STOP
  endif
  if( mod(nx-1,numprocs) /= 0 ) then
   write(0,*) 'nx-1 must be divisible by numprocs for xyfft logic'
   STOP
  endif
  if( locnz < 4 ) then
   write(0,*) 'currently, locnz >= 4'
   write(0,*) 'WILL TRY TO CARRY ON, BE CAREFUL!!!'
  endif
  if( AB_order /= 2 .and. AB_order /= 3 .and. AB_order /= 4) then
   write(0,*) 'only 2nd, 3rd or 4th order Adams Bashforth time stepping is allowed'
   STOP
  endif

 init_derived_type=0    ! MPI derived data types not yet defined
 subslice=-9999         ! MPI derived data types not yet defined, dummy value

! Each processor imports the main parameter file:
  pi=acos(-1.0)

! set some sizing parameters that depend on case xy periodic
  do i=0,Grid(0)%n_mg_levels-1

   if( boundary_type2(1,1)=='periodic' .and.  boundary_type2(1,1)=='periodic' ) then
    Grid(i)%n1_it=1
    Grid(i)%n2_it=1
    Grid(i)%n3_it=Grid(i)%n3     ! sizes for U&V  arrays in matvec
   else
    Grid(i)%n1_it=Grid(i)%n1
    Grid(i)%n2_it=Grid(i)%n2
    Grid(i)%n3_it=Grid(i)%locn3     ! sizes for U&V  arrays in matvec
   endif

! setup the PIM data for iterative solves at each grid level
   if( boundary_type2(1,1)=='periodic' .and. boundary_type4(1,1)=='periodic' ) then
    lda = Grid(i)%n3
    loclen = lda            ! iterations local to each processor
    pret = 0                ! preconditioning type 1=left, 0 =none
    stopt = 4               ! stop criteria (4 fastest, GMRES has its own)
    tol = 1.0E-19           ! stopping tolerance eps
    basis=maxits            ! "
   else  ! iterative method for 3d pressur field
    lda = Grid(i)%n1 * Grid(i)%n2 * Grid(i)%n3
    loclen = LDA/numprocs
    pret = 0                ! preconditioning type 1=left, 0 =none
    stopt = 4               ! stop criteria (4 fastest, GMRES has its own)
    tol = 1.0E-19           ! stopping tolerance eps
    basis=maxits            ! "
   endif
!  REMOVING BECAUSE WE WILL USE SOLVE_FOR_XYPER_PRESSURE INSTEAD
!   CALL PIMSSETPAR(Grid(i)%ipar,Grid(i)%spar,  &
!                   lda,lda,loclen,loclen,basis,numprocs,myid,pret,stopt,maxits,tol)

  enddo

#if DEBUG_FLAG >= 1
  if(myid .eq. 0 ) write(0,*) ' initialize_cfd: user parameter file ../input/params.h included '
#endif

   write(0,*) 'derive inside initialize_cfd, nu,kappa1,kappa2,myid =', nu,kappa1,kappa2,myid
! Derive dimensionless parameters given user input:
  Re = U_scale*L_scale/nu               ! bulk Reynolds number
  Pr1 = nu/kappa1                       ! "Prandtl" number for scalar 1
  Pr2 = nu/kappa2                       ! "Prandtl" number for scalar 2
  if( closure_flag .eq. 'LES' ) then
   C_s = 0.17   ! see Kaltenbach et el JFM 94, or
                ! Lesieur & Metais Ann Rev of Fluid Mech vol28
   Pr = 1.0
  endif
  Ri = (Nsquared * L_scale**2)/U_scale**2     ! bulk Richardson number
  Rot = f*L_scale/U_scale                     ! bulk inverse Rossby number

#if DEBUG_FLAG >= 1
  if(myid .eq. 0 ) write(0,*) ' Re, Pr1, Pr2, Ri, Rot : ', Re, Pr1, Pr2, Ri, Rot
#endif


 if( restart_flag == 'no' ) then

! Set the ambient stratification profiles s1_bar(x,z), s2_bar(x,z), d'less values returned
  call set_ambient_stratification

! set rho_bar(x,z) to -999.9  (will be overwritten in 1st call to equation_of_state)
  rho_bar(:,:)=-999.9

! Each processor initializes its portion of the dep. variable arrays
  if( ics_flag == 'homogeneous' ) then
   U=0.0
   v=0.0
   W=0.0
   s1=0.0
   if( num_scalars > 1 ) s2=0.0
   pressure=0.0
  else
!  read in ics data
  endif

! call equation_of_state to compute d'less pd and rho_bar
  call equation_of_state

! Compute xi and zeta derivatives of s1_bar and s2_bar
  flag=1;end_values=-1e32

#if DEBUG_FLAG >= 1
  if(myid .eq. 0 ) write(0,*) ' initialize_cfd: about to differentiate s1_bar & s2_bar '
#endif

  call compact_ddx(s1_bar,s1_bar_xi,flag,end_values,nx,1,locnz)
  if(num_scalars>1) call compact_ddx(s2_bar,s2_bar_xi,flag,end_values,nx,1,locnz)
  
#if DEBUG_FLAG >= 1
  if(myid .eq. 0 ) write(0,*) ' initialize_cfd: return from ddx '
#endif

  call compact_ddz_mpi(s1_bar,s1_bar_zeta,flag,end_values,nx,1,locnz)
  if(num_scalars>1) call compact_ddz_mpi(s2_bar,s2_bar_zeta,flag,end_values,nx,1,locnz)
  
#if DEBUG_FLAG >= 1
  if(myid .eq. 0 ) write(0,*) ' initialize_cfd: return from ddz '
#endif

  init_derived_type=0    ! MPI derived data types not yet defined
  subslice=-9999         ! MPI derived data types not yet defined, dummy value

#if DEBUG_FLAG >= 1
  if(myid .eq. 0 ) write(0,*) ' initialize_cfd: differentiated s1/s2_bar '
#endif


 endif    ! end restart_flag == 'no' block

! Each processor fills weight arrays for spatial integration dxi,deta&dzeta:
  call set_integration_weights(Grid(0)%dxi,Grid(0)%deta,Grid(0)%dzeta)

! Set integer BC flags
  BC_flag(:,:)=1   ! default=no end conditions specified
  if( boundary_type1(1,1) == 'periodic' ) BC_flag(:,1)=5
  if( boundary_type3(1,1) == 'periodic' ) BC_flag(:,2)=5
  if( boundary_type5(1,1) == 'periodic' ) BC_flag(:,3)=5

! Initial values of counters etc
 if( restart_flag == 'no' ) then
  istart=0
  iend=istart+nsteps
  new=1;old=2
  MM0=1;MM1=2;MM2=3;MM3=4
  step_flag='EULER'
  write3D_counter=1

! Intermediate variables initial data values
  b=0.0
  div_u=0.0
  u_star=0.0
  v_star=0.0
  w_star=0.0
  u_star_old=0.0
  v_star_old=0.0
  w_star_old=0.0
  grad_x=0.0
  grad_y=0.0
  grad_z=0.0
  phiY=0.0
 elseif( restart_flag == 'yes' ) then
  call restart
!!!      maxu = MAXVAL(u(:,:,:))
!!!      minu = MINVAL(u(:,:,:))
!!!      maxv = MAXVAL(v(:,:,:))
!!!      minv = MINVAL(v(:,:,:))
!!!      write(0,*) 'MAXMIN',myid,maxu,minu,maxv,minv
 endif
 
! Dimensionless eddy viscosity 
  K_m(:,:,:)=1./Re

#if DEBUG_FLAG >= 1
  if(myid .eq. 0 ) write(0,*) ' initialize_cfd:  calling initialize immersed bdry'
#endif

  call initialize_immersed_bdry
end subroutine initialize_cfd
