subroutine initialize_immersed_bdry
#define DEBUG_LEVEL 2

  use grid_info
  use mpi_parameters
  use immersed_boundary
  use counters_flags_etc
  use boundary_information
  use dependent_variables
  use grid_info
  use dimensional_scales
  implicit none

  include '../input/problem_size.h'
  include 'mpif.h'

  integer		  :: npts,j1,j2,jplane,N,root
  real                    :: z0,z1,est
  character(len=3)                     ::  cnum
  character(len=80)                    ::  filename


if( immersed_bdry_flag == 'no' ) return
#if DEBUG_LEVEL >= 1
  if(myid==0) then
   write(0,*) ' '
   write(0,*) ' '
   write(0,*) 'hello world from initialize_immersed_bdry'
  endif
#endif

!  set width parameters of discrete 3d delta function
 delta_1 = 2.*( Grid(0)%x_xi(1,1)*Grid(0)%dxi )
 delta_2 = 1.*( Grid(0)%y_eta(1)*Grid(0)%deta )
 delta_3 = 2.*( Grid(0)%z_zeta(1,1)*Grid(0)%dzeta )     ! think if dz not constant

!set the gain coefficients (set positive values here)
! xalpha controls the oscillation frequency of the boundary forcing
! choose value so that we have N steps per period , i.e. choose N given
! a physically chosen value of dt
 pi=4.*atan(1.)
 N=50.
                       !xalpha=( 4.*pi*pi/(N*dt) )**2    ! following GH&S I get ( 4.*pi*pi/(N*dt) )**2
 xbeta=1./(N*dt)
 xalpha=xbeta**2

! est = (xbeta + sqrt( xbeta**2 + 2.*xalpha*1 ))/xalpha
 if( myid == 0 ) then
  write(0,*) '             parameters of discrete delta function:  ',delta_1,delta_2,delta_3
  write(0,*) '             gain parameters alpha & beta : ',xalpha,xbeta
  write(0,*) '             dimensionless time step dt   : ',dt
! write(0,*) '             estimated time step limit,k=1: ',est
! if( dt > est ) then
!  write(0,*) ' alpha,beta too big for given time step  '
!  STOP
! endif
 endif

 if( myid == 0 ) then           
   open(1,file='../input/immersed_bdry_locations')
   read(1,*) n_j
   write(0,*) '     reading ',n_j, '  immersed boundary points per j plane'
   npts=n_j*ny
 endif
 root=0
 call MPI_BCAST(n_j,1,MPI_INTEGER,root,comm,ierr)
 call MPI_BCAST(npts,1,MPI_INTEGER,root,comm,ierr)

!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
!allocate arrays declared in module immersed_boundary
  allocate( z_limits(2,0:numprocs-1) ) ! lower&upper z limits for each processor
  allocate( ibdry_locations(2,npts), ibdry_normals(2,npts) )
  allocate( ibdry_owner(npts) )
  allocate( ibdry_data(4,npts,3) )        ! 4 variables, 3 time levels
  allocate( ibdry_integrals(4,npts,3) )   ! 4 variables, 3 time levels
  allocate( ibdry_rhs(4,npts), local_region(6,npts) )
  allocate( mask1(nx,locnz), mask2(nx,locnz), mask3(nx,locnz) )
! mask values set in initialize_user.F90
!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

 if( myid == 0 ) then           
! read in the immersed boundary data for ONE j plane
   do i=1,n_j
    read(1,*) ibdry_locations(1,i),ibdry_locations(2,i),   &
              ibdry_normals(1,i),ibdry_normals(2,i)
   enddo
   close(1)   ! close immersed boundary data file
   ibdry_locations=ibdry_locations/L_scale

!  set the same values for all the other j planes
   do j=2,ny
    j1=(j-1)*n_j+1
    j2=j1+n_j-1
    ibdry_locations(:,j1:j2)=ibdry_locations(:,1:n_j)
    ibdry_normals(:,j1:j2)=ibdry_normals(:,1:n_j)
   enddo
#if DEBUG_LEVEL >= 1
   write(0,*)  '     immersed bdry data read in by processor 0'
#endif
 endif   ! end logic for myid=0 only

! bcast values in ibdry_locations, ibdry_normals to ids>0
 root=0
 call MPI_BCAST(ibdry_locations,2*npts,MPI_REAL,root,comm,ierr)
 call MPI_BCAST(ibdry_normals  ,2*npts,MPI_REAL,root,comm,ierr)

!ALL do the following:
 ibdry_owner(:)=-999
 local_region(:,:)=-999.
 z0 = Grid(0)%z(1,1) - (0.5)*Grid(0)%z_zeta(1,1)*Grid(0)%dzeta
 z1 = Grid(0)%z(1,locnz) + (0.5)*Grid(0)%z_zeta(1,locnz)*Grid(0)%dzeta
 z_limits(1,myid)=z0
 z_limits(2,myid)=z1

! make z_limits known globally
  do i=0,numprocs-1
   root=i  ! root process sends, all others receive
   call MPI_BCAST( z_limits(1,i),2,MPI_REAL,root,comm,ierr)
  enddo

do i=1,npts

  jplane=aint( float(i-1)/float(n_j) ) + 1

  do k=0,numprocs-1
   if( ibdry_locations(2,i) >= z_limits(1,k) .and.    &
       ibdry_locations(2,i) <  z_limits(2,k)  ) then
    ibdry_owner(i)=k
    goto 100
   endif
  enddo  
100  continue

! owners determine local neighborhood of each immersed boundary point
 if( ibdry_owner(i)==myid ) then
   do k=1,nx
    if( Grid(0)%x(k,1) > ibdry_locations(1,i) ) goto 101
   enddo
101 continue
   local_region(1,i) = max(1,k-3)             ! 3 left of 1st point past or 1
   local_region(2,i) = min(nx,k+2)            ! 2 right of 1st point past or nx

   local_region(3,i) = jplane   !max(1,jplane-3)             
   local_region(4,i) = jplane   !min(ny,jplane+2)           

   do k=1,locnz
    if( Grid(0)%z(1,k) > ibdry_locations(2,i) ) goto 102
   enddo
   k=locnz+1
102 continue
   local_region(5,i) = myid*locnz + k -3             ! 3 below 1st point above, global indexing
   local_region(6,i) = myid*locnz + k +2             ! 2 above 1st point above, global indexing

   if( local_region(5,i) < 1  ) local_region(5,i)=1    ! global k=1  is lowest  point to be influenced
   if( local_region(6,i) > nz ) local_region(6,i)=nz   ! global k=nz is highest point to be influenced

  endif   ! end logic for owner
   
! Share local region data, all processors execute the broadcasts
  root = ibdry_owner(i)
  call MPI_BCAST(local_region(1,i),6,MPI_INTEGER,root,comm,ierr) 

enddo  ! end loop over all immersed points (in a j plane)

 do i=1,npts
  if( ibdry_owner(i) == -999 ) then
   write(0,*) ' immersed boundary point ',n_j,' not claimed. z= ',ibdry_locations(2,i)
   STOP
  endif
  if( local_region(1,i) == -999. .or. local_region(2,i) == -999. ) then
   write(0,*) 'problem in defining local x region of bdry pt ',n_j
   STOP
  endif
  if( local_region(5,i) < 0 .or. local_region(6,i) > nz ) then
   write(0,*) 'problem in defining local z region of bdry pt ',n_j
   write(0,*) ' z_0:  ',local_region(5,i)
   write(0,*) ' z_1:  ',local_region(6,i)
   STOP
  endif
 enddo

 !Create a filename for writing out ibdry information
  write(unit=cnum,fmt=200) myid
200 format(I3.3)
  filename='../output/ibdry_data_'//cnum
  open(1,file=filename)
  do i=1,n_j
   write(1,201) i, ibdry_owner(i),ibdry_locations(1,i),ibdry_locations(2,i),   &
                ibdry_normals(1,i),ibdry_normals(2,i),                         &
                local_region(1,i), local_region(2,i),                          &
                local_region(5,i), local_region(6,i)                           
  enddo
201 format(1x,2i3,4f10.6,4i6)
  close(1)

 write(0,*) ' FINISHED INITIALIZING IMMERSED BOUNDARY DATA ',myid
 call MPI_BARRIER(comm,ierr)
end subroutine initialize_immersed_bdry
