subroutine project
#define DEBUG_LEVEL 1
!      routine to project the intermediate velocity field u*
!      onto its non-divergent subspace. The projection is
!
!                u_(n+1) = u_star - dt*"grad"[p]
!
!      inputs:
!             u_star etc       intermediate velocity fields, divergent
!             grad_{x,y,z}     three components of the pressure "gradient"
!             dt               time increment
!
!      outputs:
!             u,v,w        the divergence free velocity 
!

use grid_info
use mpi_parameters
use dependent_variables
use intermediate_variables
use boundary_information
use immersed_boundary
use integration_weights
use user_parameters
implicit none

include '../input/problem_size.h'
include 'mpif.h'


#if DEBUG_LEVEL >= 1
 if(myid==0) write(0,*) 'hello world from project '
#endif

 u(:,:,:) = u_star(:,:,:) &
                -dt*grad_x(:,:,:)
 v(:,:,:) = v_star(:,:,:) &
                -dt*grad_y(:,:,:)
 w(:,:,:) = w_star(:,:,:) &
                -dt*grad_z(:,:,:)

! Overwrite the old values of u_star, v_star and w_star 
! at the edges with the current values.
! These are sometimes needed for the boundary conditions for pressure P.

! All processors do left/right and front/back faces
! LEFT AND RIGHT (xi=0,L_xi) FACES
 u_star_old(1:ny,1:locnz,1) = u_star(1,1:ny,1:locnz)
 u_star_old(1:ny,1:locnz,2) = u_star(nx,1:ny,1:locnz)

!FRONT AND BACK (eta=0,L_eta) FACES
 v_star_old(1:nx,1:locnz,1) = v_star(1:nx,1,1:locnz)
 v_star_old(1:nx,1:locnz,2) = v_star(1:nx,ny,1:locnz)

! Only processors 0 and numprocs-1 need to handle bottom
! and top faces respectively. (zeta=0,L_zeta) 
 if( myid .eq. 0 ) then
  w_star_old(:,:,1) = w_star(:,:,1)
 elseif( myid .eq. numprocs-1 ) then
  !w_star_old(:,:,2) = w_star(:,:,nz) 
  w_star_old(1:nx,1:ny,2) = w_star(1:nx,1:ny,locnz)  ! looks like it should be locnz to me!
 endif

end subroutine project
