subroutine read_grid_data(coords_file,rblock)

#define DEBUG_FLAG 1

 use grid_info
 use mpi_parameters
 implicit none

 include 'mpif.h'
 include '../input/problem_size.h'
 character(len=*)       :: coords_file
 integer                :: rblock,iblock,m1,m2,m3,i,j,k,ierr
 real                   :: tmp, L_xi, L_eta, L_zeta,xx

#if DEBUG_FLAG >= 1
 if(myid .eq. 0 ) write(0,*) '       hello world from read_grid_data',coords_file,rblock
#endif

 if( myid .ne. rblock ) return  ! one active processor per read block
  open(1,file=coords_file) 
      
  read(1,*) m1,m2,m3,L_xi,L_eta,L_zeta ! dimensionless values
  if( m1 .ne. nx .or. m2 .ne. ny .or. m3 .ne. locnz*numprocs ) then
   if( myid .eq. 0 ) then
    write(0,*) ' problem size read in from grid_metrics does not'
    write(0,*) ' match problem size specification in problem_size.h'
    write(0,*) '             L_xi  ',L_xi
    write(0,*) '             L_eta  ',L_eta
    write(0,*) '             L_zeta  ',L_zeta
    stop  
   endif
  endif

! Each processor stores only its portion of the grid & metrics data.
! Read strategy:
!  Each processor opens file and reads from the beginning in
!  locnz level blocks, keeping only that data from block (myid+1)
!  Wasteful, but only executed at startup and permits backward
!  compatable grid_coords files.

  do i=1,nx
   do iblock=0,numprocs-1
    do k=1,locnz              
     if(iblock .eq. myid) then
      read(1,*) Grid(0)%x(i,k),Grid(0)%z(i,k),Grid(0)%x_xi(i,k),Grid(0)%x_zeta(i,k),       &
                Grid(0)%z_xi(i,k),Grid(0)%z_zeta(i,k),                     &
                GM_uu(i,k,1),GM_uw(i,k,1),GM_ww(i,k,1),    &
                GM_uu(i,k,2),GM_uw(i,k,2),GM_ww(i,k,2),    &
                Grid(0)%Cxx(i,k),Grid(0)%Czz(i,k),tmp,tmp
     else  ! read values but don't bother to store them
      read(1,*) tmp,tmp,tmp,tmp,tmp,tmp,tmp,tmp,           &
                tmp,tmp,tmp,tmp,tmp,tmp,tmp,tmp
     endif
    enddo
   enddo
  enddo 

! Each processor reads and stores the y coordinate data.
 do j=1,ny
  read(1,*) Grid(0)%y(j),Grid(0)%y_eta(j),tmp,Grid(0)%Cyy(j)
 enddo
 close(1)

! Zero out (z_xi,x_zeta) on (top,side) boundaries.
! BOTTOM not oriented with x/z=const,--> these terms not zero.
! TOP
  if( myid .eq. numprocs-1 ) then
   Grid(0)%z_xi(1:nx,locnz)=0.0
   Grid(0)%x_zeta(1:nx,locnz)=0.0
  endif

! SIDES
! LEFT
  Grid(0)%x_zeta(1,1:locnz)=0.0
  Grid(0)%z_xi(1,1:locnz)=0.0
! RIGHT
  Grid(0)%x_zeta(nx,1:locnz)=0.0
  Grid(0)%z_xi(nx,1:locnz)=0.0

! Calculate J determinant and inverse of J 
! over the local data space.
  Grid(0)%det_J = Grid(0)%x_xi*Grid(0)%z_zeta - Grid(0)%x_zeta*Grid(0)%z_xi
  Grid(0)%J_matrix(:,:,1,1) =  Grid(0)%z_zeta/Grid(0)%det_J
  Grid(0)%J_matrix(:,:,1,2) = -Grid(0)%z_xi/Grid(0)%det_J
  Grid(0)%J_matrix(:,:,2,1) = -Grid(0)%x_zeta/Grid(0)%det_J 
  Grid(0)%J_matrix(:,:,2,2) =  Grid(0)%x_xi/Grid(0)%det_J

#if DEBUG_FLAG >= 1
  if( myid .eq. numprocs-1 ) then
   write(0,*) '            dless L_x :',Grid(0)%x(nx,1)-Grid(0)%x(1,1)
   write(0,*) '            dless L_y :',Grid(0)%y(ny)-Grid(0)%y(1)
   write(0,*) '            dless L_z :',Grid(0)%z(1,locnz)
   write(0,*) '            det_J ',Grid(0)%det_J(nx/2,locnz/2)
  endif
#endif

! Modify the coefficients Cxx, Cyy and Czz as required for
! the slightly different approach to the pressure solution when 
! using the self-adjoint form. Making the modifications here allows
! the grid generation routines to be backward compatible with
! earlier versions of the code that used MUDPACK for the pressure solves.

  Grid(0)%Cxx = Grid(0)%det_J*Grid(0)%Cxx
  Grid(0)%Czz = Grid(0)%det_J*Grid(0)%Czz
  Grid(0)%Cyy=1./Grid(0)%y_eta

end subroutine read_grid_data
