subroutine set_integration_weights(dxi,deta,dzeta)

#define DEBUG_LEVEL 1
 
 use mpi_parameters
 use integration_weights
 implicit none
 include '../input/problem_size.h'
 real, allocatable            :: tmp(:)
 real                         :: dxi, deta, dzeta
 integer                      :: I1,J1,K1,I2,J2,K2
  
 data I1,J1,K1,I2,J2,K2/1,1,1,nx,ny,nz/

 allocate ( tmp(nz) )
 tmp(:) = 0.

!     Routine to set the 1d integration weights for computing
!     integrals wrt to x,y, and z assuming equally spaced
!     meshes dx,dy and dz
!
!     integration in the x direction takes place between and inclusive of 
!     gridpoints I1 to I2, integration weights outside this range are set
!     to zero. This allows the "fringe" regions to be easily excluded from
!     the flow analysis done in routine energetics.f

!     scheme is alternative extended Simpson's rule error is order (1/N^4)

#if DEBUG_LEVEL == 1
 if(myid==0) write(0,*) ' hello world from set_integration_weights '
#endif

! xi direction
 if( I1 > 1 ) W1(1:I1-1)=0.0
 W1(I1)=dxi*(17./48.)
 W1(I1+1)=dxi*(59./48.)
 W1(I1+2)=dxi*(43./48.)
 W1(I1+3)=dxi*(49./48.)
 W1(I1+4:I2-4)=dxi
 W1(I2-3)=dxi*(49./48.)
 W1(I2-2)=dxi*(43./48.)
 W1(I2-1)=dxi*(59./48.)
 W1(I2  )=dxi*(17./48.)
 if(I2 < nx) W1(I2+1:nx)=0.0
      
! eta direction
 if(J1 > 1 ) W2(1:J1-1)=0.0
 W2(J1)=deta*(17./48.)
 W2(J1+1)=deta*(59./48.)
 W2(J1+2)=deta*(43./48.)
 W2(J1+3)=deta*(49./48.)
 W2(J1+4:J2-4)=deta
 W2(J2-3)=deta*(49./48.)
 W2(J2-2)=deta*(43./48.)
 W2(J2-1)=deta*(59./48.)
 W2(J2  )=deta*(17./48.)
 if(J2 < ny ) W2(J2+1:ny)=0.0
      
! zeta direction   (calculate global array)
 if( K1 > 1 ) tmp(1:K1-1)=0.0
 tmp(K1)=dzeta*(17./48.)
 tmp(K1+1)=dzeta*(59./48.)
 tmp(K1+2)=dzeta*(43./48.)
 tmp(K1+3)=dzeta*(49./48.)
 tmp(K1+4:K2-4)=dzeta
 tmp(K2-3)=dzeta*(49./48.)
 tmp(K2-2)=dzeta*(43./48.)
 tmp(K2-1)=dzeta*(59./48.)
 tmp(K2  )=dzeta*(17./48.)
 if( K2 < nz )tmp(K2+1:nz)=0.0
      
      
! extract required portion for processor myid, 
! store in locnz array W3

 W3(1:locnz)=tmp( myid*locnz+1 : (myid+1)*locnz )
 deallocate( tmp)

end subroutine set_integration_weights
