!!!!  user defined modules used in cfdXZ code  !!!!

module user_parameters
!******* ice stress problem ******************************************************
!   x_0,x_1,x_2                      positions of ice/water interface [m]
!   surface_V_of_x               surface BC for v, applied in [0,x_1] and [x_2,Lx]
!   z_N                          center position for stratified layer [m]
!   z_N_scale                    vertical scale of stratified layer   [m]
!   HV_RATIO                     nu_horiz/nu where nu is taken as nu_vertical [1]
!                                (invoked only when HV_METHOD=1)
!   Smax,Smin                    max/min Salinity values for ambient profile [ppt]
!   Tmax,Tmin                    max/min Temp values used for ambient profile [deg C]
!   Km_of_z                      eddy viscosity for momentum [m2/s]
!   Kt_of_z                      eddy diffusivity for temperature [m2/s]
!   Kh                           horizontal eddy diffusivity for both momentum
!                                and temperature when HV_METHOD=2 [m2/s]
!******* ice stress problem ******************************************************
 real                         :: x_0,x_1,x_2,V_0
 real, allocatable            :: surface_V_of_x(:)
 real, allocatable            :: Km_of_z(:),Kt_of_z(:)
 real                         :: z_N, z_N_scale
 real                         :: Smax,Smin,Tmax,Tmin
 real                         :: HV_RATIO
 integer                      :: HV_METHOD
 character(80)                :: LAPACK_FLAG
end module user_parameters
!*******


module immersed_boundary   
 integer                    :: n_j,n_left,n_right
 integer,allocatable        :: ibdry_owner(:),local_region(:,:)
 real,allocatable	    :: ibdry_locations(:,:), ibdry_normals(:,:)
 real,allocatable           :: ibdry_data(:,:,:), ibdry_integrals(:,:,:)
 real,allocatable           :: ibdry_rhs(:,:), z_limits(:,:)
 real,allocatable	    :: mask1(:,:), mask2(:,:),mask3(:,:)
 real		            :: delta_1, delta_2, delta_3
 real		            :: xalpha, xbeta,x_left,x_right,z_left,z_right
end module immersed_boundary
