#include "config.h"

module parametres

    implicit none

    ! *********************************
    !   Paramtres du fichier data.in
    ! *********************************

    ! Les valeurs DIMX, DIMY, DIMZ, PPADX, PPADY, PPADZ doivent etre remplacees par des entiers. Cette operation est effectuee
    ! par le preprocesseur en fonction des valeurs indiquees dans le fichier "config.h"

    ! dimensions physiques du probleme. Nombre de points de colocation suivant chaque direction.
    integer, parameter :: nxmax=DIMX, nymax=DIMY, nzmax=DIMZ

    ! nombre d'onde maximal calcule suivant x (la division par 2 est entiere). Les nombres d'onde suivant x sont
    ! compris entre 0 et kxmax (kxmax+1 valeurs).
    integer, parameter :: kxmax=nxmax/2

    ! Padding (augmentation artificielle des dimensions des tableaux) suivant les 3 directions.
    ! Permet d'optimiser les acces memoire.
    integer, parameter :: padx=PPADX, pady=PPADY, padz=PPADZ

    ! dimensions de stockage des donnees. On tient compte de l'espace necessaire par une fft reelle-complexe et du padding.
    integer, parameter :: dimx_tab=2*(kxmax+1)+padx, dimy_tab=nymax+pady, dimz_tab=nzmax+padz


    ! Taille de la boite :
    double precision, save :: lx,ly,lz

    ! pas de temps et instant initial
    double precision, save :: dt, begin

    ! nombre d'itrations
    integer, save :: itmax

    ! Troncature (dsaliasing)
    logical, save :: truncate
    integer, save :: type_trunc
    double precision, save :: rtrunc_x, rtrunc_y, rtrunc_z

    ! viscosit
    double precision, save :: nu

    ! stratification et rotation
    logical, save :: stratification
    double precision, save :: xns,xnsqr,omega2,schm

    ! type de run : en perturbation, linaire
    logical, save :: perturbe, lin

    ! reprise de run
    logical, save :: reprise_run

    ! lecture de l'tat de base 2D en "unformatted" ou appel subroutine
    logical, save :: base2D_sub, base2D_read

     ! lecture de la vitesse initiale 2D en "unformatted" ou appel subroutine
    logical, save :: velo2D_sub, velo2D_read
     
    ! lecture du profil de densite initiale 1D en "unformatted"
    logical, save :: dRho_read
    double precision, save :: Rhob

    ! Bruit blanc et amplitude du bruit blanc
    logical, save :: lnoise
    double precision, save :: amplitude_noise

    !introduction d'un forcage, appel de la subroutine forcage
    logical, save :: forcage_sub   
    double precision, save :: A_ics, freq
    integer, save :: it0  

    ! nombre et instants o sont appeles les differentes fonctions de sorties
    integer, save :: periode_out_physique1, periode_out_physique2, periode_out_spectral1


    ! PARAMETRES ADDITIONNELS !

    ! Tourbillons gaussiens
    integer, save :: nb_tourbillons
    double precision, dimension(100), save :: xt, yt, gamma, at


    ! *********************************
    !   Variables globales de travail
    ! *********************************

    ! coordonnes des points de colocation
    double precision, dimension(0:nxmax-1), save :: xx
    double precision, dimension(0:nymax-1), save :: yy
    double precision, dimension(0:nzmax-1), save :: zz

    ! Etat de base
    double precision, dimension(0:nxmax-1,0:nymax-1), save :: vbx,vby,vbz,wbx,wby,wbz

    ! Vitesses, densit, termes non linaire pour l'quation de la vitesse et de la densit
    double precision, dimension(0:dimx_tab-1,0:dimy_tab-1,0:dimz_tab-1), save :: vox, voy, voz, rho, dRho_dz, wdRho
    double precision, dimension(0:dimx_tab-1,0:dimy_tab-1,0:dimz_tab-1), save :: vox2, voy2, voz2, rho2
    double precision, dimension(0:dimx_tab-1,0:dimy_tab-1,0:dimz_tab-1), save :: fx, fy, fz, frho


    ! *********************************************
    !   Working array necessary for the 3D FFT
    ! *********************************************

#if (defined(JMFFT) && DIMZ!=1)
    double precision, dimension(100 + 2*(nxmax+nymax+nzmax) + 512*max(nxmax,nymax,nzmax)) :: fft_work
#elif (defined(JMFFT) && DIMZ==1)
    double precision, dimension(100 + 2*(nxmax+nymax) + 512*max(nxmax,nymax)) :: fft_work
#elif (defined(MATHKEISAN) && DIMZ!=1)
    double precision, dimension(2*(nxmax+nymax+nzmax)+195 + 4*256*max(nxmax,nymax,nzmax)) :: fft_work
#elif (defined(MATHKEISAN) && DIMZ==1)
    double precision, dimension(2*(nxmax+nymax)+130 + 6*nxmax*nymax) :: fft_work
#elif (defined(ESSL) && DIMZ==1)
    double precision, dimension(100000), save :: fft_work
#else
    double precision, dimension(1), save :: fft_work
#endif


    ! tableau de troncature (dsaliasing)
    logical, dimension(0:dimx_tab-1,0:dimy_tab-1,0:dimz_tab-1), save :: trunctab

    ! vecteurs d'onde, vecteurs d'onde au carr, termes pre-exponentiels
    double precision, dimension(0:kxmax), save :: kx,k2x
    double precision, dimension(0:nymax-1), save :: ky,k2y
    double precision, dimension(0:nzmax-1), save :: kz,k2z

    double precision, dimension(0:2*kxmax+1), save :: cax, ca2x

    double precision, dimension(0:2*kxmax+1), save :: eex, exs
    double precision, dimension(0:nymax-1), save :: eey, eeys
    double precision, dimension(0:nzmax-1), save :: eez, eezs

    ! instant courant
    integer, save :: it = 0

    ! divers
    double precision, save :: pi
    double precision, parameter :: epsilo = 1.D-100

    ! utilisee pour determiner le taux de croissance entre deux appels successifs a out_physique1
    double precision, save :: vquadanc = 0

    ! compteurs utilises pour calculer le temps de calcul passe dans chaque sous-routine de la boucle principale
    double precision, save :: tps_adams_step1, tps_adams_step2, tps_rotv, tps_fwd_fft, tps_bck_fft, tps_out, &
                              tps_vecpro2, tps_urho, tps_div_speciale, tps_projection, tps_trunc0, tps_copie, tps_total, &
                              tps_temp, tps_detect_nan, tps_out_backup

end module
