#!/usr/bin/ksh
#######################################################
#####          LANCEMENT DU RUN                 ####### 
#######################################################
#_____________________________________________________
#
#       DEFINITION DES VARIABLES
#_____________________________________________________

MODE=$1
PROJECT=$2
RUN=$3


if [ $# -ne 3 ]; then
  echo ' '
  echo " *** Mauvais arguments d'entree ! *** "
  echo '   exemple: ./lanc.ksh mode version numero_run'
  echo ' '
  exit 1
fi

echo ' '
echo ' '
case $1 in
  0)
   echo '========================'
   echo '==     MODE BATCH     =='
   echo '========================'
  ;;
  1)
   echo '========================'
   echo '==   MODE INTERACTIF  =='
   echo '========================'
  ;;
  *)
   echo '========================'
   echo '==     MODE TEST      =='
   echo '========================'
  ;;
esac
 echo ' '
 echo ' '

#_____________________________________________________
#
# PARAMETRES 
#_____________________________________________________

DATE=`date '+%Y%m%d'_'%H%M%S'`
echo `date`

HOME_DIR=$PWD
HOME_PROG=SpectralModelF90
SUB_PROJECT=sub_$HOME_PROG
WORK_PROG=${RUN}_${HOST}_${HOME_PROG}_${PROJECT}_${DATE}

if [ ! -d $WORK_DIR  ]; then
   WORK_DIR=$HOME_DIR
   echo ' '
   echo ' pas de WORK_DIR definit, execution sur:'
   echo '    --> '$WORK_DIR
   echo ' '
fi

echo ' '
echo 'Definitions des variables:'
echo '   HOME_DIR    = '$HOME_DIR
echo '   HOME_PROG   = '$HOME_PROG
echo '   PROJECT     = '$PROJECT
echo '   SUB_PROJECT = '$SUB_PROJECT
echo '   WORK_DIR    = '$WORK_DIR
echo '   WORK_PROG   = '$WORK_PROG


#_____________________________________________________
#
# CREATION DU REPERTOIRE DE COMPILATION SUR LE WORKDIR
#_____________________________________________________

echo ' '
echo 'Creation du repertoire de compilation et copie des fichiers'
if [ ! -d $HOME_DIR/PROJECTS/$PROJECT ]; then
   echo ' '
   echo '   WARNING: le projet '$PROJECT" n'existe pas"
   exit 1
fi

cd $WORK_DIR
if  [ -d $WORK_PROG ]; then 
  rm -rf $WORK_PROG
fi
mkdir $WORK_PROG 

cd $WORK_DIR/$WORK_PROG
cp -rf $HOME_DIR/BIN .
cp -rf $HOME_DIR/SRC .
cp -rf $HOME_DIR/Makefile .
cp -rf $HOME_DIR/Make.inc .
cp -rf $HOME_DIR/User.inc .
cp -rf $HOME_DIR/PROJECTS/NEW_SRC/*.f90 SRC/.
cp -rf $HOME_DIR/PROJECTS/$PROJECT/* . 
cp *.f90 SRC/.
mkdir output
cd output
mkdir 1D
mkdir 2D
mkdir 3D
mkdir TS
cd ..


#_____________________________________________________
#
#       COMPILATION DU PROG SUR LE WORKDIR
#_____________________________________________________

echo ' '
echo 'Compilation du code pour le projet: '$PROJECT
# Compilation
cd  $WORK_DIR/$WORK_PROG 
make clean
make flow.x
if [ -f flow.x ]; then
   echo '   Compilation successful ! Here we go'
else
   echo '   WARNING: Compilation failed !'
   cd $WORK_DIR
   #rm -rf $WORK_PROG
   exit 1
fi


echo ' '
case $1 in
	0)
	#_____________________________________________________
	#
	#        OFFLINE NOHUP
	#_____________________________________________________
	echo ' Soumission du job... offline.'
	cp $HOME_DIR/$SUB_PROJECT $WORK_DIR/$WORK_PROG/.
	cd $WORK_DIR/$WORK_PROG
        nohup ./$SUB_PROJECT $WORK_DIR/$WORK_PROG $HOME/$WORK_PROG > $WORK_DIR/$WORK_PROG/nohup.out &
	;;
	1)
        #_____________________________________________________
        #
        #        ONLINE 
        #_____________________________________________________
        echo ' Mode interactif: execution sur le workdir ...'
        cd $WORK_DIR/$WORK_PROG
	NPROCS=`grep 'numprocs' $WORK_DIR/$WORK_PROG/problem_params | tr -d ' ' | grep 'numprocs=' | cut -c 16-17`
	export MP_PROCS=$NPROCS
	echo '   execution sur '$MP_PROCS' procs.'
        ./flow.x 
        echo ' End : successful run !'
	;;
	*)
	#_____________________________________________________
	#
	#        TEST COMPILATION
	#_____________________________________________________
	echo ' End : successful test !'
	cd $WORK_DIR
	rm -rf $WORK_PROG
	;;
esac
echo ' '

